@@ -1,6 +1,6 @@
 //------------------------------------------------------------------------
 //
-//  AJ-BSP  Copyright (C) 2000-2018  Andrew Apted, et al
+//  AJ-BSP  Copyright (C) 2000-2019  Andrew Apted, et al
 //          Copyright (C) 1994-1998  Colin Reed
 //          Copyright (C) 1997-1998  Lee Killough
 //
@@ -64,7 +64,6 @@ public:
 	// from here on, various bits of internal state
 	int total_failed_maps;
 	int total_warnings;
-	int total_minor_issues;
 
 public:
 	nodebuildinfo_t() :
@@ -85,8 +84,7 @@ public:
 		cancelled(false),
 
 		total_failed_maps(0),
-		total_warnings(0),
-		total_minor_issues(0)
+		total_warnings(0)
 	{ }
 
 	~nodebuildinfo_t()
@@ -135,6 +133,15 @@ extern nodebuildinfo_t * cur_info;
 typedef double angle_g;  // degrees, 0 is E, 90 is N
 
 
+// prefer not to split
+#define MLF_IS_PRECIOUS  0x40000000
+
+// this is flag set when a linedef directly overlaps an earlier
+// one (a rarely-used trick to create higher mid-masked textures).
+// No segs should be created for these overlapping linedefs.
+#define MLF_IS_OVERLAP   0x20000000
+
+
 //------------------------------------------------------------------------
 // UTILITY : general purpose functions
 //------------------------------------------------------------------------
@@ -158,20 +165,8 @@ void UtilFree(void *data);
 // or NULL if an error occurred.
 char *UtilTimeString(void);
 
-// compute angle & distance from (0,0) to (dx,dy)
+// compute angle of line from (0,0) to (dx,dy)
 angle_g UtilComputeAngle(double dx, double dy);
-#define UtilComputeDist(dx,dy)  sqrt((dx) * (dx) + (dy) * (dy))
-
-// compute the parallel and perpendicular distances from a partition
-// line to a point.
-//
-#define UtilParallelDist(part,x,y)  \
-    (((x) * (part)->pdx + (y) * (part)->pdy + (part)->p_para)  \
-     / (part)->p_length)
-
-#define UtilPerpDist(part,x,y)  \
-    (((x) * (part)->pdy - (y) * (part)->pdx + (part)->p_perp)  \
-     / (part)->p_length)
 
 // checksum functions
 void Adler32_Begin(u32_t *crc);
@@ -211,27 +206,27 @@ void PutReject();
 //------------------------------------------------------------------------
 
 struct node_s;
-struct sector_s;
-struct superblock_s;
+
+class quadtree_c;
 
 
-// a wall_tip is where a wall meets a vertex
-typedef struct wall_tip_s
+// a wall-tip is where a wall meets a vertex
+typedef struct walltip_s
 {
 	// link in list.  List is kept in ANTI-clockwise order.
-	struct wall_tip_s *next;
-	struct wall_tip_s *prev;
+	struct walltip_s *next;
+	struct walltip_s *prev;
 
 	// angle that line makes at vertex (degrees).
 	angle_g angle;
 
-	// sectors on each side of wall.  Left is the side of increasing
-	// angles, right is the side of decreasing angles.  Either can be
-	// NULL for one sided walls.
-	struct sector_s *left;
-	struct sector_s *right;
+	// whether each side of wall is OPEN or CLOSED.
+	// left is the side of increasing angles, whereas
+	// right is the side of decreasing angles.
+	bool open_left;
+	bool open_right;
 }
-wall_tip_t;
+walltip_t;
 
 
 typedef struct vertex_s
@@ -244,150 +239,23 @@ typedef struct vertex_s
 	int index;
 
 	// vertex is newly created (from a seg split)
-	char is_new;
-
-	// when building normal nodes, unused vertices will be pruned.
-	char is_used;
+	bool is_new;
 
 	// usually NULL, unless this vertex occupies the same location as a
-	// previous vertex.
+	// previous vertex.  when there are multiple vertices at one spot,
+	// the second and third (etc) all refer to the first.
 	struct vertex_s *overlap;
 
-	// set of wall_tips
-	wall_tip_t *tip_set;
+	// list of wall-tips
+	walltip_t *tip_set;
 }
 vertex_t;
 
 
-typedef struct sector_s
+inline bool coalesce_sec(Sector *sec)
 {
-	// sector index.  Always valid after loading & pruning.
-	int index;
-
-	// allow segs from other sectors to coexist in a subsector.
-	char coalesce;
-
-	// -JL- non-zero if this sector contains a polyobj.
-	char has_polyobj;
-
-	// when building normal nodes, unused sectors will be pruned.
-	char is_used;
-
-	// heights
-	int floor_h, ceil_h;
-
-	// textures
-	char floor_tex[8];
-	char ceil_tex[8];
-
-	// attributes
-	int light;
-	int special;
-	int tag;
-
-	// used when building REJECT table.  Each set of sectors that are
-	// isolated from other sectors will have a different group number.
-	// Thus: on every 2-sided linedef, the sectors on both sides will be
-	// in the same group.  The rej_next, rej_prev fields are a link in a
-	// RING, containing all sectors of the same group.
-	int rej_group;
-
-	struct sector_s *rej_next;
-	struct sector_s *rej_prev;
-
-	// suppress superfluous mini warnings
-	int warned_facing;
-	char warned_unclosed;
+	return (sec->tag >= 900 && sec->tag < 1000);
 }
-sector_t;
-
-
-typedef struct sidedef_s
-{
-	// adjacent sector.  Can be NULL (invalid sidedef)
-	sector_t *sector;
-
-	// offset values
-	int x_offset, y_offset;
-
-	// texture names
-	char upper_tex[8];
-	char lower_tex[8];
-	char mid_tex[8];
-
-	// sidedef index.  Always valid after loading & pruning.
-	int index;
-
-	// when building normal nodes, unused sidedefs will be pruned.
-	char is_used;
-
-	// usually NULL, unless this sidedef is exactly the same as a
-	// previous one.  Only used during the pruning phase.
-	struct sidedef_s *equiv;
-
-	// this is true if the sidedef is on a special line.  We don't merge
-	// these sidedefs together, as they might scroll, or change texture
-	// when a switch is pressed.
-	int on_special;
-}
-sidedef_t;
-
-
-typedef struct linedef_s
-{
-	// link for list
-	struct linedef_s *next;
-
-	vertex_t *start;    // from this vertex...
-	vertex_t *end;      // ... to this vertex
-
-	sidedef_t *right;   // right sidedef
-	sidedef_t *left;    // left sidede, or NULL if none
-
-	// line is marked two-sided
-	char two_sided;
-
-	// prefer not to split
-	char is_precious;
-
-	// zero length (line should be totally ignored)
-	char zero_len;
-
-	// sector is the same on both sides
-	char self_ref;
-
-	int flags;
-	int type;
-	int tag;
-
-	// Hexen support
-	int specials[5];
-
-	// normally NULL, except when this linedef directly overlaps an earlier
-	// one (a rarely-used trick to create higher mid-masked textures).
-	// No segs should be created for these overlapping linedefs.
-	struct linedef_s *overlap;
-
-	// linedef index.  Always valid after loading & pruning of zero
-	// length lines has occurred.
-	int index;
-}
-linedef_t;
-
-
-typedef struct thing_s
-{
-	int x, y;
-	int type;
-	int options;
-
-	// other info (angle, and hexen stuff) omitted.  We don't need to
-	// write the THING lump, only read it.
-
-	// Always valid (thing indices never change).
-	int index;
-}
-thing_t;
 
 
 typedef struct seg_s
@@ -398,11 +266,8 @@ typedef struct seg_s
 	vertex_t *start;   // from this vertex...
 	vertex_t *end;     // ... to this vertex
 
-	// linedef that this seg goes along, or NULL if miniseg
-	linedef_t *linedef;
-
-	// adjacent sector, or NULL if invalid sidedef or miniseg
-	sector_t *sector;
+	// linedef that this seg goes along, or -1 if miniseg
+	int linedef;
 
 	// 0 for right, 1 for left
 	int side;
@@ -421,11 +286,11 @@ typedef struct seg_s
 	// start and end vertices produces the same location).  It should be
 	// ignored when writing the SEGS or V1 GL_SEGS lumps.  [Note: there
 	// won't be any of these when writing the V2 GL_SEGS lump].
-	char is_degenerate;
+	bool is_degenerate;
 
-	// the superblock that contains this seg, or NULL if the seg is no
-	// longer in any superblock (e.g. now in a subsector).
-	struct superblock_s *block;
+	// the quad-tree node that contains this seg, or NULL if the seg
+	// is now in a subsector.
+	quadtree_c *quad;
 
 	// precomputed data for faster calculations
 	double psx, psy;
@@ -439,11 +304,39 @@ typedef struct seg_s
 	// linedef that this seg initially comes from.  For "real" segs,
 	// this is just the same as the 'linedef' field above.  For
 	// "minisegs", this is the linedef of the partition line.
-	linedef_t *source_line;
+	int source_line;
+
+	// this only used by ClockwiseOrder()
+	angle_g cmp_angle;
+
+public:
+	// compute the seg private info (psx/y, pex/y, pdx/y, etc).
+	void Recompute();
+
+	// returns SIDE_LEFT, SIDE_RIGHT or 0 for being "on" the line.
+	int PointOnLineSide(double x, double y) const;
+
+	// compute the parallel and perpendicular distances from a partition
+	// line to a point.
+	//
+	inline double ParallelDist(double x, double y) const
+	{
+		return (x * pdx + y * pdy + p_para) / p_length;
+	}
+
+	inline double PerpDist(double x, double y) const
+	{
+		return (x * pdy - y * pdx + p_perp) / p_length;
+	}
 }
 seg_t;
 
 
+// a seg with this index is removed by SortSegs().
+// it must be a very high value.
+#define SEG_IS_GARBAGE  (1 << 29)
+
+
 typedef struct subsec_s
 {
 	// list of segs
@@ -459,6 +352,17 @@ typedef struct subsec_s
 	// approximate middle point
 	double mid_x;
 	double mid_y;
+
+public:
+	void DetermineMiddle();
+	void ClockwiseOrder();
+	void RenumberSegs();
+
+	void RoundOff();
+	void Normalise();
+
+	void SanityCheckClosed();
+	void SanityCheckHasRealSeg();
 }
 subsec_t;
 
@@ -485,8 +389,12 @@ child_t;
 
 typedef struct node_s
 {
-	int x, y;     // starting point
-	int dx, dy;   // offset to ending point
+	// these coordinates are high precision to support UDMF.
+	// in non-UDMF maps, they will actually be integral since a
+	// partition line *always* comes from a normal linedef.
+
+	double x, y;     // starting point
+	double dx, dy;   // offset to ending point
 
 	// right & left children
 	child_t r;
@@ -496,81 +404,81 @@ typedef struct node_s
 	// created.
 	int index;
 
-	// the node is too long, and the (dx,dy) values should be halved
-	// when writing into the NODES lump.
-	int too_long;
+public:
+	void SetPartition(const seg_t *part);
 }
 node_t;
 
 
-typedef struct superblock_s
+class quadtree_c
 {
-	// parent of this block, or NULL for a top-level block
-	struct superblock_s *parent;
-
+public:
 	// coordinates on map for this block, from lower-left corner to
-	// upper-right corner.  Pseudo-inclusive, i.e (x,y) is inside block
-	// if and only if x1 <= x < x2 and y1 <= y < y2.
+	// upper-right corner.  Fully inclusive, i.e (x,y) is inside this
+	// block when x1 < x < x2 and y1 < y < y2.
 	int x1, y1;
 	int x2, y2;
 
-	// sub-blocks.  NULL when empty.  [0] has the lower coordinates, and
-	// [1] has the higher coordinates.  Division of a square always
-	// occurs horizontally (e.g. 512x512 -> 256x512 -> 256x256).
-	struct superblock_s *subs[2];
+	// sub-trees.  NULL for leaf nodes.
+	// [0] has the lower coordinates, and [1] has the higher coordinates.
+	// Division of a square always occurs horizontally (e.g. 512x512 -> 256x512).
+	quadtree_c *subs[2];
 
-	// number of real segs and minisegs contained by this block
-	// (including all sub-blocks below it).
+	// count of real/mini segs contained in this node AND ALL CHILDREN.
 	int real_num;
 	int mini_num;
 
-	// list of segs completely contained by this block.
-	seg_t *segs;
-}
-superblock_t;
+	// list of segs contained in this node itself.
+	seg_t *list;
 
-#define SUPER_IS_LEAF(s)  \
-    ((s)->x2-(s)->x1 <= 256 && (s)->y2-(s)->y1 <= 256)
+public:
+	quadtree_c(int _x1, int _y1, int _x2, int _y2);
+	~quadtree_c();
+
+	void AddSeg(seg_t *seg);
+	void AddList(seg_t *list);
+
+	inline bool Empty() const
+	{
+		return (real_num + mini_num) == 0;
+	}
+
+	void ConvertToList(seg_t **list);
+
+	// check relationship between this box and the partition line.
+	// returns SIDE_LEFT or SIDE_RIGHT if box is definitively on a
+	// particular side, or 0 if the line intersects/touches the box.
+	//
+	int OnLineSide(const seg_t *part) const;
+};
 
 
 /* ----- Level data arrays ----------------------- */
 
-extern int num_vertices;
-extern int num_linedefs;
-extern int num_sidedefs;
-extern int num_sectors;
-extern int num_things;
-extern int num_segs;
-extern int num_subsecs;
-extern int num_nodes;
+extern std::vector<vertex_t *>  lev_vertices;
+extern std::vector<seg_t *>     lev_segs;
+extern std::vector<subsec_t *>  lev_subsecs;
+extern std::vector<node_t *>    lev_nodes;
+extern std::vector<walltip_t *> lev_walltips;
+
+#define num_vertices  ((int)lev_vertices.size())
+#define num_segs      ((int)lev_segs.size())
+#define num_subsecs   ((int)lev_subsecs.size())
+#define num_nodes     ((int)lev_nodes.size())
+#define num_walltips  ((int)lev_walltips.size())
 
 extern int num_old_vert;
 extern int num_new_vert;
-extern int num_complete_seg;
 
 
 /* ----- function prototypes ----------------------- */
 
 // allocation routines
-vertex_t *NewVertex(void);
-linedef_t *NewLinedef(void);
-sidedef_t *NewSidedef(void);
-sector_t *NewSector(void);
-thing_t *NewThing(void);
-seg_t *NewSeg(void);
-subsec_t *NewSubsec(void);
-node_t *NewNode(void);
-wall_tip_t *NewWallTip(void);
-
-// lookup routines
-vertex_t *LookupVertex(int index);
-linedef_t *LookupLinedef(int index);
-sidedef_t *LookupSidedef(int index);
-sector_t *LookupSector(int index);
-thing_t *LookupThing(int index);
-seg_t *LookupSeg(int index);
-subsec_t *LookupSubsec(int index);
-node_t *LookupNode(int index);
+vertex_t  *NewVertex();
+seg_t     *NewSeg();
+subsec_t  *NewSubsec();
+node_t    *NewNode();
+walltip_t *NewWallTip();
 
 Lump_c * CreateGLMarker();
 Lump_c * CreateLevelLump(const char *name, int max_size = -1);
@@ -607,29 +515,26 @@ void DetectOverlappingVertices(void);
 void DetectOverlappingLines(void);
 void DetectPolyobjSectors(void);
 
-// pruning routines
-void PruneVerticesAtEnd(void);
-
 // computes the wall tips for all of the vertices
-void CalculateWallTips(void);
+void CalculateWallTips();
 
-// return a new vertex (with correct wall_tip info) for the split that
+// return a new vertex (with correct wall-tip info) for the split that
 // happens along the given seg at the given location.
 //
 vertex_t *NewVertexFromSplitSeg(seg_t *seg, double x, double y);
 
 // return a new end vertex to compensate for a seg that would end up
 // being zero-length (after integer rounding).  Doesn't compute the
-// wall_tip info (thus this routine should only be used _after_ node
+// wall-tip info (thus this routine should only be used _after_ node
 // building).
 //
 vertex_t *NewVertexDegenerate(vertex_t *start, vertex_t *end);
 
-// check whether a line with the given delta coordinates and beginning
-// at this vertex is open.  Returns a sector reference if it's open,
-// or NULL if closed (void space or directly along a linedef).
+// check whether a line with the given delta coordinates from this
+// vertex is open or closed.  If there exists a walltip at same
+// angle, it is closed, likewise if line is in void space.
 //
-sector_t * VertexCheckOpen(vertex_t *vert, double dx, double dy);
+bool VertexCheckOpen(vertex_t *vert, double dx, double dy);
 
 
 //------------------------------------------------------------------------
@@ -640,11 +545,11 @@ sector_t * VertexCheckOpen(vertex_t *vert, double dx, double dy);
 #define IFFY_LEN  4.0
 
 
-// smallest distance between two points before being considered equal
-#define DIST_EPSILON  (1.0 / 128.0)
-
-// smallest degrees between two angles before being considered equal
-#define ANG_EPSILON  (1.0 / 1024.0)
+inline void ListAddSeg(seg_t **list_ptr, seg_t *seg)
+{
+	seg->next = *list_ptr;
+	*list_ptr = seg;
+}
 
 
 // an "intersection" remembers the vertex that touches a BSP divider
@@ -669,10 +574,10 @@ typedef struct intersection_s
 	// true if this intersection was on a self-referencing linedef
 	bool self_ref;
 
-	// sector on each side of the vertex (along the partition),
-	// or NULL when that direction isn't OPEN.
-	sector_t *before;
-	sector_t *after;
+	// status of each side of the vertex (along the partition),
+	// true if OPEN and false if CLOSED.
+	bool open_before;
+	bool open_after;
 }
 intersection_t;
 
@@ -684,13 +589,10 @@ intersection_t;
 // The 'depth' parameter is the current depth in the tree, used for
 // computing the current progress.
 //
-seg_t *PickNode(superblock_t *seg_list, int depth, const bbox_t *bbox);
+seg_t *PickNode(quadtree_c *tree, int depth, const bbox_t *bbox);
 
 // compute the boundary of the list of segs
-void FindLimits(superblock_t *seg_list, bbox_t *bbox);
-
-// compute the seg private info (psx/y, pex/y, pdx/y, etc).
-void RecomputeSeg(seg_t *seg);
+void FindLimits2(seg_t *list, bbox_t *bbox);
 
 // take the given seg 'cur', compare it with the partition line, and
 // determine it's fate: moving it into either the left or right lists
@@ -699,15 +601,15 @@ void RecomputeSeg(seg_t *seg);
 // the partition line.
 //
 void DivideOneSeg(seg_t *cur, seg_t *part,
-    superblock_t *left_list, superblock_t *right_list,
+    quadtree_c *left_quad, quadtree_c *right_quad,
     intersection_t ** cut_list);
 
 // remove all the segs from the list, partitioning them into the left
 // or right lists based on the given partition line.  Adds any
-// intersections onto the intersection list as it goes.
+// intersections into the intersection list as it goes.
 //
-void SeparateSegs(superblock_t *seg_list, seg_t *part,
-    superblock_t *left_list, superblock_t *right_list,
+void SeparateSegs(quadtree_c *tree, seg_t *part,
+    quadtree_c *left_quad, quadtree_c *right_quad,
     intersection_t ** cut_list);
 
 // analyse the intersection list, and add any needed minisegs to the
@@ -715,7 +617,7 @@ void SeparateSegs(superblock_t *seg_list, seg_t *part,
 // structures will be freed back into a quick-alloc list.
 //
 void AddMinisegs(seg_t *part,
-    superblock_t *left_list, superblock_t *right_list,
+    quadtree_c *left_quad, quadtree_c *right_list,
     intersection_t *cut_list);
 
 // free the quick allocation cut list
@@ -727,27 +629,13 @@ void FreeQuickAllocCuts(void);
 //------------------------------------------------------------------------
 
 
-// check the relationship between the given box and the partition
-// line.  Returns -1 if box is on left side, +1 if box is on right
-// size, or 0 if the line intersects the box.
-//
-int BoxOnLineSide(superblock_t *box, seg_t *part);
-
-// add the seg to the given list
-void AddSegToSuper(superblock_t *block, seg_t *seg);
-
-// increase the counts within the superblock, to account for the given
-// seg being split.
-//
-void SplitSegInSuper(superblock_t *block, seg_t *seg);
 
 // scan all the linedef of the level and convert each sidedef into a
 // seg (or seg pair).  Returns the list of segs.
 //
-superblock_t *CreateSegs(void);
+seg_t *CreateSegs(void);
 
-// free a super block.
-void FreeSuper(superblock_t *block);
+quadtree_c *TreeFromSegList(seg_t *list);
 
 // takes the seg list and determines if it is convex.  When it is, the
 // segs are converted to a subsector, and '*S' is the new subsector
@@ -756,8 +644,8 @@ void FreeSuper(superblock_t *block);
 // and '*N' is the new node (and '*S' is set to NULL).  Normally
 // returns BUILD_OK, or BUILD_Cancelled if user stopped it.
 //
-build_result_e BuildNodes(superblock_t *seg_list,
-    node_t ** N, subsec_t ** S, int depth, const bbox_t *bbox);
+build_result_e BuildNodes(seg_t *list, bbox_t *bounds /* output */,
+    node_t ** N, subsec_t ** S, int depth);
 
 // compute the height of the bsp tree, starting at 'node'.
 int ComputeBspHeight(node_t *node);
