@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -41,6 +41,7 @@
 // need these for the XXX_Notify() prototypes
 #include "e_cutpaste.h"
 #include "e_objects.h"
+#include "r_render.h"
 
 
 std::vector<Thing *>   Things;
@@ -111,6 +112,7 @@ static void DoClearChangeStatus()
 	Clipboard_NotifyBegin();
 	Selection_NotifyBegin();
 	 MapStuff_NotifyBegin();
+	 Render3D_NotifyBegin();
 	ObjectBox_NotifyBegin();
 }
 
@@ -125,6 +127,7 @@ static void DoProcessChangeStatus()
 	Clipboard_NotifyEnd();
 	Selection_NotifyEnd();
 	 MapStuff_NotifyEnd();
+	 Render3D_NotifyEnd();
 	ObjectBox_NotifyEnd();
 }
 
@@ -149,6 +152,16 @@ const char *BA_GetString(int offset)
 }
 
 
+fixcoord_t MakeValidCoord(double x)
+{
+	if (Level_format == MAPF_UDMF)
+		return TO_COORD(x);
+
+	// in standard format, coordinates must be integral
+	return TO_COORD(I_ROUND(x));
+}
+
+
 const char * Sector::FloorTex() const
 {
 	return basis_strtab.get(floor_tex);
@@ -270,12 +283,19 @@ int LineDef::WhatSideDef(int side) const
 }
 
 
+bool LineDef::IsSelfRef() const
+{
+	return (left >= 0) && (right >= 0) &&
+			SideDefs[left]->sector == SideDefs[right]->sector;
+}
+
+
 double LineDef::CalcLength() const
 {
-	int dx = Start()->x - End()->x;
-	int dy = Start()->y - End()->y;
+	double dx = Start()->x() - End()->x();
+	double dy = Start()->y() - End()->y();
 
-	return sqrt(dx * dx + dy * dy);
+	return hypot(dx, dy);
 }
 
 
@@ -408,6 +428,7 @@ static void RawInsert(obj_type_e objtype, int objnum, int *ptr)
 	Clipboard_NotifyInsert(objtype, objnum);
 	Selection_NotifyInsert(objtype, objnum);
 	 MapStuff_NotifyInsert(objtype, objnum);
+	 Render3D_NotifyInsert(objtype, objnum);
 	ObjectBox_NotifyInsert(objtype, objnum);
 
 	switch (objtype)
@@ -546,6 +567,7 @@ static int * RawDelete(obj_type_e objtype, int objnum)
 	Clipboard_NotifyDelete(objtype, objnum);
 	Selection_NotifyDelete(objtype, objnum);
 	 MapStuff_NotifyDelete(objtype, objnum);
+	 Render3D_NotifyDelete(objtype, objnum);
 	ObjectBox_NotifyDelete(objtype, objnum);
 
 	switch (objtype)
@@ -596,22 +618,27 @@ static void RawChange(obj_type_e objtype, int objnum, int field, int *value)
 	switch (objtype)
 	{
 		case OBJ_THINGS:
+			SYS_ASSERT(0 <= objnum && objnum < NumThings);
 			pos = (int*) Things[objnum];
 			break;
 
 		case OBJ_VERTICES:
+			SYS_ASSERT(0 <= objnum && objnum < NumVertices);
 			pos = (int*) Vertices[objnum];
 			break;
 
 		case OBJ_SECTORS:
+			SYS_ASSERT(0 <= objnum && objnum < NumSectors);
 			pos = (int*) Sectors[objnum];
 			break;
 
 		case OBJ_SIDEDEFS:
+			SYS_ASSERT(0 <= objnum && objnum < NumSideDefs);
 			pos = (int*) SideDefs[objnum];
 			break;
 
 		case OBJ_LINEDEFS:
+			SYS_ASSERT(0 <= objnum && objnum < NumLineDefs);
 			pos = (int*) LineDefs[objnum];
 			break;
 
@@ -627,6 +654,7 @@ static void RawChange(obj_type_e objtype, int objnum, int field, int *value)
 	Clipboard_NotifyChange(objtype, objnum, field);
 	Selection_NotifyChange(objtype, objnum, field);
 	 MapStuff_NotifyChange(objtype, objnum, field);
+	 Render3D_NotifyChange(objtype, objnum, field);
 	ObjectBox_NotifyChange(objtype, objnum, field);
 }
 
@@ -825,7 +853,10 @@ void BA_End()
 	if (cur_group->Empty())
 		delete cur_group;
 	else
+	{
 		undo_history.push_front(cur_group);
+		Status_Set("%s", cur_group->GetMsg());
+	}
 
 	cur_group = NULL;
 
@@ -1027,7 +1058,7 @@ bool BA_Undo()
 	undo_group_c * grp = undo_history.front();
 	undo_history.pop_front();
 
-	Status_Set("Undo: %s", grp->GetMsg());
+	Status_Set("UNDO: %s", grp->GetMsg());
 
 	grp->ReApply();
 
@@ -1153,8 +1184,8 @@ bool BA_ChangeLD(int line, byte field, int value)
 
 static void ChecksumThing(crc32_c& crc, const Thing * T)
 {
-	crc += T->x;
-	crc += T->y;
+	crc += T->raw_x;
+	crc += T->raw_y;
 	crc += T->angle;
 	crc += T->type;
 	crc += T->options;
@@ -1162,8 +1193,8 @@ static void ChecksumThing(crc32_c& crc, const Thing * T)
 
 static void ChecksumVertex(crc32_c& crc, const Vertex * V)
 {
-	crc += V->x;
-	crc += V->y;
+	crc += V->raw_x;
+	crc += V->raw_y;
 }
 
 static void ChecksumSector(crc32_c& crc, const Sector * SEC)
