@@ -276,7 +276,7 @@ struct vertex_X_CMP_pred
 		const Vertex *V1 = Vertices[A];
 		const Vertex *V2 = Vertices[B];
 
-		return V1->x < V2->x;
+		return V1->raw_x < V2->raw_x;
 	}
 };
 
@@ -306,9 +306,9 @@ void Vertex_FindOverlaps(selection_c& sel)
 
 	for (int k = 0 ; k < NumVertices ; k++)
 	{
-		for (int n = k + 1 ; n < NumVertices && VERT_N->x == VERT_K->x ; n++)
+		for (int n = k + 1 ; n < NumVertices && VERT_N->raw_x == VERT_K->raw_x ; n++)
 		{
-			if (VERT_N->y == VERT_K->y)
+			if (VERT_N->raw_y == VERT_K->raw_y)
 			{
 				sel.set(sorted_list[k]);
 			}
@@ -363,14 +363,12 @@ static void Vertex_MergeOne(int idx, selection_c& merge_verts)
 void Vertex_MergeOverlaps()
 {
 	selection_c verts;
-	selection_iterator_c it;
-
 	Vertex_FindOverlaps(verts);
 
 	BA_Begin();
 	BA_Message("merged overlapping vertices");
 
-	for (verts.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(verts) ; !it.done() ; it.next())
 	{
 		Vertex_MergeOne(*it, verts);
 	}
@@ -501,7 +499,7 @@ check_result_e CHECK_Vertices(int min_severity = 0)
 			dialog->AddLine("No overlapping vertices");
 		else
 		{
-			sprintf(check_message, "%d overlapping vertices", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d overlapping vertices", sel.count_obj());
 
 			dialog->AddLine(check_message, 2, 210,
 			                "Show",  &UI_Check_Vertices::action_highlight,
@@ -515,7 +513,7 @@ check_result_e CHECK_Vertices(int min_severity = 0)
 			dialog->AddLine("No dangling vertices");
 		else
 		{
-			sprintf(check_message, "%d dangling vertices", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d dangling vertices", sel.count_obj());
 
 			dialog->AddLine(check_message, 2, 210,
 			                "Show",  &UI_Check_Vertices::action_show_danglers);
@@ -528,7 +526,7 @@ check_result_e CHECK_Vertices(int min_severity = 0)
 			dialog->AddLine("No unused vertices");
 		else
 		{
-			sprintf(check_message, "%d unused vertices", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d unused vertices", sel.count_obj());
 
 			dialog->AddLine(check_message, 1, 210,
 			                "Show",   &UI_Check_Vertices::action_show_unused,
@@ -728,7 +726,7 @@ void Sectors_FindUnknown(selection_c& list, std::map<int, int>& types)
 
 	list.change_type(OBJ_SECTORS);
 
-	int max_type = (game_info.gen_sectors == 2) ? 8191 : 2047;
+	int max_type = (Features.gen_sectors == 2) ? 8191 : 2047;
 
 	for (int n = 0 ; n < NumSectors ; n++)
 	{
@@ -746,9 +744,9 @@ void Sectors_FindUnknown(selection_c& list, std::map<int, int>& types)
 		}
 
 		// Boom and ZDoom generalized sectors
-		if (game_info.gen_sectors == 2)
+		if (Features.gen_sectors == 2)
 			type_num &= 255;
-		else if (game_info.gen_sectors)
+		else if (Features.gen_sectors)
 			type_num &= 31;
 
 		const sectortype_t *info = M_GetSectorType(type_num);
@@ -804,12 +802,10 @@ void Sectors_ClearUnknown()
 
 	Sectors_FindUnknown(sel, types);
 
-	selection_iterator_c it;
-
 	BA_Begin();
 	BA_Message("cleared unknown sector types");
 
-	for (sel.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(sel) ; !it.done() ; it.next())
 		BA_ChangeSEC(*it, Sector::F_TYPE, 0);
 
 	BA_End();
@@ -850,8 +846,6 @@ void Sectors_RemoveUnused()
 	DeleteObjects(&sel);
 
 	BA_End();
-
-//??	Status_Set("Removed %d vertices", sel.count_obj());
 }
 
 
@@ -933,8 +927,6 @@ void SideDefs_RemoveUnused()
 	DeleteObjects(&sel);
 
 	BA_End();
-
-//??	Status_Set("Removed %d vertices", sel.count_obj());
 }
 
 
@@ -1056,7 +1048,9 @@ void SideDefs_Unpack(bool is_after_load)
 	}
 
 	if (is_after_load)
+	{
 		BA_Abort(true /* keep changes */);
+	}
 	else
 	{
 		BA_Message("unpacked all sidedefs");
@@ -1192,7 +1186,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 			dialog->AddLine("No unclosed sectors");
 		else
 		{
-			sprintf(check_message, "%d unclosed sectors", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d unclosed sectors", sel.count_obj());
 
 			dialog->AddLine(check_message, 2, 220,
 			                "Show",  &UI_Check_Sectors::action_show_unclosed,
@@ -1206,7 +1200,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 			dialog->AddLine("No mismatched sectors");
 		else
 		{
-			sprintf(check_message, "%d mismatched sectors", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d mismatched sectors", sel.count_obj());
 
 			dialog->AddLine(check_message, 2, 220,
 			                "Show",  &UI_Check_Sectors::action_show_mismatch,
@@ -1220,7 +1214,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 			dialog->AddLine("No sectors with ceil < floor");
 		else
 		{
-			sprintf(check_message, "%d sectors with ceil < floor", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d sectors with ceil < floor", sel.count_obj());
 
 			dialog->AddLine(check_message, 2, 220,
 			                "Show", &UI_Check_Sectors::action_show_ceil,
@@ -1236,7 +1230,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 			dialog->AddLine("No unknown sector types");
 		else
 		{
-			sprintf(check_message, "%d unknown sector types", (int)types.size());
+			snprintf(check_message, sizeof(check_message), "%d unknown sector types", (int)types.size());
 
 			dialog->AddLine(check_message, 2, 220,
 			                "Show",   &UI_Check_Sectors::action_show_unknown,
@@ -1253,7 +1247,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 		{
 			int approx_num = sel.count_obj();
 
-			sprintf(check_message, "%d shared sidedefs", approx_num);
+			snprintf(check_message, sizeof(check_message), "%d shared sidedefs", approx_num);
 
 			dialog->AddLine(check_message, 1, 200,
 			                "Show",   &UI_Check_Sectors::action_show_packed,
@@ -1267,7 +1261,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 			dialog->AddLine("No unused sectors");
 		else
 		{
-			sprintf(check_message, "%d unused sectors", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d unused sectors", sel.count_obj());
 
 			dialog->AddLine(check_message, 1, 170,
 			                "Remove", &UI_Check_Sectors::action_remove);
@@ -1280,7 +1274,7 @@ check_result_e CHECK_Sectors(int min_severity = 0)
 			dialog->AddLine("No unused sidedefs");
 		else
 		{
-			sprintf(check_message, "%d unused sidedefs", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d unused sidedefs", sel.count_obj());
 
 			dialog->AddLine(check_message, 1, 170,
 			                "Remove", &UI_Check_Sectors::action_remove_sidedefs);
@@ -1419,8 +1413,8 @@ void Things_FindInVoid(selection_c& list)
 
 	for (int n = 0 ; n < NumThings ; n++)
 	{
-		int x = Things[n]->x;
-		int y = Things[n]->y;
+		double x = Things[n]->x();
+		double y = Things[n]->y();
 
 		Objid obj;
 
@@ -1440,8 +1434,8 @@ void Things_FindInVoid(selection_c& list)
 
 		for (int corner = 0 ; corner < 4 ; corner++)
 		{
-			int x2 = x + ((corner & 1) ? -4 : +4);
-			int y2 = y + ((corner & 2) ? -4 : +4);
+			double x2 = x + ((corner & 1) ? -4 : +4);
+			double y2 = y + ((corner & 2) ? -4 : +4);
 
 			GetNearObject(obj, OBJ_SECTORS, x2, y2);
 
@@ -1524,16 +1518,16 @@ void Things_FindDuds(selection_c& list)
 		int modes   = 1;
 		int classes = 1;
 
-		if (Level_format == MAPF_Hexen)
+		if (Level_format != MAPF_Doom)
 		{
 			modes = T->options & (MTF_Hexen_SP | MTF_Hexen_COOP | MTF_Hexen_DM);
 		}
-		else if (game_info.coop_dm_flags)
+		else if (Features.coop_dm_flags)
 		{
 			modes = (~T->options) & (MTF_Not_SP | MTF_Not_COOP | MTF_Not_DM);
 		}
 
-		if (Level_format == MAPF_Hexen)
+		if (Level_format != MAPF_Doom)
 		{
 			classes = T->options & (MTF_Hexen_Cleric | MTF_Hexen_Fighter | MTF_Hexen_Mage);
 		}
@@ -1582,14 +1576,14 @@ void Things_FixDuds()
 		if (skills == 0)
 			new_options |= MTF_Easy | MTF_Medium | MTF_Hard;
 
-		if (Level_format == MAPF_Hexen)
+		if (Level_format != MAPF_Doom)
 		{
 			modes = T->options & (MTF_Hexen_SP | MTF_Hexen_COOP | MTF_Hexen_DM);
 
 			if (modes == 0)
 				new_options |= MTF_Hexen_SP | MTF_Hexen_COOP | MTF_Hexen_DM;
 		}
-		else if (game_info.coop_dm_flags)
+		else if (Features.coop_dm_flags)
 		{
 			modes = (~T->options) & (MTF_Not_SP | MTF_Not_COOP | MTF_Not_DM);
 
@@ -1597,7 +1591,7 @@ void Things_FixDuds()
 				new_options &= ~(MTF_Not_SP | MTF_Not_COOP | MTF_Not_DM);
 		}
 
-		if (Level_format == MAPF_Hexen)
+		if (Level_format != MAPF_Doom)
 		{
 			classes = T->options & (MTF_Hexen_Cleric | MTF_Hexen_Fighter | MTF_Hexen_Mage);
 
@@ -1680,11 +1674,11 @@ static bool ThingStuckInThing(const Thing *T1, const thingtype_t *info1,
 	else if (info2->group == 'm' && info1->group != 'p')
 		r2 = MAX(4, r2 - MONSTER_STEP_DIST);
 
-	if (T1->x - r1 >= T2->x + r2) return false;
-	if (T1->y - r1 >= T2->y + r2) return false;
+	if (T1->x() - r1 >= T2->x() + r2) return false;
+	if (T1->y() - r1 >= T2->y() + r2) return false;
 
-	if (T1->x + r1 <= T2->x - r2) return false;
-	if (T1->y + r1 <= T2->y - r2) return false;
+	if (T1->x() + r1 <= T2->x() - r2) return false;
+	if (T1->y() + r1 <= T2->y() - r2) return false;
 
 	// teleporters and DM starts can safely overlap moving actors
 	if ((info1->flags & THINGDEF_TELEPT) && is_actor2) return false;
@@ -1694,7 +1688,7 @@ static bool ThingStuckInThing(const Thing *T1, const thingtype_t *info1,
 	int opt1 = T1->options;
 	int opt2 = T2->options;
 
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		if (info1->group == 'p') opt1 |= 0x7E7;
 		if (info2->group == 'p') opt2 |= 0x7E7;
@@ -1761,10 +1755,10 @@ static bool ThingStuckInWall(const Thing *T, int r, char group)
 	// bounding box, not just touch it.
 	r = r - 1;
 
-	int x1 = T->x - r;
-	int y1 = T->y - r;
-	int x2 = T->x + r;
-	int y2 = T->y + r;
+	double x1 = T->x() - r;
+	double y1 = T->y() - r;
+	double x2 = T->x() + r;
+	double y2 = T->y() + r;
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
@@ -1910,7 +1904,7 @@ check_result_e CHECK_Things(int min_severity = 0)
 			dialog->AddLine("No unknown thing types");
 		else
 		{
-			sprintf(check_message, "%d unknown things", (int)types.size());
+			snprintf(check_message, sizeof(check_message), "%d unknown things", (int)types.size());
 
 			dialog->AddLine(check_message, 2, 200,
 			                "Show",   &UI_Check_Things::action_show_unknown,
@@ -1925,7 +1919,7 @@ check_result_e CHECK_Things(int min_severity = 0)
 			dialog->AddLine("No stuck actors");
 		else
 		{
-			sprintf(check_message, "%d stuck actors", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d stuck actors", sel.count_obj());
 
 			dialog->AddLine(check_message, 2, 200,
 			                "Show",  &UI_Check_Things::action_show_stuck);
@@ -1938,7 +1932,7 @@ check_result_e CHECK_Things(int min_severity = 0)
 			dialog->AddLine("No things in the void");
 		else
 		{
-			sprintf(check_message, "%d things in the void", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d things in the void", sel.count_obj());
 
 			dialog->AddLine(check_message, 1, 200,
 			                "Show",   &UI_Check_Things::action_show_void,
@@ -1952,7 +1946,7 @@ check_result_e CHECK_Things(int min_severity = 0)
 			dialog->AddLine("No unspawnable things -- skill flags are OK");
 		else
 		{
-			sprintf(check_message, "%d unspawnable things", sel.count_obj());
+			snprintf(check_message, sizeof(check_message), "%d unspawnable things", sel.count_obj());
 			dialog->AddLine(check_message, 1, 200,
 			                "Show", &UI_Check_Things::action_show_duds,
 			                "Fix",  &UI_Check_Things::action_fix_duds);
@@ -1966,7 +1960,7 @@ check_result_e CHECK_Things(int min_severity = 0)
 
 		mask = Things_FindStarts(&dm_num);
 
-		if (game_info.no_need_players)
+		if (Features.no_need_players)
 			dialog->AddLine("Player starts not needed, no check done");
 		else if (! (mask & 1))
 			dialog->AddLine("Player 1 start is missing!", 2);
@@ -1979,7 +1973,7 @@ check_result_e CHECK_Things(int min_severity = 0)
 		else
 			dialog->AddLine("Found all 4 player starts");
 
-		if (game_info.no_need_players)
+		if (Features.no_need_players)
 		{
 			// leave a blank space
 		}
@@ -1987,21 +1981,21 @@ check_result_e CHECK_Things(int min_severity = 0)
 		{
 			dialog->AddLine("Map is missing deathmatch starts", 1);
 		}
-		else if (dm_num < game_info.min_dm_starts)
+		else if (dm_num < Misc_info.min_dm_starts)
 		{
-			sprintf(check_message, "Found %d deathmatch starts -- need at least %d", dm_num,
-			        game_info.min_dm_starts);
+			snprintf(check_message, sizeof(check_message), "Found %d deathmatch starts -- need at least %d", dm_num,
+			        Misc_info.min_dm_starts);
 			dialog->AddLine(check_message, 1);
 		}
-		else if (dm_num > game_info.max_dm_starts)
+		else if (dm_num > Misc_info.max_dm_starts)
 		{
-			sprintf(check_message, "Found %d deathmatch starts -- maximum is %d", dm_num,
-			        game_info.max_dm_starts);
+			snprintf(check_message, sizeof(check_message), "Found %d deathmatch starts -- maximum is %d", dm_num,
+			        Misc_info.max_dm_starts);
 			dialog->AddLine(check_message, 2);
 		}
 		else
 		{
-			sprintf(check_message, "Found %d deathmatch starts -- OK", dm_num);
+			snprintf(check_message, sizeof(check_message), "Found %d deathmatch starts -- OK", dm_num);
 			dialog->AddLine(check_message);
 		}
 
@@ -2038,7 +2032,7 @@ void LineDefs_FindZeroLen(selection_c& lines)
 	lines.change_type(OBJ_LINEDEFS);
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
-		if (LineDefs[n]->isZeroLength())
+		if (LineDefs[n]->IsZeroLength())
 			lines.set(n);
 }
 
@@ -2049,7 +2043,7 @@ void LineDefs_RemoveZeroLen()
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
-		if (LineDefs[n]->isZeroLength())
+		if (LineDefs[n]->IsZeroLength())
 			lines.set(n);
 	}
 
@@ -2287,7 +2281,7 @@ void LineDefs_FindUnknown(selection_c& list, std::map<int, int>& types)
 		const linetype_t *info = M_GetLineType(type_num);
 
 		// Boom generalized line type?
-		if (game_info.gen_types && is_genline(type_num))
+		if (Features.gen_types && is_genline(type_num))
 			continue;
 
 		if (strncmp(info->desc, "UNKNOWN", 7) == 0)
@@ -2342,12 +2336,10 @@ void LineDefs_ClearUnknown()
 
 	LineDefs_FindUnknown(sel, types);
 
-	selection_iterator_c it;
-
 	BA_Begin();
 	BA_Message("cleared unknown line types");
 
-	for (sel.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(sel) ; !it.done() ; it.next())
 		BA_ChangeLD(*it, LineDef::F_TYPE, 0);
 
 	BA_End();
@@ -2362,15 +2354,15 @@ static int linedef_pos_cmp(int A, int B)
 	const LineDef *AL = LineDefs[A];
 	const LineDef *BL = LineDefs[B];
 
-	int A_x1 = AL->Start()->x;
-	int A_y1 = AL->Start()->y;
-	int A_x2 = AL->End()->x;
-	int A_y2 = AL->End()->y;
+	int A_x1 = AL->Start()->x();
+	int A_y1 = AL->Start()->y();
+	int A_x2 = AL->End()->x();
+	int A_y2 = AL->End()->y();
 
-	int B_x1 = BL->Start()->x;
-	int B_y1 = BL->Start()->y;
-	int B_x2 = BL->End()->x;
-	int B_y2 = BL->End()->y;
+	int B_x1 = BL->Start()->x();
+	int B_y1 = BL->Start()->y();
+	int B_x2 = BL->End()->x();
+	int B_y2 = BL->End()->y();
 
 	if (A_x1 > A_x2 || (A_x1 == A_x2 && A_y1 > A_y2))
 	{
@@ -2413,10 +2405,10 @@ struct linedef_minx_CMP_pred
 		const LineDef *AL = LineDefs[A];
 		const LineDef *BL = LineDefs[B];
 
-		int A_x1 = MIN(AL->Start()->x, AL->End()->x);
-		int B_x1 = MIN(BL->Start()->x, BL->End()->x);
+		fixcoord_t A_x = MIN(AL->Start()->raw_x, AL->End()->raw_x);
+		fixcoord_t B_x = MIN(BL->Start()->raw_x, BL->End()->raw_x);
 
-		return A_x1 < B_x1;
+		return A_x < B_x;
 	}
 };
 
@@ -2447,7 +2439,7 @@ void LineDefs_FindOverlaps(selection_c& lines)
 		int ld2 = sorted_list[n + 1];
 
 		// ignore zero-length lines
-		if (LineDefs[ld2]->isZeroLength())
+		if (LineDefs[ld2]->IsZeroLength())
 			continue;
 
 		// only the second (or third, etc) linedef is stored
@@ -2504,7 +2496,7 @@ static int CheckLinesCross(int A, int B)
 	const LineDef *BL = LineDefs[B];
 
 	// ignore zero-length lines
-	if (AL->isZeroLength() || BL->isZeroLength())
+	if (AL->IsZeroLength() || BL->IsZeroLength())
 		return 0;
 
 	// ignore directly overlapping here
@@ -2517,14 +2509,14 @@ static int CheckLinesCross(int A, int B)
 	// the algorithm in LineDefs_FindCrossings() ensures that A and B
 	// already overlap on the X axis.  hence only check Y axis here.
 
-	if (MIN(AL->Start()->y, AL->End()->y) >
-	    MAX(BL->Start()->y, BL->End()->y))
+	if (MIN(AL->Start()->raw_y, AL->End()->raw_y) >
+	    MAX(BL->Start()->raw_y, BL->End()->raw_y))
 	{
 		return 0;
 	}
 
-	if (MIN(BL->Start()->y, BL->End()->y) >
-	    MAX(AL->Start()->y, AL->End()->y))
+	if (MIN(BL->Start()->raw_y, BL->End()->raw_y) >
+	    MAX(AL->Start()->raw_y, AL->End()->raw_y))
 	{
 		return 0;
 	}
@@ -2532,15 +2524,15 @@ static int CheckLinesCross(int A, int B)
 
 	// precise (but slower) intersection test
 
-	int ax1 = AL->Start()->x;
-	int ay1 = AL->Start()->y;
-	int ax2 = AL->End()->x;
-	int ay2 = AL->End()->y;
+	double ax1 = AL->Start()->x();
+	double ay1 = AL->Start()->y();
+	double ax2 = AL->End()->x();
+	double ay2 = AL->End()->y();
 
-	int bx1 = BL->Start()->x;
-	int by1 = BL->Start()->y;
-	int bx2 = BL->End()->x;
-	int by2 = BL->End()->y;
+	double bx1 = BL->Start()->x();
+	double by1 = BL->Start()->y();
+	double bx2 = BL->End()->x();
+	double by2 = BL->End()->y();
 
 	double c = PerpDist(bx1, by1,  ax1, ay1, ax2, ay2);
 	double d = PerpDist(bx2, by2,  ax1, ay1, ax2, ay2);
@@ -2629,14 +2621,13 @@ void LineDefs_FindCrossings(selection_c& lines)
 
 	std::sort(sorted_list.begin(), sorted_list.end(), linedef_minx_CMP_pred());
 
-
 	for (n = 0 ; n < NumLineDefs ; n++)
 	{
 		int n2 = sorted_list[n];
 
 		const LineDef *L1 = LineDefs[n2];
 
-		int max_x = MAX(L1->Start()->x, L1->End()->x);
+		fixcoord_t max_x = MAX(L1->Start()->raw_x, L1->End()->raw_x);
 
 		for (int k = n + 1 ; k < NumLineDefs ; k++)
 		{
@@ -2644,7 +2635,7 @@ void LineDefs_FindCrossings(selection_c& lines)
 
 			const LineDef *L2 = LineDefs[k2];
 
-			int min_x = MIN(L2->Start()->x, L2->End()->x);
+			fixcoord_t min_x = MIN(L2->Start()->raw_x, L2->End()->raw_x);
 
 			// stop when all remaining linedefs are to the right of L1
 			if (min_x > max_x)
@@ -2812,7 +2803,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No zero-length linedefs");
 		else
 		{
-			sprintf(check_buffer, "%d zero-length linedefs", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d zero-length linedefs", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 220,
 			                "Show",   &UI_Check_LineDefs::action_show_zero,
@@ -2826,7 +2817,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No overlapping linedefs");
 		else
 		{
-			sprintf(check_buffer, "%d overlapping linedefs", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d overlapping linedefs", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 220,
 			                "Show",   &UI_Check_LineDefs::action_show_overlap,
@@ -2840,7 +2831,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No criss-crossing linedefs");
 		else
 		{
-			sprintf(check_buffer, "%d criss-crossing linedefs", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d criss-crossing linedefs", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 220,
 			                "Show", &UI_Check_LineDefs::action_show_crossing);
@@ -2855,7 +2846,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No unknown line types");
 		else
 		{
-			sprintf(check_buffer, "%d unknown line types", (int)types.size());
+			snprintf(check_buffer, sizeof(check_buffer), "%d unknown line types", (int)types.size());
 
 			dialog->AddLine(check_buffer, 1, 210,
 			                "Show",   &UI_Check_LineDefs::action_show_unknown,
@@ -2870,7 +2861,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No linedefs without a right side");
 		else
 		{
-			sprintf(check_buffer, "%d linedefs without right side", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d linedefs without right side", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 300,
 			                "Show", &UI_Check_LineDefs::action_show_mis_right);
@@ -2883,7 +2874,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No manual doors on 1S linedefs");
 		else
 		{
-			sprintf(check_buffer, "%d manual doors on 1S linedefs", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d manual doors on 1S linedefs", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 300,
 			                "Show", &UI_Check_LineDefs::action_show_manual_doors,
@@ -2897,7 +2888,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No non-blocking one-sided linedefs");
 		else
 		{
-			sprintf(check_buffer, "%d non-blocking one-sided linedefs", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d non-blocking one-sided linedefs", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 300,
 			                "Show", &UI_Check_LineDefs::action_show_lack_impass,
@@ -2911,7 +2902,7 @@ check_result_e CHECK_LineDefs(int min_severity)
 			dialog->AddLine("No linedefs with wrong 2S flag");
 		else
 		{
-			sprintf(check_buffer, "%d linedefs with wrong 2S flag", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d linedefs with wrong 2S flag", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 300,
 			                "Show", &UI_Check_LineDefs::action_show_bad_2s_flag,
@@ -2968,7 +2959,7 @@ void Tags_UsedRange(int *min_tag, int *max_tag)
 		int tag = Sectors[i]->tag;
 
 		// ignore special tags
-		if (game_info.tag_666 && (tag == 666 || tag == 667))
+		if (Features.tag_666 && (tag == 666 || tag == 667))
 			continue;
 
 		if (tag > 0)
@@ -2988,25 +2979,20 @@ void Tags_UsedRange(int *min_tag, int *max_tag)
 
 void Tags_ApplyNewValue(int new_tag)
 {
-	selection_c list;
-	selection_iterator_c it;
+	// uses the current selection (caller must set it up)
 
-	if (GetCurrentObjects(&list))
+	BA_Begin();
+	BA_MessageForSel("new tag for", edit.Selected);
+
+	for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 	{
-		BA_Begin();
-
-		BA_MessageForSel("new tag for", &list);
-
-		for (list.begin(&it); !it.at_end(); ++it)
-		{
-			if (edit.mode == OBJ_LINEDEFS)
-				BA_ChangeLD(*it, LineDef::F_TAG, new_tag);
-			else if (edit.mode == OBJ_SECTORS)
-				BA_ChangeSEC(*it, Sector::F_TAG, new_tag);
-		}
-
-		BA_End();
+		if (edit.mode == OBJ_LINEDEFS)
+			BA_ChangeLD(*it, LineDef::F_TAG, new_tag);
+		else if (edit.mode == OBJ_SECTORS)
+			BA_ChangeSEC(*it, Sector::F_TAG, new_tag);
 	}
+
+	BA_End();
 }
 
 
@@ -3037,42 +3023,33 @@ void CMD_ApplyTag()
 	}
 
 
-	bool unselect = false;
-
-	if (edit.Selected->empty())
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
-		if (edit.highlight.is_nil())
-		{
-			Beep("ApplyTag: nothing selected");
-			return;
-		}
-
-		edit.Selected->set(edit.highlight.num);
-		unselect = true;
+		Beep("ApplyTag: nothing selected");
+		return;
 	}
 
-
 	int min_tag, max_tag;
 
 	Tags_UsedRange(&min_tag, &max_tag);
 
 	int new_tag = max_tag + (do_last ? 0 : 1);
-
 	if (new_tag <= 0)
 	{
 		Beep("No last tag");
-		return;
 	}
 	else if (new_tag > 32767)
 	{
 		Beep("Out of tag numbers");
-		return;
+	}
+	else
+	{
+		Tags_ApplyNewValue(new_tag);
 	}
 
-	Tags_ApplyNewValue(new_tag);
-
-	if (unselect)
-		Selection_Clear(true);
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
@@ -3109,7 +3086,7 @@ void Tags_FindUnmatchedSectors(selection_c& secs)
 
 		// DOOM and Heretic use tag #666 to open doors (etc) on the
 		// death of boss monsters.
-		if (game_info.tag_666 && (tag == 666 || tag == 667))
+		if (Features.tag_666 && (tag == 666 || tag == 667))
 			continue;
 
 		if (! LD_tag_exists(tag))
@@ -3205,7 +3182,7 @@ void Tags_ShowMissingTags()
 
 static bool SEC_check_beast_mark(int tag)
 {
-	if (! game_info.tag_666)
+	if (! Features.tag_666)
 		return true;
 
 	if (tag == 667)
@@ -3217,7 +3194,7 @@ static bool SEC_check_beast_mark(int tag)
 	if (tag == 666)
 	{
 		// for Heretic, the map must be an end-of-episode map: ExM8
-		if (game_info.tag_666 == 2)
+		if (Features.tag_666 == 2)
 		{
 			if (strlen(Level_name) != 4)
 				return false;
@@ -3343,7 +3320,7 @@ check_result_e CHECK_Tags(int min_severity)
 			dialog->AddLine("No linedefs missing a needed tag");
 		else
 		{
-			sprintf(check_buffer, "%d linedefs missing a needed tag", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d linedefs missing a needed tag", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 320,
 			                "Show", &UI_Check_Tags::action_show_missing_tag);
@@ -3356,7 +3333,7 @@ check_result_e CHECK_Tags(int min_severity)
 			dialog->AddLine("No tagged linedefs w/o a matching sector");
 		else
 		{
-			sprintf(check_buffer, "%d tagged linedefs w/o a matching sector", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d tagged linedefs w/o a matching sector", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 2, 350,
 			                "Show", &UI_Check_Tags::action_show_unmatch_line);
@@ -3369,7 +3346,7 @@ check_result_e CHECK_Tags(int min_severity)
 			dialog->AddLine("No tagged sectors w/o a matching linedef");
 		else
 		{
-			sprintf(check_buffer, "%d tagged sectors w/o a matching linedef", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d tagged sectors w/o a matching linedef", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 350,
 			                "Show", &UI_Check_Tags::action_show_unmatch_sec);
@@ -3382,7 +3359,7 @@ check_result_e CHECK_Tags(int min_severity)
 			dialog->AddLine("No sectors with tag 666 or 667 used on the wrong map");
 		else
 		{
-			sprintf(check_buffer, "%d sectors have an invalid 666/667 tag", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d sectors have an invalid 666/667 tag", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 350,
 			                "Show", &UI_Check_Tags::action_show_beast_marks);
@@ -3399,7 +3376,7 @@ check_result_e CHECK_Tags(int min_severity)
 			dialog->AddLine("No tags are in use");
 		else
 		{
-			sprintf(check_buffer, "Lowest tag: %d   Highest tag: %d", min_tag, max_tag);
+			snprintf(check_buffer, sizeof(check_buffer), "Lowest tag: %d   Highest tag: %d", min_tag, max_tag);
 			dialog->AddLine(check_buffer);
 		}
 
@@ -4277,7 +4254,7 @@ check_result_e CHECK_Textures(int min_severity)
 			dialog->AddLine("No unknown textures");
 		else
 		{
-			sprintf(check_buffer, "%d unknown textures", (int)names.size());
+			snprintf(check_buffer, sizeof(check_buffer), "%d unknown textures", (int)names.size());
 
 			dialog->AddLine(check_buffer, 2, 200,
 			                "Show", &UI_Check_Textures::action_show_unk_tex,
@@ -4292,7 +4269,7 @@ check_result_e CHECK_Textures(int min_severity)
 			dialog->AddLine("No unknown flats");
 		else
 		{
-			sprintf(check_buffer, "%d unknown flats", (int)names.size());
+			snprintf(check_buffer, sizeof(check_buffer), "%d unknown flats", (int)names.size());
 
 			dialog->AddLine(check_buffer, 2, 200,
 			                "Show", &UI_Check_Textures::action_show_unk_flat,
@@ -4301,7 +4278,7 @@ check_result_e CHECK_Textures(int min_severity)
 		}
 
 
-		if (! game_info.medusa_fixed)
+		if (! Features.medusa_fixed)
 		{
 			Textures_FindMedusa(sel, names);
 
@@ -4309,7 +4286,7 @@ check_result_e CHECK_Textures(int min_severity)
 				dialog->AddLine("No textures causing Medusa Effect");
 			else
 			{
-				sprintf(check_buffer, "%d Medusa textures", (int)names.size());
+				snprintf(check_buffer, sizeof(check_buffer), "%d Medusa textures", (int)names.size());
 
 				dialog->AddLine(check_buffer, 2, 200,
 								"Show", &UI_Check_Textures::action_show_medusa,
@@ -4327,7 +4304,7 @@ check_result_e CHECK_Textures(int min_severity)
 			dialog->AddLine("No missing textures on walls");
 		else
 		{
-			sprintf(check_buffer, "%d missing textures on walls", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d missing textures on walls", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 275,
 			                "Show", &UI_Check_Textures::action_show_missing,
@@ -4341,7 +4318,7 @@ check_result_e CHECK_Textures(int min_severity)
 			dialog->AddLine("No transparent textures on solids");
 		else
 		{
-			sprintf(check_buffer, "%d transparent textures on solids", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d transparent textures on solids", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 275,
 			                "Show", &UI_Check_Textures::action_show_transparent,
@@ -4356,7 +4333,7 @@ check_result_e CHECK_Textures(int min_severity)
 			dialog->AddLine("No non-animating switch textures");
 		else
 		{
-			sprintf(check_buffer, "%d non-animating switch textures", sel.count_obj());
+			snprintf(check_buffer, sizeof(check_buffer), "%d non-animating switch textures", sel.count_obj());
 
 			dialog->AddLine(check_buffer, 1, 275,
 			                "Show", &UI_Check_Textures::action_show_dup_switch,
