@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2009-2016 Andrew Apted
+//  Copyright (C) 2009-2019 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -97,7 +97,40 @@ public:
 		return (int)num;
 	}
 
-	void CentreOfThings(int *cx, int *cy)
+	void Paste_BA_Message() const
+	{
+		size_t t = things.size();
+		size_t v = verts.size();
+		size_t s = sectors.size();
+		size_t l = lines.size();
+
+		if (s > 0)
+		{
+			const char *name = (s == 1) ? "sector" : "sectors";
+			BA_Message("pasted %d %s", s, name);
+		}
+		else if (l > 0)
+		{
+			const char *name = (l == 1) ? "linedef" : "linedefs";
+			BA_Message("pasted %d %s", l, name);
+		}
+		else if (t > 0)
+		{
+			const char *name = (t == 1) ? "thing" : "things";
+			BA_Message("pasted %d %s", t, name);
+		}
+		else if (v > 0)
+		{
+			const char *name = (v == 1) ? "vertex" : "vertices";
+			BA_Message("pasted %d %s", v, name);
+		}
+		else
+		{
+			BA_Message("pasted something");
+		}
+	}
+
+	void CentreOfThings(double *cx, double *cy)
 	{
 		*cx = *cy = 0;
 
@@ -109,39 +142,18 @@ public:
 
 		for (unsigned int i = 0 ; i < things.size() ; i++)
 		{
-			sum_x += things[i]->x;
-			sum_y += things[i]->y;
+			sum_x += things[i]->x();
+			sum_y += things[i]->y();
 		}
 
 		sum_x /= (double)things.size();
 		sum_y /= (double)things.size();
 
-		*cx = I_ROUND(sum_x);
-		*cy = I_ROUND(sum_y);
-
-#if 0
-		// find closest thing to this point
-		double best_dist = 9e9;
-
-		for (unsigned int k = 0 ; k < things.size() ; k++)
-		{
-			double dx = things[k]->x - sum_x;
-			double dy = things[k]->y - sum_y;
-
-			double dist = dx*dx + dy*dy;
-
-			if (dist < best_dist)
-			{
-				*cx = things[k]->x;
-				*cy = things[k]->y;
-
-				best_dist = dist;
-			}
-		}
-#endif
+		*cx = sum_x;
+		*cy = sum_y;
 	}
 
-	void CentreOfVertices(int *cx, int *cy)
+	void CentreOfVertices(double *cx, double *cy)
 	{
 		*cx = *cy = 0;
 
@@ -153,36 +165,15 @@ public:
 
 		for (unsigned int i = 0 ; i < verts.size() ; i++)
 		{
-			sum_x += verts[i]->x;
-			sum_y += verts[i]->y;
+			sum_x += verts[i]->x();
+			sum_y += verts[i]->y();
 		}
 
 		sum_x /= (double)verts.size();
 		sum_y /= (double)verts.size();
 
-		*cx = I_ROUND(sum_x);
-		*cy = I_ROUND(sum_y);
-
-#if 0
-		// find closest vertex to this point
-		double best_dist = 9e9;
-
-		for (unsigned int k = 0 ; k < verts.size() ; k++)
-		{
-			double dx = verts[k]->x - sum_x;
-			double dy = verts[k]->y - sum_y;
-
-			double dist = dx*dx + dy*dy;
-
-			if (dist < best_dist)
-			{
-				*cx = verts[k]->x;
-				*cy = verts[k]->y;
-
-				best_dist = dist;
-			}
-		}
-#endif
+		*cx = sum_x;
+		*cy = sum_y;
 	}
 
 	bool HasSectorRefs(int s1, int s2)
@@ -329,6 +320,72 @@ void Clipboard_NotifyChange(obj_type_e type, int objnum, int field)
 }
 
 
+//----------------------------------------------------------------------
+//  Texture Clipboard
+//----------------------------------------------------------------------
+
+namespace tex_clipboard
+{
+	std::string tex;
+	std::string flat;
+
+	int thing;
+};
+
+
+void Texboard_Clear()
+{
+	tex_clipboard::tex.clear();
+	tex_clipboard::flat.clear();
+	tex_clipboard::thing = 0;
+}
+
+int Texboard_GetTexNum()
+{
+	if (tex_clipboard::tex.empty())
+		tex_clipboard::tex = default_wall_tex;
+
+	return BA_InternaliseString(tex_clipboard::tex.c_str());
+}
+
+int Texboard_GetFlatNum()
+{
+	if (tex_clipboard::flat.empty())
+		tex_clipboard::flat = default_floor_tex;
+
+	return BA_InternaliseString(tex_clipboard::flat.c_str());
+}
+
+int Texboard_GetThing()
+{
+	if (tex_clipboard::thing == 0)
+		return default_thing;
+
+	return tex_clipboard::thing;
+}
+
+void Texboard_SetTex(const char *new_tex)
+{
+	tex_clipboard::tex = new_tex;
+
+	if (Features.mix_textures_flats)
+		tex_clipboard::flat = new_tex;
+}
+
+void Texboard_SetFlat(const char *new_flat)
+{
+	tex_clipboard::flat = new_flat;
+
+	if (Features.mix_textures_flats)
+		tex_clipboard::tex = new_flat;
+}
+
+void Texboard_SetThing(int new_id)
+{
+	tex_clipboard::thing = new_id;
+}
+
+
 //------------------------------------------------------------------------
 
 static void CopyGroupOfObjects(selection_c *list)
@@ -338,8 +395,6 @@ static void CopyGroupOfObjects(selection_c *list)
 
 	bool is_sectors = (list->what_type() == OBJ_SECTORS) ? true : false;
 
-	selection_iterator_c it;
-
 	selection_c vert_sel(OBJ_VERTICES);
 	selection_c side_sel(OBJ_SIDEDEFS);
 	selection_c line_sel(OBJ_LINEDEFS);
@@ -348,7 +403,7 @@ static void CopyGroupOfObjects(selection_c *list)
 	ConvertSelection(&line_sel, &vert_sel);
 
 	// determine needed sidedefs
-	for (line_sel.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(line_sel) ; !it.done() ; it.next())
 	{
 		LineDef *L = LineDefs[*it];
 
@@ -363,7 +418,7 @@ static void CopyGroupOfObjects(selection_c *list)
 	std::map<int, int> side_map;
 
 
-	for (vert_sel.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(vert_sel) ; !it.done() ; it.next())
 	{
 		vert_map[*it] = (int)clip_board->verts.size();
 
@@ -374,7 +429,7 @@ static void CopyGroupOfObjects(selection_c *list)
 
 	if (is_sectors)
 	{
-		for (list->begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(list) ; !it.done() ; it.next())
 		{
 			sector_map[*it] = (int)clip_board->sectors.size();
 
@@ -384,7 +439,7 @@ static void CopyGroupOfObjects(selection_c *list)
 		}
 	}
 
-	for (side_sel.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(side_sel) ; !it.done() ; it.next())
 	{
 		side_map[*it] = (int)clip_board->sides.size();
 
@@ -400,7 +455,7 @@ static void CopyGroupOfObjects(selection_c *list)
 		}
 	}
 
-	for (line_sel.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(line_sel) ; !it.done() ; it.next())
 	{
 		LineDef * L = new LineDef;
 		L->RawCopy(LineDefs[*it]);
@@ -434,7 +489,7 @@ static void CopyGroupOfObjects(selection_c *list)
 
 		ConvertSelection(list, &thing_sel);
 
-		for (thing_sel.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(thing_sel) ; !it.done() ; it.next())
 		{
 			Thing * T = new Thing;
 			T->RawCopy(Things[*it]);
@@ -446,22 +501,22 @@ static void CopyGroupOfObjects(selection_c *list)
 
 static bool Clipboard_DoCopy()
 {
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 		return false;
 
 	// create storage for the copied objects
 	if (clip_board)
 		delete clip_board;
 
+	bool result = true;
+
 	clip_board = new clipboard_data_c(edit.mode);
 
 	switch (edit.mode)
 	{
 		case OBJ_THINGS:
-			for (list.begin(&it) ; !it.at_end() ; ++it)
+			for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 			{
 				Thing * T = new Thing;
 				T->RawCopy(Things[*it]);
@@ -470,7 +525,7 @@ static bool Clipboard_DoCopy()
 			break;
 
 		case OBJ_VERTICES:
-			for (list.begin(&it) ; !it.at_end() ; ++it)
+			for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 			{
 				Vertex * V = new Vertex;
 				V->RawCopy(Vertices[*it]);
@@ -480,23 +535,32 @@ static bool Clipboard_DoCopy()
 
 		case OBJ_LINEDEFS:
 		case OBJ_SECTORS:
-			CopyGroupOfObjects(&list);
+			CopyGroupOfObjects(edit.Selected);
 			break;
 
 		default:
-			return false;
+			result = false;
+			break;
 	}
 
-	return true;
+	int total = edit.Selected->count_obj();
+	if (total == 1)
+		Status_Set("copied %s #%d", NameForObjectType(edit.Selected->what_type()), edit.Selected->find_first());
+	else
+		Status_Set("copied %d %s", total, NameForObjectType(edit.Selected->what_type(), true /* plural */));
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
+
+	return result;
 }
 
 
 //------------------------------------------------------------------------
 
-
-static void PasteGroupOfObjects(int pos_x, int pos_y)
+static void PasteGroupOfObjects(double pos_x, double pos_y)
 {
-	int cx, cy;
+	double cx, cy;
 	clip_board->CentreOfVertices(&cx, &cy);
 
 	// these hold the mapping from clipboard index --> real index
@@ -515,8 +579,8 @@ static void PasteGroupOfObjects(int pos_x, int pos_y)
 
 		V->RawCopy(clip_board->verts[i]);
 
-		V->x += pos_x - cx;
-		V->y += pos_y - cy;
+		V->SetRawX(V->x() + pos_x - cx);
+		V->SetRawY(V->y() + pos_y - cy);
 	}
 
 	for (i = 0 ; i < clip_board->sectors.size() ; i++)
@@ -598,8 +662,8 @@ static void PasteGroupOfObjects(int pos_x, int pos_y)
 
 		T->RawCopy(clip_board->things[i]);
 
-		T->x += pos_x - cx;
-		T->y += pos_y - cy;
+		T->SetRawX(T->x() + pos_x - cx);
+		T->SetRawY(T->y() + pos_y - cy);
 	}
 }
 
@@ -674,13 +738,13 @@ static bool Clipboard_DoPaste()
 		return false;
 
 	// figure out where to put stuff
-	int pos_x = edit.map_x;
-	int pos_y = edit.map_y;
+	double pos_x = edit.map_x;
+	double pos_y = edit.map_y;
 
 	if (! edit.pointer_in_window)
 	{
-		pos_x = I_ROUND(grid.orig_x);
-		pos_y = I_ROUND(grid.orig_y);
+		pos_x = grid.orig_x;
+		pos_y = grid.orig_y;
 	}
 
 	// honor the grid snapping setting
@@ -688,8 +752,7 @@ static bool Clipboard_DoPaste()
 	pos_y = grid.SnapY(pos_y);
 
 	BA_Begin();
-
-	BA_Message("pasted %d objects", clip_board->TotalSize());
+	clip_board->Paste_BA_Message();
 
 	clip_doing_paste = true;
 
@@ -697,7 +760,7 @@ static bool Clipboard_DoPaste()
 	{
 		case OBJ_THINGS:
 		{
-			int cx, cy;
+			double cx, cy;
 			clip_board->CentreOfThings(&cx, &cy);
 
 			for (unsigned int i = 0 ; i < clip_board->things.size() ; i++)
@@ -707,8 +770,8 @@ static bool Clipboard_DoPaste()
 
 				T->RawCopy(clip_board->things[i]);
 
-				T->x += pos_x - cx;
-				T->y += pos_y - cy;
+				T->SetRawX(T->x() + pos_x - cx);
+				T->SetRawY(T->y() + pos_y - cy);
 
 				recent_things.insert_number(T->type);
 			}
@@ -717,7 +780,7 @@ static bool Clipboard_DoPaste()
 
 		case OBJ_VERTICES:
 		{
-			int cx, cy;
+			double cx, cy;
 			clip_board->CentreOfVertices(&cx, &cy);
 
 			for (i = 0 ; i < clip_board->verts.size() ; i++)
@@ -727,8 +790,8 @@ static bool Clipboard_DoPaste()
 
 				V->RawCopy(clip_board->verts[i]);
 
-				V->x += pos_x - cx;
-				V->y += pos_y - cy;
+				V->SetRawX(V->x() + pos_x - cx);
+				V->SetRawY(V->y() + pos_y - cy);
 			}
 			break;
 		}
@@ -779,9 +842,9 @@ void CMD_Clipboard_Cut()
 	if (main_win->ClipboardOp('x'))
 		return;
 
-	if (edit.render3d)
+	if (edit.render3d && edit.mode != OBJ_THINGS)
 	{
-		Render3D_ClipboardOp('x');
+		Render3D_CB_Cut();
 		return;
 	}
 
@@ -800,9 +863,9 @@ void CMD_Clipboard_Copy()
 	if (main_win->ClipboardOp('c'))
 		return;
 
-	if (edit.render3d)
+	if (edit.render3d && edit.mode != OBJ_THINGS)
 	{
-		Render3D_ClipboardOp('c');
+		Render3D_CB_Copy();
 		return;
 	}
 
@@ -819,9 +882,9 @@ void CMD_Clipboard_Paste()
 	if (main_win->ClipboardOp('v'))
 		return;
 
-	if (edit.render3d)
+	if (edit.render3d && edit.mode != OBJ_THINGS)
 	{
-		Render3D_ClipboardOp('v');
+		Render3D_CB_Paste();
 		return;
 	}
 
@@ -973,9 +1036,7 @@ void DuddedSectors(selection_c *verts, selection_c *lines, selection_c *result)
 
 static void FixupLineDefs(selection_c *lines, selection_c *sectors)
 {
-	selection_iterator_c it;
-
-	for (lines->begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(lines) ; !it.done() ; it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
@@ -1061,7 +1122,6 @@ static bool DeleteVertex_MergeLineDefs(int v_num)
 
 
 	BA_Begin();
-
 	BA_Message("deleted vertex #%d\n", v_num);
 
 	if (L1->start == v_num)
@@ -1181,15 +1241,8 @@ void CMD_Delete()
 	if (main_win->ClipboardOp('d'))
 		return;
 
-	if (edit.render3d)
-	{
-		Render3D_ClipboardOp('d');
-		return;
-	}
-
-	selection_c list;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("Nothing to delete");
 		return;
@@ -1199,9 +1252,9 @@ void CMD_Delete()
 
 	// special case for a single vertex connected to two linedefs,
 	// we delete the vertex but merge the two linedefs.
-	if (edit.mode == OBJ_VERTICES && list.count_obj() == 1)
+	if (edit.mode == OBJ_VERTICES && edit.Selected->count_obj() == 1)
 	{
-		int v_num = list.find_first();
+		int v_num = edit.Selected->find_first();
 		SYS_ASSERT(v_num >= 0);
 
 		if (Vertex_HowManyLineDefs(v_num) == 2)
@@ -1214,17 +1267,19 @@ void CMD_Delete()
 	}
 
 	BA_Begin();
-	BA_MessageForSel("deleted", &list);
+	BA_MessageForSel("deleted", edit.Selected);
 
-	DeleteObjects_WithUnused(&list, keep, false /* keep_verts */, keep);
+	DeleteObjects_WithUnused(edit.Selected, keep, false /* keep_verts */, keep);
 
 	BA_End();
 
 success:
 	Editor_ClearAction();
 
+	// always clear the selection (deleting objects invalidates it)
 	Selection_Clear();
 
+	r_view.current_hl.clear();
 	edit.highlight.clear();
 	edit.split_line.clear();
 
