@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -38,17 +38,17 @@
 extern int vertex_radius(double scale);
 
 
-float ApproxDistToLineDef(const LineDef * L, float x, float y)
+double ApproxDistToLineDef(const LineDef * L, double x, double y)
 {
-	int x1 = L->Start()->x;
-	int y1 = L->Start()->y;
-	int x2 = L->End()->x;
-	int y2 = L->End()->y;
+	double x1 = L->Start()->x();
+	double y1 = L->Start()->y();
+	double x2 = L->End()->x();
+	double y2 = L->End()->y();
 
-	int dx = x2 - x1;
-	int dy = y2 - y1;
+	double dx = x2 - x1;
+	double dy = y2 - y1;
 
-	if (abs(dx) > abs(dy))
+	if (fabs(dx) > fabs(dy))
 	{
 		// The linedef is rather horizontal
 
@@ -66,7 +66,7 @@ float ApproxDistToLineDef(const LineDef * L, float x, float y)
 
 		// case 3: x is in-between (and not equal to) both vertices
 		//         hence use slope formula to get intersection point
-		float y3 = y1 + (x - x1) * (float)dy / (float)dx;
+		double y3 = y1 + (x - x1) * dy / dx;
 
 		return fabs(y3 - y);
 	}
@@ -82,35 +82,39 @@ float ApproxDistToLineDef(const LineDef * L, float x, float y)
 			return hypot(x - (dy > 0 ? x2 : x1),
 						 y - (dy > 0 ? y2 : y1));
 
-		float x3 = x1 + (y - y1) * (float)dx / (float)dy;
+		double x3 = x1 + (y - y1) * dx / dy;
 
 		return fabs(x3 - x);
 	}
 }
 
 
-int ClosestLine_CastingHoriz(int x, int y, int *side)
+int ClosestLine_CastingHoriz(double x, double y, int *side)
 {
-	int   best_match = -1;
-	float best_dist  = 9e9;
+	int    best_match = -1;
+	double best_dist  = 9e9;
+
+	// most lines have integral X coords, so offset slightly to
+	// avoid hitting vertices.
+	y += 0.04;
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
-		int ly1 = LineDefs[n]->Start()->y;
-		int ly2 = LineDefs[n]->End()->y;
+		double ly1 = LineDefs[n]->Start()->y();
+		double ly2 = LineDefs[n]->End()->y();
 
 		// ignore purely horizontal lines
 		if (ly1 == ly2)
 			continue;
 
 		// does the linedef cross the horizontal ray?
-		if (MIN(ly1, ly2) >= y + 1 || MAX(ly1, ly2) <= y)
+		if (MIN(ly1, ly2) >= y || MAX(ly1, ly2) <= y)
 			continue;
 
-		int lx1 = LineDefs[n]->Start()->x;
-		int lx2 = LineDefs[n]->End()->x;
+		double lx1 = LineDefs[n]->Start()->x();
+		double lx2 = LineDefs[n]->End()->x();
 
-		float dist = lx1 - (x + 0.5) + (lx2 - lx1) * (y + 0.5 - ly1) / (float)(ly2 - ly1);
+		double dist = lx1 - x + (lx2 - lx1) * (y - ly1) / (ly2 - ly1);
 
 		if (fabs(dist) < best_dist)
 		{
@@ -119,7 +123,7 @@ int ClosestLine_CastingHoriz(int x, int y, int *side)
 
 			if (side)
 			{
-				if (best_dist < 0.2)
+				if (best_dist < 0.01)
 					*side = 0;  // on the line
 				else if ( (ly1 > ly2) == (dist > 0))
 					*side = 1;  // right side
@@ -133,28 +137,32 @@ int ClosestLine_CastingHoriz(int x, int y, int *side)
 }
 
 
-int ClosestLine_CastingVert(int x, int y, int *side)
+int ClosestLine_CastingVert(double x, double y, int *side)
 {
-	int   best_match = -1;
-	float best_dist  = 9e9;
+	int    best_match = -1;
+	double best_dist  = 9e9;
+
+	// most lines have integral X coords, so offset slightly to
+	// avoid hitting vertices.
+	x += 0.04;
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
-		int lx1 = LineDefs[n]->Start()->x;
-		int lx2 = LineDefs[n]->End()->x;
+		double lx1 = LineDefs[n]->Start()->x();
+		double lx2 = LineDefs[n]->End()->x();
 
 		// ignore purely vertical lines
 		if (lx1 == lx2)
 			continue;
 
 		// does the linedef cross the vertical ray?
-		if (MIN(lx1, lx2) >= x + 1 || MAX(lx1, lx2) <= x)
+		if (MIN(lx1, lx2) >= x || MAX(lx1, lx2) <= x)
 			continue;
 
-		int ly1 = LineDefs[n]->Start()->y;
-		int ly2 = LineDefs[n]->End()->y;
+		double ly1 = LineDefs[n]->Start()->y();
+		double ly2 = LineDefs[n]->End()->y();
 
-		float dist = ly1 - (y + 0.5) + (ly2 - ly1) * (x + 0.5 - lx1) / (float)(lx2 - lx1);
+		double dist = ly1 - y + (ly2 - ly1) * (x - lx1) / (lx2 - lx1);
 
 		if (fabs(dist) < best_dist)
 		{
@@ -163,7 +171,7 @@ int ClosestLine_CastingVert(int x, int y, int *side)
 
 			if (side)
 			{
-				if (best_dist < 0.2)
+				if (best_dist < 0.01)
 					*side = 0;  // on the line
 				else if ( (lx1 > lx2) == (dist < 0))
 					*side = 1;  // right side
@@ -177,10 +185,10 @@ int ClosestLine_CastingVert(int x, int y, int *side)
 }
 
 
-int ClosestLine_CastAtAngle(int x, int y, float radians)
+int ClosestLine_CastAtAngle(double x, double y, float radians)
 {
-	int   best_match = -1;
-	float best_dist  = 9e9;
+	int    best_match = -1;
+	double best_dist  = 9e9;
 
 	double x2 = x + 256 * cos(radians);
 	double y2 = y + 256 * sin(radians);
@@ -189,17 +197,17 @@ int ClosestLine_CastAtAngle(int x, int y, float radians)
 	{
 		const LineDef *L = LineDefs[n];
 
-		float a = PerpDist(L->Start()->x, L->Start()->y,  x, y, x2, y2);
-		float b = PerpDist(L->  End()->x, L->  End()->y,  x, y, x2, y2);
+		double a = PerpDist(L->Start()->x(), L->Start()->y(),  x, y, x2, y2);
+		double b = PerpDist(L->  End()->x(), L->  End()->y(),  x, y, x2, y2);
 
 		// completely on one side of the vector?
 		if (a > 0 && b > 0) continue;
 		if (a < 0 && b < 0) continue;
 
-		float c = AlongDist(L->Start()->x, L->Start()->y,  x, y, x2, y2);
-		float d = AlongDist(L->  End()->x, L->  End()->y,  x, y, x2, y2);
+		double c = AlongDist(L->Start()->x(), L->Start()->y(),  x, y, x2, y2);
+		double d = AlongDist(L->  End()->x(), L->  End()->y(),  x, y, x2, y2);
 
-		float dist;
+		double dist;
 
 		if (fabs(a) < 1 && fabs(b) < 1)
 			dist = MIN(c, d);
@@ -209,7 +217,7 @@ int ClosestLine_CastAtAngle(int x, int y, float radians)
 			dist = d;
 		else
 		{
-			float factor = a / (a - b);
+			double factor = a / (a - b);
 			dist = c * (1 - factor) + d * factor;
 		}
 
@@ -227,22 +235,26 @@ int ClosestLine_CastAtAngle(int x, int y, float radians)
 }
 
 
-bool PointOutsideOfMap(int x, int y)
+bool PointOutsideOfMap(double x, double y)
 {
 	// this keeps track of directions tested
 	int dirs = 0;
 
+	// most end-points will be integral, so look in-between
+	double x2 = x + 0.04;
+	double y2 = y + 0.04;
+
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
-		int lx1 = LineDefs[n]->Start()->x;
-		int ly1 = LineDefs[n]->Start()->y;
-		int lx2 = LineDefs[n]->End()->x;
-		int ly2 = LineDefs[n]->End()->y;
+		double lx1 = LineDefs[n]->Start()->x();
+		double ly1 = LineDefs[n]->Start()->y();
+		double lx2 = LineDefs[n]->End()->x();
+		double ly2 = LineDefs[n]->End()->y();
 
 		// does the linedef cross the horizontal ray?
-		if (MIN(ly1, ly2) <= y && MAX(ly1, ly2) >= y + 1)
+		if (MIN(ly1, ly2) < y2 && MAX(ly1, ly2) > y2)
 		{
-			float dist = lx1 - (x + 0.5) + (lx2 - lx1) * (y + 0.5 - ly1) / (float)(ly2 - ly1);
+			double dist = lx1 - x + (lx2 - lx1) * (y2 - ly1) / (ly2 - ly1);
 
 			dirs |= (dist < 0) ? 1 : 2;
 
@@ -250,9 +262,9 @@ bool PointOutsideOfMap(int x, int y)
 		}
 
 		// does the linedef cross the vertical ray?
-		if (MIN(lx1, lx2) <= x && MAX(lx1, lx2) >= x + 1)
+		if (MIN(lx1, lx2) < x2 && MAX(lx1, lx2) > x2)
 		{
-			float dist = ly1 - (y - 0.5) + (ly2 - ly1) * (x + 0.5 - lx1) / (float)(lx2 - lx1);
+			double dist = ly1 - y + (ly2 - ly1) * (x2 - lx1) / (lx2 - lx1);
 
 			dirs |= (dist < 0) ? 4 : 8;
 
@@ -276,15 +288,16 @@ typedef struct
 
 	int * result_side;
 
-	int dx, dy;
+	double dx, dy;
 
 	// origin of casting line
-	float x, y;
+	double x, y;
 
-	bool is_horizontal;
+	// casting line is horizontal?
+	bool cast_horizontal;
 
-	int   best_match;
-	float best_dist;
+	int    best_match;
+	double best_dist;
 
 public:
 	void ComputeCastOrigin()
@@ -295,24 +308,28 @@ public:
 
 		const LineDef * L = LineDefs[ld];
 
-		dx = L->End()->x - L->Start()->x;
-		dy = L->End()->y - L->Start()->y;
+		dx = L->End()->x() - L->Start()->x();
+		dy = L->End()->y() - L->Start()->y();
 
-		is_horizontal = abs(dy) >= abs(dx);
+		cast_horizontal = fabs(dy) >= fabs(dx);
 
-		x = L->Start()->x + dx * 0.5;
-		y = L->Start()->y + dy * 0.5;
+		x = L->Start()->x() + dx * 0.5;
+		y = L->Start()->y() + dy * 0.5;
 
-		if (is_horizontal && (dy & 1) == 0 && abs(dy) > 0)
+		if (cast_horizontal && fabs(dy) > 0)
 		{
-			y = y + 0.5;
-			x = x + 0.5 * dx / (float)dy;
+			if (fabs(dy) >= 2.0) {
+				y = y + 0.4;
+				x = x + 0.4 * dx / dy;
+			}
 		}
 
-		if (!is_horizontal && (dx & 1) == 0 && abs(dx) > 0)
+		if (!cast_horizontal && fabs(dx) > 0)
 		{
-			x = x + 0.5;
-			y = y + 0.5 * dy / (float)dx;
+			if (fabs(dx) >= 2.0) {
+				x = x + 0.4;
+				y = y + 0.4 * dy / dx;
+			}
 		}
 	}
 
@@ -321,12 +338,12 @@ public:
 		if (ld == n)  // ignore input line
 			return;
 
-		int nx1 = LineDefs[n]->Start()->x;
-		int ny1 = LineDefs[n]->Start()->y;
-		int nx2 = LineDefs[n]->End()->x;
-		int ny2 = LineDefs[n]->End()->y;
+		double nx1 = LineDefs[n]->Start()->x();
+		double ny1 = LineDefs[n]->Start()->y();
+		double nx2 = LineDefs[n]->End()->x();
+		double ny2 = LineDefs[n]->End()->y();
 
-		if (is_horizontal)
+		if (cast_horizontal)
 		{
 			if (ny1 == ny2)
 				return;
@@ -334,12 +351,12 @@ public:
 			if (MIN(ny1, ny2) > y || MAX(ny1, ny2) < y)
 				return;
 
-			float dist = nx1 + (nx2 - nx1) * (y - ny1) / (float)(ny2 - ny1) - x;
+			double dist = nx1 + (nx2 - nx1) * (y - ny1) / (ny2 - ny1) - x;
 
 			if ( (dy < 0) == (ld_side > 0) )
 				dist = -dist;
 
-			if (dist > 0.2 && dist < best_dist)
+			if (dist > 0 && dist < best_dist)
 			{
 				best_match = n;
 				best_dist  = dist;
@@ -361,12 +378,12 @@ public:
 			if (MIN(nx1, nx2) > x || MAX(nx1, nx2) < x)
 				return;
 
-			float dist = ny1 + (ny2 - ny1) * (x - nx1) / (float)(nx2 - nx1) - y;
+			double dist = ny1 + (ny2 - ny1) * (x - nx1) / (nx2 - nx1) - y;
 
 			if ( (dx > 0) == (ld_side > 0) )
 				dist = -dist;
 
-			if (dist > 0.2 && dist < best_dist)
+			if (dist > 0 && dist < best_dist)
 			{
 				best_match = n;
 				best_dist  = dist;
@@ -413,6 +430,17 @@ public:
 		delete hi_child;
 	}
 
+private:
+	void Subdivide()
+	{
+		if (hi - lo <= FASTOPP_DIST)
+			return;
+
+		lo_child = new fastopp_node_c(lo, mid);
+		hi_child = new fastopp_node_c(mid, hi);
+	}
+
+public:
 	/* horizontal tree */
 
 	void AddLine_X(int ld, int x1, int x2)
@@ -438,13 +466,14 @@ public:
 	{
 		const LineDef *L = LineDefs[ld];
 
-		int x1 = MIN(L->Start()->x, L->End()->x);
-		int x2 = MAX(L->Start()->x, L->End()->x);
-
 		// can ignore purely vertical lines
-		if (x1 == x2) return;
+		if (L->IsVertical())
+			return;
 
-		AddLine_X(ld, x1, x2);
+		double x1 = MIN(L->Start()->x(), L->End()->x());
+		double x2 = MAX(L->Start()->x(), L->End()->x());
+
+		AddLine_X(ld, (int)floor(x1), (int)ceil(x2));
 	}
 
 	/* vertical tree */
@@ -472,16 +501,17 @@ public:
 	{
 		const LineDef *L = LineDefs[ld];
 
-		int y1 = MIN(L->Start()->y, L->End()->y);
-		int y2 = MAX(L->Start()->y, L->End()->y);
-
 		// can ignore purely horizonal lines
-		if (y1 == y2) return;
+		if (L->IsHorizontal())
+			return;
 
-		AddLine_Y(ld, y1, y2);
+		double y1 = MIN(L->Start()->y(), L->End()->y());
+		double y2 = MAX(L->Start()->y(), L->End()->y());
+
+		AddLine_Y(ld, (int)floor(y1), (int)ceil(y2));
 	}
 
-	void Process(opp_test_state_t& test, float coord)
+	void Process(opp_test_state_t& test, double coord)
 	{
 		for (unsigned int k = 0 ; k < lines.size() ; k++)
 			test.ProcessLine(lines[k]);
@@ -495,21 +525,11 @@ public:
 		//
 		// hence we never need to recurse down BOTH sides here.
 
-		if (coord < mid)
+		if (coord < (double)mid)
 			lo_child->Process(test, coord);
 		else
 			hi_child->Process(test, coord);
 	}
-
-private:
-	void Subdivide()
-	{
-		if (hi - lo <= FASTOPP_DIST)
-			return;
-
-		lo_child = new fastopp_node_c(lo, mid);
-		hi_child = new fastopp_node_c(mid, hi);
-	}
 };
 
 
@@ -565,7 +585,7 @@ int OppositeLineDef(int ld, int ld_side, int *result_side, bitvec_c *ignore_line
 
 		SYS_ASSERT(ignore_lines == NULL);
 
-		if (test.is_horizontal)
+		if (test.cast_horizontal)
 			fastopp_Y_tree->Process(test, test.y);
 		else
 			fastopp_X_tree->Process(test, test.x);
@@ -602,12 +622,12 @@ int OppositeSector(int ld, int ld_side)
 
 
 // result: -1 for back, +1 for front, 0 for _exactly_on_ the line
-int PointOnLineSide(int x, int y, int lx1, int ly1, int lx2, int ly2)
+int PointOnLineSide(double x, double y, double lx1, double ly1, double lx2, double ly2)
 {
 	x   -= lx1; y   -= ly1;
 	lx2 -= lx1; ly2 -= ly1;
 
-	int tmp = (x * ly2 - y * lx2);
+	double tmp = (x * ly2 - y * lx2);
 
 	return (tmp < 0) ? -1 : (tmp > 0) ? +1 : 0;
 }
@@ -648,34 +668,34 @@ public:
 //
 // determine which linedef is under the pointer
 //
-static Objid NearestLineDef(float x, float y)
+static Objid NearestLineDef(double x, double y)
 {
 	// slack in map units
-	float mapslack = 2 + 16.0f / grid.Scale;
+	double mapslack = 2.5 + 16.0f / grid.Scale;
 
-	int lx = floor(x - mapslack);
-	int ly = floor(y - mapslack);
-	int hx =  ceil(x + mapslack);
-	int hy =  ceil(y + mapslack);
+	double lx = x - mapslack;
+	double ly = y - mapslack;
+	double hx = x + mapslack;
+	double hy = y + mapslack;
 
 	int    best = -1;
 	double best_dist = 9e9;
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
-		int x1 = LineDefs[n]->Start()->x;
-		int y1 = LineDefs[n]->Start()->y;
-		int x2 = LineDefs[n]->End()->x;
-		int y2 = LineDefs[n]->End()->y;
+		double x1 = LineDefs[n]->Start()->x();
+		double y1 = LineDefs[n]->Start()->y();
+		double x2 = LineDefs[n]->End()->x();
+		double y2 = LineDefs[n]->End()->y();
 
 		// Skip all lines of which all points are more than <mapslack>
-		// units away from (x,y). In a typical level, this test will
+		// units away from (x,y).  In a typical level, this test will
 		// filter out all the linedefs but a handful.
 		if (MAX(x1,x2) < lx || MIN(x1,x2) > hx ||
 		    MAX(y1,y2) < ly || MIN(y1,y2) > hy)
 			continue;
 
-		float dist = ApproxDistToLineDef(LineDefs[n], x, y);
+		double dist = ApproxDistToLineDef(LineDefs[n], x, y);
 
 		if (dist > mapslack)
 			continue;
@@ -701,19 +721,21 @@ static Objid NearestLineDef(float x, float y)
 // determine which linedef would be split if a new vertex were
 // added at the given coordinates.
 //
-static Objid NearestSplitLine(int x, int y, int ignore_vert)
+static Objid NearestSplitLine(double x, double y, int ignore_vert)
 {
 	// slack in map units
-	int mapslack = 1 + (int)ceil(8.0f / grid.Scale);
+	double mapslack = 1.5 + ceil(8.0f / grid.Scale);
 
-	int lx = x - mapslack;
-	int ly = y - mapslack;
-	int hx = x + mapslack;
-	int hy = y + mapslack;
+	double lx = x - mapslack;
+	double ly = y - mapslack;
+	double hx = x + mapslack;
+	double hy = y + mapslack;
 
 	int    best = -1;
 	double best_dist = 9e9;
 
+	double too_small = (Level_format == MAPF_UDMF) ? 0.2 : 4.0;
+
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
 		LineDef *L = LineDefs[n];
@@ -721,22 +743,21 @@ static Objid NearestSplitLine(int x, int y, int ignore_vert)
 		if (L->start == ignore_vert || L->end == ignore_vert)
 			continue;
 
-		int x1 = L->Start()->x;
-		int y1 = L->Start()->y;
-		int x2 = L->End()->x;
-		int y2 = L->End()->y;
+		double x1 = L->Start()->x();
+		double y1 = L->Start()->y();
+		double x2 = L->End()->x();
+		double y2 = L->End()->y();
 
 		if (MAX(x1,x2) < lx || MIN(x1,x2) > hx ||
 		    MAX(y1,y2) < ly || MIN(y1,y2) > hy)
 			continue;
 
-		// skip linedef if point matches a vertex
+		// skip linedef if given point matches a vertex
 		if (x == x1 && y == y1) continue;
 		if (x == x2 && y == y2) continue;
 
 		// skip linedef if too small to split
-		if (abs(L->Start()->x - L->End()->x) < 4 &&
-			abs(L->Start()->y - L->End()->y) < 4)
+		if (fabs(x2 - x1) < too_small && fabs(y2 - y1) < too_small)
 			continue;
 
 		double dist = ApproxDistToLineDef(L, x, y);
@@ -762,7 +783,7 @@ static Objid NearestSplitLine(int x, int y, int ignore_vert)
 //
 //  determine which sector is under the pointer
 //
-static Objid NearestSector(int x, int y)
+static Objid NearestSector(double x, double y)
 {
 	/* hack, hack...  I look for the first LineDef crossing
 	   an horizontal half-line drawn from the cursor */
@@ -806,24 +827,24 @@ static Objid NearestSector(int x, int y)
 //
 // determine which thing is under the mouse pointer
 //
-static Objid NearestThing(float x, float y)
+static Objid NearestThing(double x, double y)
 {
-	float mapslack = 1 + 16.0f / grid.Scale;
+	double mapslack = 1 + 16.0f / grid.Scale;
 
-	int max_radius = MAX_RADIUS + ceil(mapslack);
+	double max_radius = MAX_RADIUS + ceil(mapslack);
 
-	int lx = x - max_radius;
-	int ly = y - max_radius;
-	int hx = x + max_radius;
-	int hy = y + max_radius;
+	double lx = x - max_radius;
+	double ly = y - max_radius;
+	double hx = x + max_radius;
+	double hy = y + max_radius;
 
 	int best = -1;
 	thing_comparer_t best_comp;
 
 	for (int n = 0 ; n < NumThings ; n++)
 	{
-		int tx = Things[n]->x;
-		int ty = Things[n]->y;
+		double tx = Things[n]->x();
+		double ty = Things[n]->y();
 
 		// filter out things that are outside the search bbox.
 		// this search box is enlarged by MAX_RADIUS.
@@ -833,7 +854,7 @@ static Objid NearestThing(float x, float y)
 		const thingtype_t *info = M_GetThingType(Things[n]->type);
 
 		// more accurate bbox test using the real radius
-		int r = info->radius + mapslack;
+		double r = info->radius + mapslack;
 
 		if (x < tx - r - mapslack || x > tx + r + mapslack ||
 			y < ty - r - mapslack || y > ty + r + mapslack)
@@ -842,7 +863,7 @@ static Objid NearestThing(float x, float y)
 		thing_comparer_t th_comp;
 
 		th_comp.distance = hypot(x - tx, y - ty);
-		th_comp.radius   = r;
+		th_comp.radius   = (int)r;
 		th_comp.inside   = (x > tx - r && x < tx + r && y > ty - r && y < ty + r);
 
 		if (best < 0 || th_comp <= best_comp)
@@ -863,28 +884,28 @@ static Objid NearestThing(float x, float y)
 //
 // determine which vertex is under the pointer
 //
-static Objid NearestVertex(float x, float y)
+static Objid NearestVertex(double x, double y)
 {
 	const int screen_pix = vertex_radius(grid.Scale);
 
-	float mapslack = 1 + (4 + screen_pix) / grid.Scale;
+	double mapslack = 1 + (4 + screen_pix) / grid.Scale;
 
 	// workaround for overly zealous highlighting when zoomed in far
 	if (grid.Scale >= 15.0) mapslack *= 0.7;
 	if (grid.Scale >= 31.0) mapslack *= 0.5;
 
-	int lx = floor(x - mapslack);
-	int ly = floor(y - mapslack);
-	int hx =  ceil(x + mapslack);
-	int hy =  ceil(y + mapslack);
+	double lx = x - mapslack - 0.5;
+	double ly = y - mapslack - 0.5;
+	double hx = x + mapslack + 0.5;
+	double hy = y + mapslack + 0.5;
 
 	int    best = -1;
 	double best_dist = 9e9;
 
 	for (int n = 0 ; n < NumVertices ; n++)
 	{
-		int vx = Vertices[n]->x;
-		int vy = Vertices[n]->y;
+		double vx = Vertices[n]->x();
+		double vy = Vertices[n]->y();
 
 		// filter out vertices that are outside the search bbox
 		if (vx < lx || vx > hx || vy < ly || vy > hy)
@@ -917,7 +938,7 @@ static Objid NearestVertex(float x, float y)
 //  coordinates.  when several objects are close, the smallest
 //  is chosen.
 //
-void GetNearObject(Objid& o, obj_type_e objtype, float x, float y)
+void GetNearObject(Objid& o, obj_type_e objtype, double x, double y)
 {
 	switch (objtype)
 	{
@@ -952,44 +973,110 @@ void GetNearObject(Objid& o, obj_type_e objtype, float x, float y)
 }
 
 
-void GetSplitLineDef(Objid& o, int x, int y, int drag_vert)
+void FindSplitLine(Objid& out, double& out_x, double& out_y,
+				   double ptr_x, double ptr_y, int ignore_vert)
 {
-	o = NearestSplitLine(x, y, drag_vert);
+	out_x = out_y = 0;
 
-	// don't highlight the line if the new vertex would snap onto
-	// the same coordinate as the start or end of the linedef.
-	// [ I tried a bbox test here, but it's bad for axis-aligned lines ]
+	out = NearestSplitLine(ptr_x, ptr_y, ignore_vert);
 
-	if (o.valid() && grid.snap)
+	if (! out.valid())
+		return;
+
+	const LineDef * L = LineDefs[out.num];
+
+	double x1 = L->Start()->x();
+	double y1 = L->Start()->y();
+	double x2 = L->End()->x();
+	double y2 = L->End()->y();
+
+	double len = hypot(x2 - x1, y2 - y1);
+
+	if (grid.ratio > 0 && edit.action == ACT_DRAW_LINE)
 	{
-		int snap_x = grid.SnapX(x);
-		int snap_y = grid.SnapY(y);
+		Vertex *V = Vertices[edit.draw_from.num];
 
-		const LineDef * L = LineDefs[o.num];
+		// convert ratio into a vector, use it to intersect the linedef
+		double px1 = V->x();
+		double py1 = V->y();
 
-		if ( (L->Start()->x == snap_x && L->Start()->y == snap_y) ||
-			 (L->  End()->x == snap_x && L->  End()->y == snap_y) )
+		double px2 = ptr_x;
+		double py2 = ptr_y;
+
+		grid.RatioSnapXY(px2, py2, px1, py1);
+
+		if (fabs(px1 - px2) < 0.1 && fabs(py1 - py2) < 0.1)
 		{
-			o.clear();
+			out.clear();
+			return;
 		}
 
-		// also require snap coordinate be not TOO FAR from the line
-		double len = L->CalcLength();
+		// compute intersection point
+		double c = PerpDist(x1, y1,  px1, py1, px2, py2);
+		double d = PerpDist(x2, y2,  px1, py1, px2, py2);
 
-		double along = AlongDist(snap_x, snap_y, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
-		double  perp =  PerpDist(snap_x, snap_y, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
+		int c_side = (c < -0.02) ? -1 : (c > 0.02) ? +1 : 0;
+		int d_side = (d < -0.02) ? -1 : (d > 0.02) ? +1 : 0;
 
-		if (along <= 0 || along >= len || fabs(perp) > len * 0.2)
+		if (c_side * d_side >= 0)
 		{
-			o.clear();
+			out.clear();
+			return;
 		}
+
+		c = c / (c - d);
+
+		out_x = x1 + c * (x2 - x1);
+		out_y = y1 + c * (y2 - y1);
+	}
+	else if (grid.snap)
+	{
+		// don't highlight the line if the new vertex would snap onto
+		// the same coordinate as the start or end of the linedef.
+		// [ I tried a bbox test here, but it was bad for axis-aligned lines ]
+
+		out_x = grid.ForceSnapX(ptr_x);
+		out_y = grid.ForceSnapY(ptr_y);
+
+		// snapped onto an end point?
+		if (L->TouchesCoord(TO_COORD(out_x), TO_COORD(out_y)))
+		{
+			out.clear();
+			return;
+		}
+
+		// require snap coordinate be not TOO FAR from the line
+		double perp = PerpDist(out_x, out_y, x1, y1, x2, y2);
+
+		if (fabs(perp) > len * 0.2)
+		{
+			out.clear();
+			return;
+		}
+	}
+	else
+	{
+		// in FREE mode, ensure split point is directly on the linedef
+		out_x = ptr_x;
+		out_y = ptr_y;
+
+		MoveCoordOntoLineDef(out.num, &out_x, &out_y);
+	}
+
+	// always ensure result is along the linedef (not off the ends)
+	double along = AlongDist(out_x, out_y, x1, y1, x2, y2);
+
+	if (along < 0.05 || along > len - 0.05)
+	{
+		out.clear();
+		return;
 	}
 }
 
 
-void GetSplitLineForDangler(Objid& o, int v_num)
+void FindSplitLineForDangler(Objid& out, int v_num)
 {
-	o = NearestSplitLine(Vertices[v_num]->x, Vertices[v_num]->y, v_num);
+	out = NearestSplitLine(Vertices[v_num]->x(), Vertices[v_num]->y(), v_num);
 }
 
 
@@ -1016,14 +1103,14 @@ void crossing_state_c::add_vert(int v, double dist)
 
 	pt.vert = v;
 	pt.ld   = -1;
-	pt.x    = Vertices[v]->x;
-	pt.y    = Vertices[v]->y;
+	pt.x    = Vertices[v]->x();
+	pt.y    = Vertices[v]->y();
 	pt.dist = dist;
 
 	points.push_back(pt);
 }
 
-void crossing_state_c::add_line(int ld, int new_x, int new_y, double dist)
+void crossing_state_c::add_line(int ld, double new_x, double new_y, double dist)
 {
 	cross_point_t pt;
 
@@ -1072,8 +1159,7 @@ void crossing_state_c::SplitAllLines()
 
 			Vertex *V = Vertices[points[i].vert];
 
-			V->x = points[i].x;
-			V->y = points[i].y;
+			V->SetRawXY(points[i].x, points[i].y);
 
 			SplitLineDefAtVertex(points[i].ld, points[i].vert);
 		}
@@ -1081,87 +1167,38 @@ void crossing_state_c::SplitAllLines()
 }
 
 
-#if 0
-static void find_closest_cross_handler(int x, int y, double dist, int v, int ld, void *data)
-{
-	cross_state_t *cross = (cross_state_t *)data;
-
-	if (dist >= cross->distance)
-		return;
-
-	cross->distance = dist;
-
-	if (v >= 0)
-	{
-		cross->vert = v;
-		cross->line = -1;
-
-		cross->x = x;
-		cross->y = y;
-
-	}
-	else
-	{
-		cross->vert = -1;
-		cross->line = ld;
-
-		cross->x = x;
-		cross->y = y;
-	}
-}
-
-
-bool FindClosestCrossPoint(int v1, int v2, cross_state_t *cross)
-{
-	SYS_ASSERT(v1 != v2);
-
-	cross->vert = -1;
-	cross->line = -1;
-	cross->distance = 9e9;
-
-	const Vertex *VA = Vertices[v1];
-	const Vertex *VB = Vertices[v2];
-
-	FindAllCrossPoints(VA->x, VA->y, v1, VB->x, VB->y, v2,
-					   find_closest_cross_handler, cross);
-
-	return (cross->vert >= 0) || (cross->line >= 0);
-}
-#endif
-
-
-#define CROSSING_EPSILON  0.8
-#define    ALONG_EPSILON  0.4
+#define CROSSING_EPSILON  0.2
+#define    ALONG_EPSILON  0.1
 
 
 static void FindCrossingLines(crossing_state_c& cross,
-						int x1, int y1, int possible_v1,
-						int x2, int y2, int possible_v2)
+						double x1, double y1, int possible_v1,
+						double x2, double y2, int possible_v2)
 {
 	// this could happen when two vertices are overlapping
-	if (x1 == x2 && y1 == y2)
+	if (fabs(x1 - x2) < ALONG_EPSILON && fabs(y1 - y2) < ALONG_EPSILON)
 		return;
 
-	// distances along the WHOLE original line
+	// distances along WHOLE original line for this segment
 	double along1 = AlongDist(x1, y1,  cross.start_x, cross.start_y, cross.end_x, cross.end_y);
 	double along2 = AlongDist(x2, y2,  cross.start_x, cross.start_y, cross.end_x, cross.end_y);
 
-	// bounding box
-	int bbox_x1 = MIN(x1, x2);
-	int bbox_y1 = MIN(y1, y2);
+	// bounding box of segment
+	double bbox_x1 = MIN(x1, x2) - 0.25;
+	double bbox_y1 = MIN(y1, y2) - 0.25;
 
-	int bbox_x2 = MAX(x1, x2);
-	int bbox_y2 = MAX(y1, y2);
+	double bbox_x2 = MAX(x1, x2) + 0.25;
+	double bbox_y2 = MAX(y1, y2) + 0.25;
 
 
 	for (int ld = 0 ; ld < NumLineDefs ; ld++)
 	{
 		const LineDef * L = LineDefs[ld];
 
-		int lx1 = L->Start()->x;
-		int ly1 = L->Start()->y;
-		int lx2 = L->End()->x;
-		int ly2 = L->End()->y;
+		double lx1 = L->Start()->x();
+		double ly1 = L->Start()->y();
+		double lx2 = L->End()->x();
+		double ly2 = L->End()->y();
 
 		// bbox test -- eliminate most lines from consideration
 		if (MAX(lx1,lx2) < bbox_x1 || MIN(lx1,lx2) > bbox_x2 ||
@@ -1170,21 +1207,14 @@ static void FindCrossingLines(crossing_state_c& cross,
 			continue;
 		}
 
-		if (L->isZeroLength())
-			continue;
-
-		// skip linedef if an end-point is one of the vertices already
-		// in the crossing state (including the very start or very end).
-		if (L->TouchesCoord(x1, y1) || L->TouchesCoord(x2, y2))
-			continue;
-
-		if (L->TouchesCoord(cross.start_x, cross.start_y) ||
-			L->TouchesCoord(cross.  end_x, cross.  end_y))
+		if (L->IsZeroLength())
 			continue;
 
 		if (cross.HasLine(ld))
 			continue;
 
+		// skip linedef if an end-point is one of the vertices already
+		// in the crossing state (including the very start or very end).
 		if (cross.HasVertex(L->start) || cross.HasVertex(L->end))
 			continue;
 
@@ -1203,20 +1233,12 @@ static void FindCrossingLines(crossing_state_c& cross,
 		// compute intersection point
 		double l_along = a / (a - b);
 
-		double ix = (double)lx1 + l_along * (lx2 - lx1);
-		double iy = (double)ly1 + l_along * (ly2 - ly1);
-
-		int new_x = I_ROUND(ix);
-		int new_y = I_ROUND(iy);
-
-		// ensure new vertex does not match the start or end points
-		if (new_x == x1 && new_y == y1) continue;
-		if (new_x == x2 && new_y == y2) continue;
-
-		if (L->TouchesCoord(new_x, new_y)) continue;
+		double new_x = lx1 + l_along * (lx2 - lx1);
+		double new_y = ly1 + l_along * (ly2 - ly1);
 
 		double along = AlongDist(new_x, new_y,  cross.start_x, cross.start_y, cross.end_x, cross.end_y);
 
+		// ensure new vertex lies within this segment (and not too close to ends)
 		if (along > along1 + ALONG_EPSILON && along < along2 - ALONG_EPSILON)
 		{
 			cross.add_line(ld, new_x, new_y, along);
@@ -1226,8 +1248,8 @@ static void FindCrossingLines(crossing_state_c& cross,
 
 
 void FindCrossingPoints(crossing_state_c& cross,
-						int x1, int y1, int possible_v1,
-						int x2, int y2, int possible_v2)
+						double x1, double y1, int possible_v1,
+						double x2, double y2, int possible_v2)
 {
 	cross.clear();
 
@@ -1243,15 +1265,15 @@ void FindCrossingPoints(crossing_state_c& cross,
 	close_dist = CLAMP(1.0, close_dist, 12.0);
 
 
-	int dx = x2 - x1;
-	int dy = y2 - y1;
-
-	// same coords?  (sanity check)
-	if (dx == 0 && dy == 0)
-		return;
+	double dx = x2 - x1;
+	double dy = y2 - y1;
 
 	double length = hypot(dx, dy);
 
+	// same coords?  (sanity check)
+	if (length < 0.01)
+		return;
+
 
 	/* must do all vertices FIRST */
 
@@ -1262,17 +1284,18 @@ void FindCrossingPoints(crossing_state_c& cross,
 
 		const Vertex * VC = Vertices[v];
 
-		// ignore vertices ar same coordinates as v1 or v2
-		if (VC->Matches(x1, y1) || VC->Matches(x2, y2))
+		// ignore vertices at same coordinates as v1 or v2
+		if (VC->Matches(TO_COORD(x1), TO_COORD(y1)) ||
+			VC->Matches(TO_COORD(x2), TO_COORD(y2)))
 			continue;
 
 		// is this vertex sitting on the line?
-		double perp = PerpDist(VC->x, VC->y, x1,y1, x2,y2);
+		double perp = PerpDist(VC->x(), VC->y(), x1,y1, x2,y2);
 
 		if (fabs(perp) > close_dist)
 			continue;
 
-		double along = AlongDist(VC->x, VC->y, x1,y1, x2,y2);
+		double along = AlongDist(VC->x(), VC->y(), x1,y1, x2,y2);
 
 		if (along > ALONG_EPSILON && along < length - ALONG_EPSILON)
 		{
@@ -1285,28 +1308,29 @@ void FindCrossingPoints(crossing_state_c& cross,
 
 	/* process each pair of adjacent vertices to find split lines */
 
-	int cur_x1 = x1;
-	int cur_y1 = y1;
-	int cur_v  = possible_v1;
+	double cur_x1 = x1;
+	double cur_y1 = y1;
+	int    cur_v  = possible_v1;
 
 	// grab number of points now, since we will adding split points
-	// (and we assume those new points get added at the end).
-	unsigned int num_verts = static_cast<unsigned>(cross.points.size());
+	// (at the end) and we only want vertices here.
+	size_t num_verts = cross.points.size();
 
-	for (unsigned int k = 0 ; k < num_verts ; k++)
+	for (size_t k = 0 ; k < num_verts ; k++)
 	{
-		cross_point_t& next_p = cross.points[k];
+		double next_x2 = cross.points[k].x;
+		double next_y2 = cross.points[k].y;
+		double next_v  = cross.points[k].vert;
 
 		FindCrossingLines(cross, cur_x1, cur_y1, cur_v,
-						  next_p.x, next_p.y, next_p.vert);
+						  next_x2, next_y2, next_v);
 
-		cur_x1 = next_p.x;
-		cur_y1 = next_p.y;
-		cur_v  = next_p.vert;
+		cur_x1 = next_x2;
+		cur_y1 = next_y2;
+		cur_v  = next_v;
 	}
 
-	FindCrossingLines(cross, cur_x1, cur_y1, cur_v,
-	                  x2, y2, possible_v2);
+	FindCrossingLines(cross, cur_x1, cur_y1, cur_v, x2, y2, possible_v2);
 
 	cross.Sort();
 }
