@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -59,29 +59,34 @@ bool LineDefAlreadyExists(int v1, int v2)
 // existing line.  By "overlap" I mean parallel and sitting on top
 // (this does NOT test for lines crossing each other).
 //
-bool LineDefWouldOverlap(int v1, int x2, int y2)
+bool LineDefWouldOverlap(int v1, double x2, double y2)
 {
-	int x1 = Vertices[v1]->x;
-	int y1 = Vertices[v1]->y;
+	double x1 = Vertices[v1]->x();
+	double y1 = Vertices[v1]->y();
 
 	for (int n = 0 ; n < NumLineDefs ; n++)
 	{
 		LineDef *L = LineDefs[n];
 
 		// zero-length lines should not exist, but don't get stroppy if they do
-		if (L->isZeroLength())
+		if (L->IsZeroLength())
 			continue;
 
+		double lx1 = L->Start()->x();
+		double ly1 = L->Start()->y();
+		double lx2 = L->End()->x();
+		double ly2 = L->End()->y();
+
 		double a, b;
 
-		a = PerpDist(x1, y1, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
-		b = PerpDist(x2, y2, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
+		a = PerpDist(x1, y1, lx1, ly1, lx2, ly2);
+		b = PerpDist(x2, y2, lx1, ly1, lx2, ly2);
 
 		if (fabs(a) >= 2.0 || fabs(b) >= 2.0)
 			continue;
 
-		a = AlongDist(x1, y1, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
-		b = AlongDist(x2, y2, L->Start()->x, L->Start()->y, L->End()->x, L->End()->y);
+		a = AlongDist(x1, y1, lx1, ly1, lx2, ly2);
+		b = AlongDist(x2, y2, lx1, ly1, lx2, ly2);
 
 		double len = L->CalcLength();
 
@@ -100,14 +105,16 @@ bool LineDefWouldOverlap(int v1, int x2, int y2)
 
 //------------------------------------------------------------------------
 
-static inline const LineDef * LD_ptr(const Obj3d_t& obj)
+static inline const LineDef * LD_ptr(const Objid& obj)
 {
 	return LineDefs[obj.num];
 }
 
-static inline const SideDef * SD_ptr(const Obj3d_t& obj)
+static inline const SideDef * SD_ptr(const Objid& obj)
 {
-	int sd = LD_ptr(obj)->WhatSideDef(obj.side);
+	int where = (obj.parts & PART_LF_ALL) ? SIDE_LEFT : SIDE_RIGHT;
+
+	int sd = LD_ptr(obj)->WhatSideDef(where);
 
 	return (sd >= 0) ? SideDefs[sd] : NULL;
 }
@@ -135,7 +142,7 @@ static int PartialTexCmp(const char *A, const char *B)
 #endif
 
 
-static bool PartIsVisible(const Obj3d_t& obj, char part)
+static bool PartIsVisible(const Objid& obj, char part)
 {
 	const LineDef *L  = LD_ptr(obj);
 	const SideDef *SD = SD_ptr(obj);
@@ -146,7 +153,7 @@ static bool PartIsVisible(const Obj3d_t& obj, char part)
 	const Sector *front = L->Right()->SecRef();
 	const Sector *back  = L->Left ()->SecRef();
 
-	if (obj.side == SIDE_LEFT)
+	if (obj.parts & PART_LF_ALL)
 		std::swap(front, back);
 
 	// ignore sky walls
@@ -174,9 +181,8 @@ static bool PartIsVisible(const Obj3d_t& obj, char part)
 // calculate vertical range that the given surface occupies.
 // when part is zero, we use obj.type instead.
 //
-static void PartCalcExtent(const Obj3d_t& obj, char part, int *z1, int *z2)
+static void PartCalcExtent(const Objid& obj, char part, int *z1, int *z2)
 {
-
 	const LineDef *L  = LD_ptr(obj);
 	const SideDef *SD = SD_ptr(obj);
 
@@ -197,9 +203,9 @@ static void PartCalcExtent(const Obj3d_t& obj, char part, int *z1, int *z2)
 
 	if (! part)
 	{
-		if (obj.type == OB3D_Upper)
+		if (obj.parts & (PART_RT_UPPER | PART_LF_UPPER))
 			part = 'u';
-		else if (obj.type == OB3D_Rail)
+		else if (obj.parts & (PART_RT_RAIL | PART_LF_RAIL))
 			part = 'r';
 		else
 			part = 'l';
@@ -208,7 +214,7 @@ static void PartCalcExtent(const Obj3d_t& obj, char part, int *z1, int *z2)
 	const Sector *front = L->Right()->SecRef();
 	const Sector *back  = L->Left ()->SecRef();
 
-	if (obj.side == SIDE_LEFT)
+	if (obj.parts & PART_LF_ALL)
 		std::swap(front, back);
 
 	if (part == 'r')
@@ -229,7 +235,7 @@ static void PartCalcExtent(const Obj3d_t& obj, char part, int *z1, int *z2)
 }
 
 
-static inline int ScoreTextureMatch(const Obj3d_t& adj, const Obj3d_t& cur)
+static inline int ScoreTextureMatch(const Objid& adj, const Objid& cur)
 {
 	// result is in the range 1..999
 
@@ -260,9 +266,9 @@ static inline int ScoreTextureMatch(const Obj3d_t& adj, const Obj3d_t& cur)
 
 	if (N->TwoSided())
 	{
-		if (adj.type == OB3D_Lower)
+		if (adj.parts & (PART_RT_LOWER | PART_LF_LOWER))
 			adj_tex = NS->LowerTex();
-		else if (adj.type == OB3D_Upper)
+		else if (adj.parts & (PART_RT_UPPER | PART_LF_UPPER))
 			adj_tex = NS->UpperTex();
 	}
 
@@ -273,10 +279,10 @@ static inline int ScoreTextureMatch(const Obj3d_t& adj, const Obj3d_t& cur)
 
 	if (L->TwoSided())
 	{
-		if (adj.type == OB3D_Lower)
-			adj_tex = LS->LowerTex();
-		else if (adj.type == OB3D_Upper)
-			adj_tex = LS->UpperTex();
+		if (cur.parts & (PART_RT_LOWER | PART_LF_LOWER))
+			cur_tex = LS->LowerTex();
+		else if (cur.parts & (PART_RT_UPPER | PART_LF_UPPER))
+			cur_tex = LS->UpperTex();
 	}
 
 	if (PartialTexCmp(cur_tex, adj_tex) != 0)
@@ -305,8 +311,8 @@ static inline int ScoreTextureMatch(const Obj3d_t& adj, const Obj3d_t& cur)
 //
 // Returns < 0 for surfaces that cannot be used.
 //
-static int ScoreAdjoiner(const Obj3d_t& adj,
-						 const Obj3d_t& cur, int align_flags)
+static int ScoreAdjoiner(const Objid& adj,
+						 const Objid& cur, int align_flags)
 {
 	bool do_right = (align_flags & LINALIGN_Right) ? true : false;
 
@@ -327,8 +333,8 @@ static int ScoreAdjoiner(const Obj3d_t& adj,
 	// does the adjoiner sidedef actually mate up with the sidedef
 	// we are aligning (and is on the wanted side) ?
 
-	int v1 = (((cur.side == SIDE_RIGHT) ? 1:0) == (do_right ? 1:0)) ? L->end : L->start;
-	int v2 = (((adj.side == SIDE_RIGHT) ? 1:0) == (do_right ? 1:0)) ? N->start : N->end;
+	int v1 = (((cur.parts & PART_LF_ALL) ? 0:1) == (do_right ? 1:0)) ? L->end : L->start;
+	int v2 = (((adj.parts & PART_LF_ALL) ? 0:1) == (do_right ? 1:0)) ? N->start : N->end;
 
 	if (v1 != v2)
 		return -1;
@@ -351,10 +357,10 @@ static int ScoreAdjoiner(const Obj3d_t& adj,
 //
 // Determine the adjoining surface to align with.
 //
-// Returns -1 for none.
+// Returns a nil Objid for none.
 //
-static void DetermineAdjoiner(Obj3d_t& result,
-							  const Obj3d_t& cur, int align_flags)
+static void DetermineAdjoiner(Objid& result,
+							  const Objid& cur, int align_flags)
 {
 	int best_score = 0;
 
@@ -367,7 +373,7 @@ static void DetermineAdjoiner(Obj3d_t& result,
 		if (N == L)
 			continue;
 
-		if (N->isZeroLength())
+		if (N->IsZeroLength())
 			continue;
 
 		if (! (N->TouchesVertex(L->start) || N->TouchesVertex(L->end)))
@@ -376,11 +382,14 @@ static void DetermineAdjoiner(Obj3d_t& result,
 		for (int side = 0 ; side < 2 ; side++)
 		for (int what = 0 ; what < 3 ; what++)
 		{
-			Obj3d_t adj;
+			Objid adj;
 
-			adj.type = (obj3d_type_e)(OB3D_Lower + what);
-			adj.num  = n;
-			adj.side = side ? SIDE_LEFT : SIDE_RIGHT;
+			adj.type  = OBJ_LINEDEFS;
+			adj.num   = n;
+			adj.parts = (what == 0) ? PART_RT_LOWER : (what == 1) ? PART_RT_UPPER : PART_RT_RAIL;
+
+			if (side == 1)
+				adj.parts = adj.parts << 4;
 
 			int score = ScoreAdjoiner(adj, cur, align_flags);
 
@@ -394,7 +403,7 @@ static void DetermineAdjoiner(Obj3d_t& result,
 }
 
 
-static int CalcReferenceH(const Obj3d_t& obj)
+static int CalcReferenceH(const Objid& obj)
 {
 	const LineDef *L  = LD_ptr(obj);
 	const SideDef *SD = SD_ptr(obj);
@@ -416,45 +425,40 @@ static int CalcReferenceH(const Obj3d_t& obj)
 	const Sector *front = L->Right()->SecRef();
 	const Sector *back  = L->Left ()->SecRef();
 
-	if (obj.side == SIDE_LEFT)
+	if (obj.parts & PART_LF_ALL)
 		std::swap(front, back);
 
 	SYS_ASSERT(front);
 	SYS_ASSERT(back);
 
-	switch (obj.type)
+	if (obj.parts & (PART_RT_UPPER | PART_LF_UPPER))
+	{
+		if (! (L->flags & MLF_UpperUnpegged))
+			return back->ceilh + W_GetTextureHeight(SD->UpperTex());
+
+		return front->ceilh;
+	}
+	else
 	{
 		// TODO : verify if this correct for RAIL
 
-		case OB3D_Lower:
-		case OB3D_Rail:
-			if (! (L->flags & MLF_LowerUnpegged))
-				return back->floorh;
+		if (! (L->flags & MLF_LowerUnpegged))
+			return back->floorh;
 
-			return front->ceilh;
-
-		case OB3D_Upper:
-			if (! (L->flags & MLF_UpperUnpegged))
-				return back->ceilh + W_GetTextureHeight(SD->UpperTex());
-
-			return front->ceilh;
-
-		default:
-			// should not happen
-			return 256;
+		return front->ceilh;
 	}
 }
 
 
-static void DoAlignX(const Obj3d_t& cur,
-					 const Obj3d_t& adj, int align_flags)
+static void DoAlignX(const Objid& cur,
+					 const Objid& adj, int align_flags)
 {
 	const LineDef *cur_L  = LD_ptr(cur);
 
 	const LineDef *adj_L  = LD_ptr(adj);
 	const SideDef *adj_SD = SD_ptr(adj);
 
-	bool on_left = adj_L->TouchesVertex(cur.side < 0 ? cur_L->end : cur_L->start);
+	bool on_left = adj_L->TouchesVertex((cur.parts & PART_LF_ALL) ? cur_L->end : cur_L->start);
 
 	int new_offset = adj_SD->x_offset;
 
@@ -473,14 +477,16 @@ static void DoAlignX(const Obj3d_t& cur,
 			new_offset = - (-new_offset & 1023);
 	}
 
-	int sd = cur_L->WhatSideDef(cur.side);
+	int where = (cur.parts & PART_LF_ALL) ? SIDE_LEFT : SIDE_RIGHT;
+
+	int sd = cur_L->WhatSideDef(where);
 
 	BA_ChangeSD(sd, SideDef::F_X_OFFSET, new_offset);
 }
 
 
-static void DoAlignY(const Obj3d_t& cur,
-					 const Obj3d_t& adj, int align_flags)
+static void DoAlignY(const Objid& cur,
+					 const Objid& adj, int align_flags)
 {
 	const LineDef *L  = LD_ptr(cur);
 	const SideDef *SD = SD_ptr(cur);
@@ -526,7 +532,9 @@ static void DoAlignY(const Obj3d_t& cur,
 		new_offset &= 255;
 
 
-	int sd = L->WhatSideDef(cur.side);
+	int where = (cur.parts & PART_LF_ALL) ? SIDE_LEFT : SIDE_RIGHT;
+
+	int sd = L->WhatSideDef(where);
 
 	BA_ChangeSD(sd, SideDef::F_Y_OFFSET, new_offset);
 
@@ -537,9 +545,11 @@ static void DoAlignY(const Obj3d_t& cur,
 }
 
 
-static void DoClearOfs(const Obj3d_t& cur, int align_flags)
+static void DoClearOfs(const Objid& cur, int align_flags)
 {
-	int sd = LD_ptr(cur)->WhatSideDef(cur.side);
+	int where = (cur.parts & PART_LF_ALL) ? SIDE_LEFT : SIDE_RIGHT;
+
+	int sd = LD_ptr(cur)->WhatSideDef(where);
 
 	if (sd < 0)  // should not happen
 		return;
@@ -565,7 +575,7 @@ static void DoClearOfs(const Obj3d_t& cur, int align_flags)
 // The given flags control which stuff to change, and where
 // to look for the surface to align with.
 //
-bool Line_AlignOffsets(const Obj3d_t& obj, int align_flags)
+bool Line_AlignOffsets(const Objid& obj, int align_flags)
 {
 	if (align_flags & LINALIGN_Clear)
 	{
@@ -573,8 +583,7 @@ bool Line_AlignOffsets(const Obj3d_t& obj, int align_flags)
 		return true;
 	}
 
-	Obj3d_t adj;
-
+	Objid adj;
 	DetermineAdjoiner(adj, obj, align_flags);
 
 	if (! adj.valid())
@@ -592,22 +601,22 @@ bool Line_AlignOffsets(const Obj3d_t& obj, int align_flags)
 
 // returns true if the surface at 'k' MUST be aligned before the
 // surface at 'j'.
-static bool Align_CheckAdjacent(std::vector<Obj3d_t> & group,
+static bool Align_CheckAdjacent(std::vector<Objid> & group,
 								int j, int k, bool do_right)
 {
-	const Obj3d_t& ob_j = group[j];
-	const Obj3d_t& ob_k = group[k];
+	const Objid& ob_j = group[j];
+	const Objid& ob_k = group[k];
 
 	int vj = 0;
 
-	if (((ob_j.side == SIDE_RIGHT) ? 1 : 0) == (do_right ? 1 : 0))
+	if (((ob_j.parts & PART_LF_ALL) ? 0 : 1) == (do_right ? 1 : 0))
 		vj = LineDefs[ob_j.num]->end;
 	else
 		vj = LineDefs[ob_j.num]->start;
 
 	int vk = 0;
 
-	if (((ob_k.side == SIDE_RIGHT) ? 1 : 0) == (do_right ? 1 : 0))
+	if (((ob_k.parts & PART_LF_ALL) ? 0 : 1) == (do_right ? 1 : 0))
 		vk = LineDefs[ob_k.num]->start;
 	else
 		vk = LineDefs[ob_k.num]->end;
@@ -621,7 +630,7 @@ static bool Align_CheckAdjacent(std::vector<Obj3d_t> & group,
 // any other unvisited surface.  In the case of loops, we pick
 // an arbitrary surface.
 //
-static int Align_PickNextSurface(std::vector<Obj3d_t> & group,
+static int Align_PickNextSurface(std::vector<Objid> & group,
 								 const std::vector<byte>& seen, bool do_right)
 {
 	int fallback = -1;
@@ -658,7 +667,7 @@ static int Align_PickNextSurface(std::vector<Obj3d_t> & group,
 }
 
 
-void Line_AlignGroup(std::vector<Obj3d_t> & group, int align_flags)
+void Line_AlignGroup(std::vector<Objid> & group, int align_flags)
 {
 	// we will do each surface in the selection one-by-one,
 	// and the order is significant when doing X offsets, so
@@ -692,15 +701,6 @@ void Line_AlignGroup(std::vector<Obj3d_t> & group, int align_flags)
 
 void CMD_LIN_Align()
 {
-	selection_c list;
-
-	if (! GetCurrentObjects(&list))
-	{
-		Beep("no lines to align");
-		return;
-	}
-
-
 	// parse the flags
 	bool do_X = Exec_HasFlag("/x");
 	bool do_Y = Exec_HasFlag("/y");
@@ -713,7 +713,7 @@ void CMD_LIN_Align()
 
 	bool do_clear = Exec_HasFlag("/clear");
 	bool do_right = Exec_HasFlag("/right");
-	bool do_unpeg = true;
+	bool do_unpeg = true;  // TODO a flag to disable
 
 	int align_flags = 0;
 
@@ -725,29 +725,54 @@ void CMD_LIN_Align()
 	if (do_clear) align_flags |= LINALIGN_Clear;
 
 
-	// convert selection to group of surfaces
-
-	std::vector< Obj3d_t > group;
-
-	selection_iterator_c it;
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (edit.mode != OBJ_LINEDEFS || unselect == SOH_Empty)
 	{
-		Obj3d_t obj;
+		Beep("no lines to align");
+		return;
+	}
 
-		obj.num = *it;
+	/* convert selection to group of surfaces */
 
-		const LineDef *L = LineDefs[obj.num];
+	std::vector< Objid > group;
 
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
+	{
+		int parts = edit.Selected->get_ext(*it);
+		parts &= ~1;
+
+		const LineDef *L = LineDefs[*it];
+
+		// safety check
+		if (L->left  < 0) parts &= ~PART_LF_ALL;
+		if (L->right < 0) parts &= ~PART_RT_ALL;
+
+		Objid obj(OBJ_LINEDEFS, *it);
+
+		// handle lines with individual parts selected
+		if (parts != 0)
+		{
+			for (int bit = 0x02 ; bit <= 0x80 ; bit <<= 1)
+			{
+				if (parts & bit)
+				{
+					obj.parts = parts & bit;
+					group.push_back(obj);
+				}
+			}
+			continue;
+		}
+
+		// handle "simply selected" lines, determine parts to use
 		for (int pass = 0 ; pass < 2 ; pass++)
 		{
-			obj.side = pass ? SIDE_LEFT : SIDE_RIGHT;
+			int where = pass ? SIDE_LEFT : SIDE_RIGHT;
 
-			if (L->WhatSideDef(obj.side) < 0)
+			if (L->WhatSideDef(where) < 0)
 				continue;
 
 			// decide whether to use upper or lower
-			// TODO : this could be smarter....
+			// WISH : this could be smarter....
 
 			bool lower_vis = PartIsVisible(obj, 'l');
 			bool upper_vis = PartIsVisible(obj, 'u');
@@ -755,7 +780,9 @@ void CMD_LIN_Align()
 			if (! (lower_vis || upper_vis))
 				continue;
 
-			obj.type = lower_vis ? OB3D_Lower : OB3D_Upper;
+			obj.parts = lower_vis ? PART_RT_LOWER : PART_RT_UPPER;
+			if (where < 0)
+				obj.parts = obj.parts << 4;
 
 			group.push_back(obj);
 		}
@@ -764,6 +791,8 @@ void CMD_LIN_Align()
 	if (group.empty())
 	{
 		Beep("no visible surfaces");
+		if (unselect == SOH_Unselect)
+			Selection_Clear(true /* nosave */);
 		return;
 	}
 
@@ -777,12 +806,14 @@ void CMD_LIN_Align()
 		BA_Message("aligned offsets");
 
 	BA_End();
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
 //------------------------------------------------------------------------
 
-
 static void FlipLine_verts(int ld)
 {
 	int old_start = LineDefs[ld]->start;
@@ -821,11 +852,9 @@ void FlipLineDef_safe(int ld)
 }
 
 
-void FlipLineDefGroup(selection_c& flip)
+void FlipLineDefGroup(selection_c *flip)
 {
-	selection_iterator_c it;
-
-	for (flip.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(flip) ; !it.done() ; it.next())
 	{
 		FlipLineDef(*it);
 	}
@@ -837,9 +866,8 @@ void FlipLineDefGroup(selection_c& flip)
 //
 void CMD_LIN_Flip()
 {
-	selection_c list;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("No lines to flip");
 		return;
@@ -848,10 +876,9 @@ void CMD_LIN_Flip()
 	bool force_it = Exec_HasFlag("/force");
 
 	BA_Begin();
+	BA_MessageForSel("flipped", edit.Selected);
 
-	selection_iterator_c it;
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		if (force_it)
 			FlipLineDef(*it);
@@ -859,34 +886,34 @@ void CMD_LIN_Flip()
 			FlipLineDef_safe(*it);
 	}
 
-	BA_MessageForSel("flipped", &list);
-
 	BA_End();
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
 void CMD_LIN_SwapSides()
 {
-	selection_c list;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("No lines to swap sides");
 		return;
 	}
 
 	BA_Begin();
+	BA_MessageForSel("swapped sides on", edit.Selected);
 
-	selection_iterator_c it;
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		FlipLine_sides(*it);
 	}
 
-	BA_MessageForSel("swapped sides on", &list);
-
 	BA_End();
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
@@ -910,13 +937,8 @@ int SplitLineDefAtVertex(int ld, int new_v)
 	BA_ChangeLD(ld, LineDef::F_END, new_v);
 
 	// compute lengths (to update sidedef X offsets)
-	int orig_length = (int)ComputeDist(
-			L->Start()->x - L->End()->x,
-			L->Start()->y - L->End()->y);
-
-	int new_length = (int)ComputeDist(
-			L->Start()->x - V->x,
-			L->Start()->y - V->y);
+	int orig_length = I_ROUND(L->CalcLength());
+	int new_length  = I_ROUND(hypot(L->Start()->x() - V->x(), L->Start()->y() - V->y()));
 
 	// update sidedefs
 
@@ -950,20 +972,19 @@ static bool DoSplitLineDef(int ld)
 {
 	LineDef * L = LineDefs[ld];
 
-	int new_x = (L->Start()->x + L->End()->x) / 2;
-	int new_y = (L->Start()->y + L->End()->y) / 2;
-
 	// prevent creating tiny lines (especially zero-length)
-	if (abs(L->Start()->x - L->End()->x) < 4 &&
-	    abs(L->Start()->y - L->End()->y) < 4)
+	if (abs(L->Start()->x() - L->End()->x()) < 4 &&
+	    abs(L->Start()->y() - L->End()->y()) < 4)
 		return false;
 
+	double new_x = (L->Start()->x() + L->End()->x()) / 2;
+	double new_y = (L->Start()->y() + L->End()->y()) / 2;
+
 	int new_v = BA_New(OBJ_VERTICES);
 
 	Vertex * V = Vertices[new_v];
 
-	V->x = new_x;
-	V->y = new_y;
+	V->SetRawXY(new_x, new_y);
 
 	SplitLineDefAtVertex(ld, new_v);
 
@@ -976,46 +997,37 @@ static bool DoSplitLineDef(int ld)
 //
 void CMD_LIN_SplitHalf(void)
 {
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("No lines to split");
 		return;
 	}
 
-	bool was_selected = edit.Selected->notempty();
-
-	// clear current selection, since the size needs to grow due to
-	// new linedefs being added to the map.
-	Selection_Clear(true);
-
 	int new_first = NumLineDefs;
 	int new_count = 0;
 
 	BA_Begin();
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		if (DoSplitLineDef(*it))
 			new_count++;
 	}
 
-	BA_MessageForSel("halved", &list);
-
+	BA_Message("halved %d lines", new_count);
 	BA_End();
 
 	// Hmmmmm -- should abort early if some lines are too short??
-	if (new_count < list.count_obj())
+	if (new_count < edit.Selected->count_obj())
 		Beep("Some lines were too short!");
 
-	if (was_selected && new_count > 0)
+	if (unselect == SOH_Unselect)
+	{
+		Selection_Clear(true /* nosave */);
+	}
+	else if (new_count > 0)
 	{
-		// reselect the old _and_ new linedefs
-		for (list.begin(&it) ; !it.at_end() ; ++it)
-			edit.Selected->set(*it);
-
 		edit.Selected->frob_range(new_first, new_first + new_count - 1, BOP_ADD);
 	}
 }
@@ -1163,7 +1175,7 @@ void CMD_LIN_MergeTwo(void)
 {
 	if (edit.Selected->count_obj() == 1 && edit.highlight.valid())
 	{
-		edit.Selected->set(edit.highlight.num);
+		Selection_Add(edit.highlight);
 	}
 
 	if (edit.Selected->count_obj() != 2)
@@ -1227,19 +1239,18 @@ void CMD_LIN_MergeTwo(void)
 	DeleteObjects_WithUnused(&del_line);
 
 	BA_Message("merged two linedefs");
-
 	BA_End();
 }
 
 
-void MoveCoordOntoLineDef(int ld, int *x, int *y)
+void MoveCoordOntoLineDef(int ld, double *x, double *y)
 {
 	const LineDef *L = LineDefs[ld];
 
-	double x1 = L->Start()->x;
-	double y1 = L->Start()->y;
-	double x2 = L->End()->x;
-	double y2 = L->End()->y;
+	double x1 = L->Start()->x();
+	double y1 = L->Start()->y();
+	double x2 = L->End()->x();
+	double y2 = L->End()->y();
 
 	double dx = x2 - x1;
 	double dy = y2 - y1;
@@ -1252,19 +1263,14 @@ void MoveCoordOntoLineDef(int ld, int *x, int *y)
 	double along = (*x - x1) * dx + (*y - y1) * dy;
 
 	// result = start + along * line unit vector
-	double new_x = x1 + along * dx / len_squared;
-	double new_y = y1 + along * dy / len_squared;
-
-	*x = I_ROUND(new_x);
-	*y = I_ROUND(new_y);
+	*x = x1 + along * dx / len_squared;
+	*y = y1 + along * dy / len_squared;
 }
 
 
 static bool LD_StartWillBeMoved(int ld, selection_c& list)
 {
-	selection_iterator_c it;
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(list) ; !it.done() ; it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
@@ -1278,9 +1284,7 @@ static bool LD_StartWillBeMoved(int ld, selection_c& list)
 
 static bool LD_EndWillBeMoved(int ld, selection_c& list)
 {
-	selection_iterator_c it;
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(list) ; !it.done() ; it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
@@ -1297,9 +1301,7 @@ static int PickLineDefToExtend(selection_c& list, bool moving_start)
 	// We want a line whose new length is not going to be wrecked
 	// by a change to a later linedef.  However we must handle loops!
 
-	selection_iterator_c it;
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(list) ; !it.done() ; it.next())
 	{
 		if (moving_start)
 		{
@@ -1341,26 +1343,27 @@ static void LD_SetLength(int ld, int new_len, int angle)
 
 	if (new_len < 0)
 	{
-		BA_ChangeVT(L->start, Vertex::F_X, L->End()->x - idx);
-		BA_ChangeVT(L->start, Vertex::F_Y, L->End()->y - idy);
+		BA_ChangeVT(L->start, Vertex::F_X, L->End()->raw_x - INT_TO_COORD(idx));
+		BA_ChangeVT(L->start, Vertex::F_Y, L->End()->raw_y - INT_TO_COORD(idy));
 	}
 	else
 	{
-		BA_ChangeVT(L->end, Vertex::F_X, L->Start()->x + idx);
-		BA_ChangeVT(L->end, Vertex::F_Y, L->Start()->y + idy);
+		BA_ChangeVT(L->end, Vertex::F_X, L->Start()->raw_x + INT_TO_COORD(idx));
+		BA_ChangeVT(L->end, Vertex::F_Y, L->Start()->raw_y + INT_TO_COORD(idy));
 	}
 }
 
 
 void LineDefs_SetLength(int new_len)
 {
-	selection_c list;
+	// this works on the current selection (caller must set it up)
 
-	if (! GetCurrentObjects(&list))
-	{
-		Beep("No lines to extend");
+	// use a copy of the selection
+	selection_c list(OBJ_LINEDEFS);
+	ConvertSelection(edit.Selected, &list);
+
+	if (list.empty())
 		return;
-	}
 
 	// remember angles
 	std::vector<int> angles(NumLineDefs);
@@ -1369,12 +1372,11 @@ void LineDefs_SetLength(int new_len)
 	{
 		const LineDef *L = LineDefs[n];
 
-		angles[n] = ComputeAngle(L->End()->x - L->Start()->x,
-								 L->End()->y - L->Start()->y);
+		angles[n] = ComputeAngle(L->End()->x() - L->Start()->x(),
+								 L->End()->y() - L->Start()->y());
 	}
 
 	BA_Begin();
-
 	BA_MessageForSel("set length of", &list);
 
 	while (! list.empty())
@@ -1419,11 +1421,11 @@ void LD_FixForLostSide(int ld)
 //
 double LD_AngleBetweenLines(int A, int B, int C)
 {
-	int a_dx = Vertices[B]->x - Vertices[A]->x;
-	int a_dy = Vertices[B]->y - Vertices[A]->y;
+	double a_dx = Vertices[B]->x() - Vertices[A]->x();
+	double a_dy = Vertices[B]->y() - Vertices[A]->y();
 
-	int c_dx = Vertices[B]->x - Vertices[C]->x;
-	int c_dy = Vertices[B]->y - Vertices[C]->y;
+	double c_dx = Vertices[B]->x() - Vertices[C]->x();
+	double c_dy = Vertices[B]->y() - Vertices[C]->y();
 
 	double AB_angle = (a_dx == 0) ? (a_dy >= 0 ? 90 : -90) : atan2(a_dy, a_dx) * 180 / M_PI;
 	double CB_angle = (c_dx == 0) ? (c_dy >= 0 ? 90 : -90) : atan2(c_dy, c_dx) * 180 / M_PI;
@@ -1512,6 +1514,93 @@ bool LD_GetTwoNeighbors(int new_ld, int v1, int v2,
 }
 
 
+std::string LD_RatioName(fixcoord_t idx, fixcoord_t idy, bool number_only)
+{
+	idx = abs(idx);
+	idy = abs(idy);
+
+	if (idx == 0 && idy == 0)
+	{
+		if (number_only)
+			return "0:0";
+		else
+			return "zero-len";
+	}
+	else if (idx == 0)
+	{
+		if (number_only)
+			return "1:0";
+		else
+			return "vertical";
+	}
+	else if (idy == 0)
+	{
+		if (number_only)
+			return "0:1";
+		else
+			return "horizontal";
+	}
+
+	// compute the greatest common divisor
+	int a = idx;
+	int b = idy;
+	int gcd = 1;
+
+	for (;;)
+	{
+		if (a > b)
+			a -= b;
+		else if (b > a)
+			b -= a;
+		else
+		{
+			gcd = a;
+			break;
+		}
+	}
+
+	idx /= gcd;
+	idy /= gcd;
+
+	// if ratio is really complex, it isn't worth showing
+	if (idx > 999 || idy > 999)
+		return std::string("---");
+
+	char buffer[256];
+	snprintf(buffer, sizeof(buffer), "%s%d:%d", number_only ? "" : "ratio ", idx, idy);
+
+	return std::string(buffer);
+}
+
+
+bool LD_RailHeights(int& z1, int& z2, const LineDef *L, const SideDef *sd,
+					const Sector *front, const Sector *back)
+{
+	const char *rail_tex = sd->MidTex();
+	if (is_null_tex(rail_tex))
+		return false;
+
+	z1 = MAX(front->floorh, back->floorh);
+	z2 = MIN(front->ceilh,  back->ceilh);
+
+	if (z2 <= z1)
+		return false;
+
+	int img_h = W_GetTextureHeight(rail_tex);
+
+	if (L->flags & MLF_LowerUnpegged)
+	{
+		z1 = z1 + sd->y_offset;
+		z2 = z1 + img_h;
+	}
+	else
+	{
+		z2 = z2 + sd->y_offset;
+		z1 = z2 - img_h;
+	}
+
+	return true;
+}
 
 
 //  SideDef packing logic -- raw from glBSP
