@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -35,6 +35,7 @@
 #include "e_things.h"
 #include "e_vertex.h"
 #include "r_render.h"
+#include "r_subdiv.h"
 #include "w_rawdef.h"
 
 #include "ui_window.h"
@@ -43,10 +44,10 @@
 Editor_State_t  edit;
 
 
-int Map_bound_x1 =  32767;   /* minimum X value of map */
-int Map_bound_y1 =  32767;   /* minimum Y value of map */
-int Map_bound_x2 = -32767;   /* maximum X value of map */
-int Map_bound_y2 = -32767;   /* maximum Y value of map */
+double Map_bound_x1 =  32767;   /* minimum X value of map */
+double Map_bound_y1 =  32767;   /* minimum Y value of map */
+double Map_bound_x2 = -32767;   /* maximum X value of map */
+double Map_bound_y2 = -32767;   /* maximum Y value of map */
 
 int MadeChanges;
 
@@ -83,10 +84,10 @@ static void zoom_fit()
 	int ScrMaxY = main_win->canvas->h();
 
 	if (Map_bound_x1 < Map_bound_x2)
-		xzoom = ScrMaxX / (double)(Map_bound_x2 - Map_bound_x1);
+		xzoom = ScrMaxX / (Map_bound_x2 - Map_bound_x1);
 
 	if (Map_bound_y1 < Map_bound_y2)
-		yzoom = ScrMaxY / (double)(Map_bound_y2 - Map_bound_y1);
+		yzoom = ScrMaxY / (Map_bound_y2 - Map_bound_y1);
 
 	grid.NearestScale(MIN(xzoom, yzoom));
 
@@ -112,10 +113,10 @@ void RedrawMap()
 
 	UpdateHighlight();
 
-	if (edit.render3d)
-		main_win->render->redraw();
-	else
-		main_win->canvas->redraw();
+	main_win->scroll->UpdateRenderMode();
+	main_win->info_bar->UpdateSecRend();
+	main_win->status_bar->redraw();
+	main_win->canvas->redraw();
 }
 
 
@@ -138,6 +139,10 @@ static void UpdatePanel()
 	int obj_idx   = edit.highlight.num;
 	int obj_count = edit.Selected->count_obj();
 
+	// the highlight is usually turned off when dragging, so compensate
+	if (obj_idx < 0 && edit.action == ACT_DRAG)
+		obj_idx = edit.dragged.num;
+
 	if (obj_idx >= 0)
 	{
 		if (! edit.Selected->get(obj_idx))
@@ -145,14 +150,13 @@ static void UpdatePanel()
 	}
 	else if (obj_count > 0)
 	{
-		// in linedef mode, we want a two-sided linedef to show
+		// in linedef mode, we prefer showing a two-sided linedef
 		if (edit.mode == OBJ_LINEDEFS && obj_count > 1)
 			obj_idx = Selection_FirstLine(edit.Selected);
 		else
 			obj_idx = edit.Selected->find_first();
 	}
 
-
 	switch (edit.mode)
 	{
 		case OBJ_THINGS:
@@ -176,123 +180,141 @@ static void UpdatePanel()
 }
 
 
-static void UpdateSplitLine(int map_x, int map_y)
+void UpdateDrawLine()
+{
+	if (edit.action != ACT_DRAW_LINE || edit.draw_from.is_nil())
+		return;
+
+	const Vertex *V = Vertices[edit.draw_from.num];
+
+	double new_x = edit.map_x;
+	double new_y = edit.map_y;
+
+	if (grid.ratio > 0)
+	{
+		grid.RatioSnapXY(new_x, new_y, V->x(), V->y());
+	}
+	else if (edit.highlight.valid())
+	{
+		new_x = Vertices[edit.highlight.num]->x();
+		new_y = Vertices[edit.highlight.num]->y();
+	}
+	else if (edit.split_line.valid())
+	{
+		new_x = edit.split_x;
+		new_y = edit.split_y;
+	}
+	else
+	{
+		new_x = grid.SnapX(new_x);
+		new_y = grid.SnapY(new_y);
+	}
+
+	edit.draw_to_x = new_x;
+	edit.draw_to_y = new_y;
+
+	// when drawing mode, highlight a vertex at the snap position
+	if (grid.snap && edit.highlight.is_nil() && edit.split_line.is_nil())
+	{
+		int near_vert = Vertex_FindExact(TO_COORD(new_x), TO_COORD(new_y));
+		if (near_vert >= 0)
+		{
+			edit.highlight = Objid(OBJ_VERTICES, near_vert);
+		}
+	}
+
+	// never highlight the vertex we are drawing from
+	if (edit.draw_from.valid() &&
+		edit.draw_from == edit.highlight)
+	{
+		edit.highlight.clear();
+	}
+}
+
+
+static void UpdateSplitLine(double map_x, double map_y)
 {
 	edit.split_line.clear();
 
-	// usually disabled while dragging stuff
-	if (edit.action == ACT_DRAG && edit.drag_single_obj < 0)
-		return;
+	// splitting usually disabled while dragging stuff, EXCEPT a single vertex
+	if (edit.action == ACT_DRAG && edit.dragged.is_nil())
+		goto done;
 
 	// in vertex mode, see if there is a linedef which would be split by
-	// adding a new vertex
+	// adding a new vertex.
 
 	if (edit.mode == OBJ_VERTICES &&
-		edit.pointer_in_window && !edit.render3d &&
+		edit.pointer_in_window &&
 	    edit.highlight.is_nil())
 	{
-		GetSplitLineDef(edit.split_line, map_x, map_y, edit.drag_single_obj);
+		FindSplitLine(edit.split_line, edit.split_x, edit.split_y,
+					  map_x, map_y, edit.dragged.num);
 
 		// NOTE: OK if the split line has one of its vertices selected
 		//       (that case is handled by Insert_Vertex)
 	}
 
-	if (edit.split_line.valid())
-	{
-		edit.split_x = grid.SnapX(map_x);
-		edit.split_y = grid.SnapY(map_y);
-
-		// in FREE mode, ensure the new vertex is directly on the linedef
-		if (! grid.snap)
-			MoveCoordOntoLineDef(edit.split_line.num, &edit.split_x, &edit.split_y);
-
-		main_win->canvas->SplitLineSet(edit.split_line.num, edit.split_x, edit.split_y);
-	}
-	else
-		main_win->canvas->SplitLineForget();
+done:
+	main_win->canvas->UpdateHighlight();
 }
 
 
 void UpdateHighlight()
 {
-	bool dragging = (edit.action == ACT_DRAG);
-
+	if (edit.render3d)
+	{
+		Render3D_UpdateHighlight();
+		UpdatePanel();
+		return;
+	}
 
 	// find the object to highlight
 	edit.highlight.clear();
 
-	if (edit.pointer_in_window && !edit.render3d &&
-	    (!dragging || (edit.mode == OBJ_VERTICES && edit.drag_single_obj >= 0)))
+	// don't highlight when dragging, EXCEPT when dragging a single vertex
+	if (edit.pointer_in_window &&
+	    (edit.action != ACT_DRAG || (edit.mode == OBJ_VERTICES && edit.dragged.valid()) ))
 	{
 		GetNearObject(edit.highlight, edit.mode, edit.map_x, edit.map_y);
 
 		// guarantee that we cannot drag a vertex onto itself
-		if (edit.drag_single_obj >= 0 && edit.highlight.valid() &&
-			edit.drag_single_obj == edit.highlight.num)
+		if (edit.action == ACT_DRAG && edit.dragged.valid() &&
+			edit.highlight.valid() && edit.dragged.num == edit.highlight.num)
 		{
 			edit.highlight.clear();
 		}
-	}
 
+		// if drawing a line and ratio lock is ON, only highlight a
+		// vertex if it is *exactly* the right ratio.
+		if (grid.ratio > 0 && edit.action == ACT_DRAW_LINE &&
+			edit.mode == OBJ_VERTICES && edit.highlight.valid())
+		{
+			const Vertex *V = Vertices[edit.highlight.num];
+			const Vertex *S = Vertices[edit.draw_from.num];
+
+			double vx = V->x();
+			double vy = V->y();
+
+			grid.RatioSnapXY(vx, vy, S->x(), S->y());
+
+			if (MakeValidCoord(vx) != V->raw_x ||
+				MakeValidCoord(vy) != V->raw_y)
+			{
+				edit.highlight.clear();
+			}
+		}
+	}
 
 	UpdateSplitLine(edit.map_x, edit.map_y);
+	UpdateDrawLine();
 
-
-	// in drawing mode, highlight extends to a vertex at the snap position
-	if (edit.mode == OBJ_VERTICES && grid.snap &&
-		edit.action == ACT_DRAW_LINE &&
-		edit.highlight.is_nil() && edit.split_line.is_nil())
-	{
-		int new_x = grid.SnapX(edit.map_x);
-		int new_y = grid.SnapY(edit.map_y);
-
-		int near_vert = Vertex_FindExact(new_x, new_y);
-
-		if (near_vert >= 0)
-		{
-			edit.highlight = Objid(OBJ_VERTICES, near_vert);
-		}
-		else
-		{
-			UpdateSplitLine(new_x, new_y);
-		}
-	}
-
-
-	if (edit.highlight.valid())
-		main_win->canvas->HighlightSet(edit.highlight);
-	else
-		main_win->canvas->HighlightForget();
-
-
-	Render3D_UpdateHighlight();
+	main_win->canvas->UpdateHighlight();
+	main_win->canvas->CheckGridSnap();
 
 	UpdatePanel();
 }
 
 
-bool GetCurrentObjects(selection_c *list)
-{
-	// returns false when there are no objects at all
-
-	list->change_type(edit.mode);  // this also clears it
-
-	if (edit.Selected->notempty())
-	{
-		list->merge(*edit.Selected);
-		return true;
-	}
-
-	if (edit.highlight.valid())
-	{
-		list->set(edit.highlight.num);
-		return true;
-	}
-
-	return false;
-}
-
-
 void Editor_ClearErrorMode()
 {
 	if (edit.error_mode)
@@ -315,8 +337,6 @@ void Editor_ChangeMode_Raw(obj_type_e new_mode)
 
 	edit.highlight.clear();
 	edit.split_line.clear();
-
-	SectorCache_Invalidate();
 }
 
 
@@ -345,7 +365,7 @@ void Editor_ChangeMode(char mode_char)
 
 		// convert the selection
 		selection_c *prev_sel = edit.Selected;
-		edit.Selected = new selection_c(edit.mode);
+		edit.Selected = new selection_c(edit.mode, true /* extended */);
 
 		ConvertSelection(prev_sel, edit.Selected);
 		delete prev_sel;
@@ -375,11 +395,11 @@ void UpdateLevelBounds(int start_vert)
 	{
 		const Vertex * V = Vertices[i];
 
-		if (V->x < Map_bound_x1) Map_bound_x1 = V->x;
-		if (V->y < Map_bound_y1) Map_bound_y1 = V->y;
+		if (V->x() < Map_bound_x1) Map_bound_x1 = V->x();
+		if (V->y() < Map_bound_y1) Map_bound_y1 = V->y();
 
-		if (V->x > Map_bound_x2) Map_bound_x2 = V->x;
-		if (V->y > Map_bound_y2) Map_bound_y2 = V->y;
+		if (V->x() > Map_bound_x2) Map_bound_x2 = V->x();
+		if (V->y() > Map_bound_y2) Map_bound_y2 = V->y();
 	}
 }
 
@@ -392,8 +412,8 @@ void CalculateLevelBounds()
 		return;
 	}
 
-	Map_bound_x1 = 999999; Map_bound_x2 = -999999;
-	Map_bound_y1 = 999999; Map_bound_y2 = -999999;
+	Map_bound_x1 = 32767; Map_bound_x2 = -32767;
+	Map_bound_y1 = 32767; Map_bound_y2 = -32767;
 
 	UpdateLevelBounds(0);
 }
@@ -424,7 +444,7 @@ void MapStuff_NotifyDelete(obj_type_e type, int objnum)
 		recalc_map_bounds = true;
 
 		if (edit.action == ACT_DRAW_LINE &&
-			edit.drawing_from == objnum)
+			edit.draw_from.num == objnum)
 		{
 			Editor_ClearAction();
 		}
@@ -441,20 +461,24 @@ void MapStuff_NotifyChange(obj_type_e type, int objnum, int field)
 
 		const Vertex * V = Vertices[objnum];
 
-		if (V->x < Map_bound_x1) Map_bound_x1 = V->x;
-		if (V->y < Map_bound_y1) Map_bound_y1 = V->y;
+		if (V->x() < Map_bound_x1) Map_bound_x1 = V->x();
+		if (V->y() < Map_bound_y1) Map_bound_y1 = V->y();
 
-		if (V->x > Map_bound_x2) Map_bound_x2 = V->x;
-		if (V->y > Map_bound_y2) Map_bound_y2 = V->y;
+		if (V->x() > Map_bound_x2) Map_bound_x2 = V->x();
+		if (V->y() > Map_bound_y2) Map_bound_y2 = V->y();
 
-		SectorCache_Invalidate();
+		// TODO: only invalidate sectors touching vertex
+		Subdiv_InvalidateAll();
 	}
 
 	if (type == OBJ_SIDEDEFS && field == SideDef::F_SECTOR)
-		SectorCache_Invalidate();
+		Subdiv_InvalidateAll();
 
 	if (type == OBJ_LINEDEFS && (field == LineDef::F_LEFT || field == LineDef::F_RIGHT))
-		SectorCache_Invalidate();
+		Subdiv_InvalidateAll();
+
+	if (type == OBJ_SECTORS && (field == Sector::F_FLOORH || field == Sector::F_CEILH))
+		Subdiv_InvalidateAll();
 }
 
 void MapStuff_NotifyEnd()
@@ -553,22 +577,18 @@ void ObjectBox_NotifyEnd()
 //------------------------------------------------------------------------
 
 static bool invalidated_selection;
-static bool invalidated_3d_selection;
 static bool invalidated_last_sel;
 
 
 void Selection_NotifyBegin()
 {
 	invalidated_selection = false;
-	invalidated_3d_selection = false;
 	invalidated_last_sel  = false;
 }
 
 
 void Selection_NotifyInsert(obj_type_e type, int objnum)
 {
-	invalidated_3d_selection = true;
-
 	if (type == edit.Selected->what_type() &&
 		objnum <= edit.Selected->max_obj())
 	{
@@ -586,8 +606,6 @@ void Selection_NotifyInsert(obj_type_e type, int objnum)
 
 void Selection_NotifyDelete(obj_type_e type, int objnum)
 {
-	invalidated_3d_selection = true;
-
 	if (objnum <= edit.Selected->max_obj())
 	{
 		invalidated_selection = true;
@@ -616,9 +634,6 @@ void Selection_NotifyEnd()
 		edit.Selected->change_type(edit.mode);
 	}
 
-	if (invalidated_3d_selection)
-		Render3D_ClearSelection();
-
 	if (invalidated_last_sel)
 		Selection_InvalidateLast();
 }
@@ -633,9 +648,7 @@ void DumpSelection(selection_c * list)
 
 	printf("Selection:");
 
-	selection_iterator_c it;
-
-	for (list->begin(&it); ! it.at_end(); ++it)
+	for (sel_iter_c it(list); ! it.done(); it.next())
 		printf(" %d", *it);
 
 	printf("\n");
@@ -653,16 +666,12 @@ void ConvertSelection(selection_c * src, selection_c * dest)
 
 	if (src->what_type() == OBJ_SECTORS && dest->what_type() == OBJ_THINGS)
 	{
-		// FIXME: get bbox of selection, skip things outside it
-
 		for (int t = 0 ; t < NumThings ; t++)
 		{
 			const Thing *T = Things[t];
 
-			// if (! thing_touches_bbox(T->x, T->y, 128, bbox))
-			//    continue;
-
-			Objid obj;  GetNearObject(obj, OBJ_SECTORS, T->x, T->y);
+			Objid obj;
+			GetNearObject(obj, OBJ_SECTORS, T->x(), T->y());
 
 			if (! obj.is_nil() && src->get(obj.num))
 			{
@@ -708,9 +717,7 @@ void ConvertSelection(selection_c * src, selection_c * dest)
 
 	if (src->what_type() == OBJ_LINEDEFS && dest->what_type() == OBJ_SIDEDEFS)
 	{
-		selection_iterator_c it;
-
-		for (src->begin(&it); ! it.at_end(); ++it)
+		for (sel_iter_c it(src); ! it.done(); it.next())
 		{
 			const LineDef *L = LineDefs[*it];
 
@@ -735,9 +742,7 @@ void ConvertSelection(selection_c * src, selection_c * dest)
 
 	if (src->what_type() == OBJ_LINEDEFS && dest->what_type() == OBJ_VERTICES)
 	{
-		selection_iterator_c it;
-
-		for (src->begin(&it); ! it.at_end(); ++it)
+		for (sel_iter_c it(src); ! it.done(); it.next())
 		{
 			const LineDef *L = LineDefs[*it];
 
@@ -815,9 +820,7 @@ void ConvertSelection(selection_c * src, selection_c * dest)
 //
 int Selection_FirstLine(selection_c *list)
 {
-	selection_iterator_c it;
-
-	for (list->begin(&it); ! it.at_end(); ++it)
+	for (sel_iter_c it(list); ! it.done(); it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
@@ -831,19 +834,35 @@ int Selection_FirstLine(selection_c *list)
 
 
 //
-// select all objects inside a given box
+// This is a helper to handle performing an operation on the
+// selection if it is non-empty, otherwise the highlight.
+// Returns false if both selection and highlight are empty.
 //
-void SelectObjectsInBox(selection_c *list, int objtype, int x1, int y1, int x2, int y2)
+soh_type_e Selection_Or_Highlight()
 {
-	if (x2 < x1)
+	if (! edit.Selected->empty())
+		return SOH_OK;
+
+	if (edit.highlight.valid())
 	{
-		int tmp = x1; x1 = x2; x2 = tmp;
+		Selection_Add(edit.highlight);
+		return SOH_Unselect;
 	}
 
+	return SOH_Empty;
+}
+
+
+//
+// select all objects inside a given box
+//
+void SelectObjectsInBox(selection_c *list, int objtype, double x1, double y1, double x2, double y2)
+{
+	if (x2 < x1)
+		std::swap(x1, x2);
+
 	if (y2 < y1)
-	{
-		int tmp = y1; y1 = y2; y2 = tmp;
-	}
+		std::swap(y1, y2);
 
 	switch (objtype)
 	{
@@ -852,8 +871,10 @@ void SelectObjectsInBox(selection_c *list, int objtype, int x1, int y1, int x2,
 			{
 				const Thing *T = Things[n];
 
-				if (x1 <= T->x && T->x <= x2 &&
-				    y1 <= T->y && T->y <= y2)
+				double tx = T->x();
+				double ty = T->y();
+
+				if (x1 <= tx && tx <= x2 && y1 <= ty && ty <= y2)
 				{
 					list->toggle(n);
 				}
@@ -865,8 +886,10 @@ void SelectObjectsInBox(selection_c *list, int objtype, int x1, int y1, int x2,
 			{
 				const Vertex *V = Vertices[n];
 
-				if (x1 <= V->x && V->x <= x2 &&
-				    y1 <= V->y && V->y <= y2)
+				double vx = V->x();
+				double vy = V->y();
+
+				if (x1 <= vx && vx <= x2 && y1 <= vy && vy <= y2)
 				{
 					list->toggle(n);
 				}
@@ -879,10 +902,10 @@ void SelectObjectsInBox(selection_c *list, int objtype, int x1, int y1, int x2,
 				const LineDef *L = LineDefs[n];
 
 				/* the two ends of the line must be in the box */
-				if (x1 <= L->Start()->x && L->Start()->x <= x2 &&
-				    y1 <= L->Start()->y && L->Start()->y <= y2 &&
-				    x1 <= L->End()->x   && L->End()->x <= x2 &&
-				    y1 <= L->End()->y   && L->End()->y <= y2)
+				if (x1 <= L->Start()->x() && L->Start()->x() <= x2 &&
+				    y1 <= L->Start()->y() && L->Start()->y() <= y2 &&
+				    x1 <= L->End()->x()   && L->End()->x() <= x2 &&
+				    y1 <= L->End()->y()   && L->End()->y() <= y2)
 				{
 					list->toggle(n);
 				}
@@ -902,10 +925,10 @@ void SelectObjectsInBox(selection_c *list, int objtype, int x1, int y1, int x2,
 				int s1 = L->Right() ? L->Right()->sector : -1;
 				int s2 = L->Left( ) ? L->Left() ->sector : -1;
 
-				if (x1 <= L->Start()->x && L->Start()->x <= x2 &&
-				    y1 <= L->Start()->y && L->Start()->y <= y2 &&
-				    x1 <= L->End()->x   && L->End()->x <= x2 &&
-				    y1 <= L->End()->y   && L->End()->y <= y2)
+				if (x1 <= L->Start()->x() && L->Start()->x() <= x2 &&
+				    y1 <= L->Start()->y() && L->Start()->y() <= y2 &&
+				    x1 <= L->End()->x()   && L->End()->x() <= x2 &&
+				    y1 <= L->End()->y()   && L->End()->y() <= y2)
 				{
 					if (s1 >= 0) in_sectors.set(s1);
 					if (s2 >= 0) in_sectors.set(s2);
@@ -930,7 +953,8 @@ void SelectObjectsInBox(selection_c *list, int objtype, int x1, int y1, int x2,
 
 void Selection_InvalidateLast()
 {
-	delete last_Sel; last_Sel = NULL;
+	delete last_Sel;
+	last_Sel = NULL;
 }
 
 
@@ -947,7 +971,7 @@ void Selection_Push()
 	if (last_Sel)
 		delete last_Sel;
 
-	last_Sel = new selection_c(edit.Selected->what_type());
+	last_Sel = new selection_c(edit.Selected->what_type(), true);
 
 	last_Sel->merge(*edit.Selected);
 }
@@ -963,12 +987,92 @@ void Selection_Clear(bool no_save)
 
 	edit.error_mode = false;
 
-	Render3D_ClearSelection();
+	if (main_win)
+		main_win->UnselectPics();
 
 	RedrawMap();
 }
 
 
+void Selection_Add(Objid& obj)
+{
+	// validate the mode is correct
+	if (obj.type != edit.mode)
+		return;
+
+	if (obj.parts == 0)
+	{
+		// ignore the add if object is already set.
+		// [ since the selection may have parts, and we don't want to
+		//   forget those parts ]
+		if (! edit.Selected->get(obj.num))
+			edit.Selected->set(obj.num);
+		return;
+	}
+
+	byte cur = edit.Selected->get_ext(obj.num);
+
+	cur = 1 | obj.parts;
+
+	edit.Selected->set_ext(obj.num, cur);
+}
+
+
+void Selection_Remove(Objid& obj)
+{
+	if (obj.type != edit.mode)
+		return;
+
+	if (obj.parts == 0)
+	{
+		edit.Selected->clear(obj.num);
+		return;
+	}
+
+	byte cur = edit.Selected->get_ext(obj.num);
+	if (cur == 0)
+		return;
+
+	cur = 1 | (cur & ~obj.parts);
+
+	// if we unset all the parts, then unset the object itself
+	if (cur == 1)
+		cur = 0;
+
+	edit.Selected->set_ext(obj.num, cur);
+}
+
+
+void Selection_Toggle(Objid& obj)
+{
+	if (obj.type != edit.mode)
+		return;
+
+	if (obj.parts == 0)
+	{
+		edit.Selected->toggle(obj.num);
+		return;
+	}
+
+	byte cur = edit.Selected->get_ext(obj.num);
+
+	// if object was simply selected, then just clear it
+	if (cur == 1)
+	{
+		edit.Selected->clear(obj.num);
+		return;
+	}
+
+	cur = 1 | (cur ^ obj.parts);
+
+	// if we toggled off all the parts, then unset the object itself
+	if (cur == 1)
+		cur = 0;
+
+	edit.Selected->set_ext(obj.num, cur);
+}
+
+
 void Selection_Validate()
 {
 	int num_obj = NumObjects(edit.mode);
@@ -1011,90 +1115,6 @@ void CMD_LastSelection()
 }
 
 
-//----------------------------------------------------------------------
-//  3D CLIPBOARD
-//----------------------------------------------------------------------
-
-render_clipboard_c  r_clipboard;
-
-
-render_clipboard_c::render_clipboard_c() :
-	thing(0)
-{
-	tex [0] = 0;
-	flat[0] = 0;
-}
-
-render_clipboard_c::~render_clipboard_c()
-{ }
-
-
-int render_clipboard_c::GetTexNum()
-{
-	if (tex[0] == 0)
-		SetTex(default_wall_tex);
-
-	return BA_InternaliseString(tex);
-}
-
-int render_clipboard_c::GetFlatNum()
-{
-	if (flat[0] == 0)
-		SetFlat(default_floor_tex);
-
-	return BA_InternaliseString(flat);
-}
-
-int render_clipboard_c::GetThing()
-{
-	if (thing == 0)
-		thing = default_thing;
-
-	return thing;
-}
-
-
-void render_clipboard_c::SetTex(const char *new_tex)
-{
-	snprintf(tex, sizeof(tex), "%s", new_tex);
-}
-
-void render_clipboard_c::SetFlat(const char *new_flat)
-{
-	snprintf(flat, sizeof(flat), "%s", new_flat);
-}
-
-void render_clipboard_c::SetThing(int new_id)
-{
-	thing = new_id;
-}
-
-
-bool render_clipboard_c::ParseUser(const char ** tokens, int num_tok)
-{
-	if (strcmp(tokens[0], "r_clipboard") != 0)
-		return false;
-
-	if (strcmp(tokens[1], "tex") == 0)
-		SetTex(tokens[2]);
-
-	if (strcmp(tokens[1], "flat") == 0)
-		SetFlat(tokens[2]);
-
-	if (strcmp(tokens[1], "thing") == 0)
-		thing = atoi(tokens[2]);
-
-	return true;
-}
-
-void render_clipboard_c::WriteUser(FILE *fp)
-{
-	fprintf(fp, "r_clipboard tex \"%s\"\n",  StringTidy(tex,  "\""));
-	fprintf(fp, "r_clipboard flat \"%s\"\n", StringTidy(flat, "\""));
-	fprintf(fp, "r_clipboard thing %d\n",    thing);
-}
-
-
 //------------------------------------------------------------------------
 //  RECENTLY USED TEXTURES (etc)
 //------------------------------------------------------------------------
@@ -1137,7 +1157,7 @@ int Recently_used::find_number(int val)
 {
 	char buffer[64];
 
-	sprintf(buffer, "%d", val);
+	snprintf(buffer, sizeof(buffer), "%d", val);
 
 	return find(buffer);
 }
@@ -1174,7 +1194,7 @@ void Recently_used::insert_number(int val)
 {
 	char buffer[64];
 
-	sprintf(buffer, "%d", val);
+	snprintf(buffer, sizeof(buffer), "%d", val);
 
 	insert(buffer);
 }
@@ -1324,8 +1344,9 @@ void Editor_Init()
 	edit.pointer_in_window = false;
 	edit.map_x = 0;
 	edit.map_y = 0;
+	edit.map_z = -1;
 
-	edit.Selected = new selection_c(edit.mode);
+	edit.Selected = new selection_c(edit.mode, true /* extended */);
 
 	edit.highlight.clear();
 	edit.split_line.clear();
@@ -1344,10 +1365,10 @@ void Editor_DefaultState()
 {
 	edit.action = ACT_NOTHING;
 	edit.sticky_mod = 0;
-	edit.is_scrolling = false;
+	edit.is_panning = false;
 
-	edit.drawing_from = -1;
-	edit.drag_single_obj = -1;
+	edit.dragged.clear();
+	edit.draw_from.clear();
 
 	edit.error_mode = false;
 	edit.show_object_numbers = false;
