@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 André Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -32,6 +32,8 @@
 #include "m_events.h"
 #include "e_things.h"
 
+class SaveBucket_c;
+
 
 typedef enum
 {
@@ -39,6 +41,8 @@ typedef enum
 	SREND_Floor,
 	SREND_Ceiling,
 	SREND_Lighting,
+	SREND_FloorBright,
+	SREND_CeilBright,
 	SREND_SoundProp
 
 } sector_rendering_mode_e;
@@ -57,26 +61,16 @@ struct Editor_State_t
 
 	keycode_t sticky_mod;  // if != 0, waiting for next key  (fake meta)
 
-	bool is_scrolling;   // user is scrolling the map (or moving in 3D view)
-	bool is_navigating;  // user is holding down a navigation key
-
 	bool pointer_in_window;  // whether the mouse is over the 2D/3D view
-	float map_x, map_y;      // map coordinates of pointer
-
+	double map_x, map_y, map_z;  // map coordinates of pointer (no Z in 2D)
 
 	selection_c *Selected;    // all selected objects (usually empty)
 
 	Objid highlight;   // the highlighted object
 
 	Objid split_line;  // linedef which would be split by a new vertex
-	int split_x;
-	int split_y;
-
-	// the object that was under the pointer when ACT_Click occurred
-	Objid clicked;
-
-	int drawing_from;     // for ACT_DRAW_LINE: vertex we are drawing a line from
-	int drag_single_obj;  // for ACT_DRAG: an object number we are dragging, or -1 for selection
+	double split_x;
+	double split_y;
 
 
 	/* rendering stuff */
@@ -88,14 +82,88 @@ struct Editor_State_t
 
 	bool show_object_numbers;
 
-	/* private navigation stuff */
 
-	float nav_scroll_left;
-	float nav_scroll_right;
-	float nav_scroll_up;
-	float nav_scroll_down;
+	/* navigation stuff */
 
-	float scroll_speed;
+	bool is_navigating;  // user is holding down a navigation key
+	bool is_panning;     // user is panning the map (turning in 3D) via RMB
+
+	float nav_fwd,    nav_back;
+	float nav_left,   nav_right;
+	float nav_up,     nav_down;
+	float nav_turn_L, nav_turn_R;
+	bool  nav_lax;
+
+	float panning_speed;
+	bool  panning_lax;
+
+
+	/* click stuff (ACT_CLICK) */
+
+	Objid clicked;    // object under the pointer when ACT_Click occurred
+
+	int click_screen_x, click_screen_y;  // screen coord of the click
+
+	double click_map_x, click_map_y, click_map_z;  // location of the click
+
+	bool click_check_drag;
+	bool click_check_select;
+	bool click_force_single;
+
+
+	/* line drawing stuff (ACT_DRAW_LINE) */
+
+	Objid draw_from;  // the vertex we are drawing a line from
+
+	double draw_to_x, draw_to_y;  // target coordinate of current line
+
+
+	/* selection-box stuff (ACT_SELBOX) */
+
+	double selbox_x1, selbox_y1;  // map coords
+	double selbox_x2, selbox_y2;
+
+
+	/* transforming state (ACT_TRANSFORM) */
+
+	double trans_start_x;
+	double trans_start_y;
+
+	transform_keyword_e trans_mode;
+	transform_t trans_param;
+
+	selection_c *trans_lines;
+
+
+	/* dragging state (ACT_DRAG) */
+
+	Objid dragged;    // the object we are dragging, or nil for whole selection
+
+	int drag_screen_dx, drag_screen_dy;
+
+	double drag_start_x, drag_start_y, drag_start_z;
+	double drag_focus_x, drag_focus_y, drag_focus_z;
+	double drag_cur_x,   drag_cur_y,   drag_cur_z;
+
+	float drag_point_dist;
+	float drag_sector_dz;
+	int   drag_other_vert;  // used to ratio-lock a dragged vertex
+
+	int   drag_thing_num;
+	float drag_thing_floorh;
+	bool  drag_thing_up_down;
+
+	selection_c *drag_lines;
+
+
+	/* adjusting state (ACT_ADJUST_OFS) */
+
+	float adjust_dx, adjust_dy;
+	bool  adjust_lax;
+
+	SaveBucket_c * adjust_bucket;
+
+	struct { float x1, y1, x2, y2; } adjust_bbox;
 };
 
 
@@ -111,17 +179,16 @@ void Editor_ClearErrorMode();
 void Editor_ChangeMode(char mode);
 void Editor_ChangeMode_Raw(obj_type_e new_mode);
 
-bool GetCurrentObjects(selection_c *list);
 void UpdateHighlight();
 
 void RedrawMap();
 void ZoomWholeMap();
 
 
-extern int Map_bound_x1;   /* minimum X value of map */
-extern int Map_bound_y1;   /* minimum Y value of map */
-extern int Map_bound_x2;   /* maximum X value of map */
-extern int Map_bound_y2;   /* maximum Y value of map */
+extern double Map_bound_x1;   /* minimum X value of map */
+extern double Map_bound_y1;   /* minimum Y value of map */
+extern double Map_bound_x2;   /* maximum X value of map */
+extern double Map_bound_y2;   /* maximum Y value of map */
 
 void CalculateLevelBounds();
 
@@ -153,10 +220,25 @@ void ConvertSelection(selection_c * src, selection_c * dest);
 
 int Selection_FirstLine(selection_c *list);
 
+void Selection_Add(Objid& obj);
+void Selection_Remove(Objid& obj);
+void Selection_Toggle(Objid& obj);
+
 void Selection_Clear(bool no_save = false);
 void Selection_Push();
 void Selection_InvalidateLast();
 
+typedef enum
+{
+	SOH_OK = 0,       // using selection, nothing else needed
+	SOH_Unselect = 1, // using highlight, must unselect at end
+	SOH_Empty = 2     // both selection or highlight are empty
+} soh_type_e;
+
+soh_type_e Selection_Or_Highlight();
+
+void SelectObjectsInBox(selection_c *list, int objtype, double x1, double y1, double x2, double y2);
+
 
 /* commands */
 
@@ -164,35 +246,9 @@ void CMD_LastSelection();
 
 
 //----------------------------------------------------------------------
-//  3D Clipboard
+//  Helper for handling either the highlight or selection
 //----------------------------------------------------------------------
 
-class render_clipboard_c
-{
-private:
-	char tex [16];
-	char flat[16];
-
-	int  thing;
-
-public:
-	 render_clipboard_c();
-	~render_clipboard_c();
-
-	// when current one is unset, these return default_wall_tex (etc)
-	int GetTexNum();
-	int GetFlatNum();
-	int GetThing();
-
-	void SetTex(const char *new_tex);
-	void SetFlat(const char *new_flat);
-	void SetThing(int new_id);
-
-	bool ParseUser(const char ** tokens, int num_tok);
-	void WriteUser(FILE *fp);
-};
-
-extern render_clipboard_c  r_clipboard;
 
 
 //----------------------------------------------------------------------
