@@ -41,6 +41,47 @@
 #include "ui_window.h"
 
 
+// this function ensures that sector won't get floor > ceil
+void SEC_SafeRaiseLower(int sec, int parts, int dz)
+{
+	if (parts == 0)
+		parts = PART_FLOOR | PART_CEIL;
+
+	int f = Sectors[sec]->floorh;
+	int c = Sectors[sec]->ceilh;
+
+	if ((parts & PART_FLOOR) != 0 && (parts & PART_CEIL) != 0)
+	{
+		f += dz;
+		c += dz;
+
+		// this won't usually happen, only if original sector was bad
+		if (f > c)
+			f = c;
+	}
+	else if (parts & PART_FLOOR)
+	{
+		f += dz;
+
+		if (f > c)
+			f = c;
+	}
+	else if (parts & PART_CEIL)
+	{
+		c += dz;
+
+		if (c < f)
+			c = f;
+	}
+
+	if (parts & PART_FLOOR)
+		BA_ChangeSEC(sec, Sector::F_FLOORH, f);
+
+	if (parts & PART_CEIL)
+		BA_ChangeSEC(sec, Sector::F_CEILH, c);
+}
+
+
 void CMD_SEC_Floor(void)
 {
 	int diff = atoi(EXEC_Param[0]);
@@ -51,20 +92,17 @@ void CMD_SEC_Floor(void)
 		return;
 	}
 
-
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("No sectors to move");
 		return;
 	}
 
-
 	BA_Begin();
+	BA_MessageForSel(diff < 0 ? "lowered floor of" : "raised floor of", edit.Selected);
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const Sector *S = Sectors[*it];
 
@@ -73,11 +111,12 @@ void CMD_SEC_Floor(void)
 		BA_ChangeSEC(*it, Sector::F_FLOORH, new_h);
 	}
 
-	BA_MessageForSel(diff < 0 ? "lowered floor of" : "raised floor of", &list);
-
 	BA_End();
 
 	main_win->sec_box->UpdateField(Sector::F_FLOORH);
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
@@ -91,19 +130,17 @@ void CMD_SEC_Ceil(void)
 		return;
 	}
 
-
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("No sectors to move");
 		return;
 	}
 
 	BA_Begin();
+	BA_MessageForSel(diff < 0 ? "lowered ceil of" : "raised ceil of", edit.Selected);
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const Sector *S = Sectors[*it];
 
@@ -112,11 +149,12 @@ void CMD_SEC_Ceil(void)
 		BA_ChangeSEC(*it, Sector::F_CEILH, new_h);
 	}
 
-	BA_MessageForSel(diff < 0 ? "lowered ceil of" : "raised ceil of", &list);
-
 	BA_End();
 
 	main_win->sec_box->UpdateField(Sector::F_CEILH);
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
@@ -143,18 +181,15 @@ static int light_add_delta(int level, int delta)
 
 void SectorsAdjustLight(int delta)
 {
-	selection_c list;
-	selection_iterator_c it;
+	// this uses the current selection (caller must set it up)
 
-	if (! GetCurrentObjects(&list))
-	{
-		Beep("No sectors to adjust light");
+	if (edit.Selected->empty())
 		return;
-	}
 
 	BA_Begin();
+	BA_MessageForSel(delta < 0 ? "darkened" : "brightened", edit.Selected);
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const Sector *S = Sectors[*it];
 
@@ -163,8 +198,6 @@ void SectorsAdjustLight(int delta)
 		BA_ChangeSEC(*it, Sector::F_LIGHT, new_lt);
 	}
 
-	BA_MessageForSel(delta < 0 ? "darkened" : "brightened", &list);
-
 	BA_End();
 
 	main_win->sec_box->UpdateField(Sector::F_LIGHT);
@@ -181,24 +214,33 @@ void CMD_SEC_Light(void)
 		return;
 	}
 
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
+	{
+		Beep("No sectors to adjust light");
+		return;
+	}
+
 	SectorsAdjustLight(diff);
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
 void CMD_SEC_SwapFlats()
 {
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	soh_type_e unselect = Selection_Or_Highlight();
+	if (unselect == SOH_Empty)
 	{
 		Beep("No sectors to swap");
 		return;
 	}
 
 	BA_Begin();
+	BA_MessageForSel("swapped flats in", edit.Selected);
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const Sector *S = Sectors[*it];
 
@@ -209,12 +251,13 @@ void CMD_SEC_SwapFlats()
 		BA_ChangeSEC(*it, Sector::F_CEIL_TEX, floor_tex);
 	}
 
-	BA_MessageForSel("swapped flats in", &list);
-
 	BA_End();
 
 	main_win->sec_box->UpdateField(Sector::F_FLOOR_TEX);
 	main_win->sec_box->UpdateField(Sector::F_CEIL_TEX);
+
+	if (unselect == SOH_Unselect)
+		Selection_Clear(true /* nosave */);
 }
 
 
@@ -255,7 +298,7 @@ void CMD_SEC_Merge(void)
 	// need a selection
 	if (edit.Selected->count_obj() == 1 && edit.highlight.valid())
 	{
-		edit.Selected->set(edit.highlight.num);
+		Selection_Add(edit.highlight);
 	}
 
 	if (edit.Selected->count_obj() < 2)
@@ -272,9 +315,7 @@ void CMD_SEC_Merge(void)
 	// select the wrong sector afterwards (due to renumbering).
 	int new_sec = edit.Selected->max_obj();
 
-	selection_iterator_c it;
-
-	for (edit.Selected->begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		new_sec = MIN(new_sec, *it);
 	}
@@ -283,7 +324,6 @@ void CMD_SEC_Merge(void)
 	selection_c unused_secs (OBJ_SECTORS);
 
 	BA_Begin();
-
 	BA_MessageForSel("merged", edit.Selected);
 
 	// keep the properties of the first selected sector
@@ -301,7 +341,7 @@ void CMD_SEC_Merge(void)
 		BA_ChangeSEC(new_sec, Sector::F_TAG,   ref->tag);
 	}
 
-	for (edit.Selected->begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		int old_sec = *it;
 
@@ -455,7 +495,7 @@ int lineloop_c::NeighboringSector() const
 	// NOTE: it does not make sense to handle islands here.
 
 	int best = -1;
-	int best_len = -1;
+	double best_len = -1;
 
 	for (unsigned int i = 0 ; i < lines.size() ; i++)
 	{
@@ -467,9 +507,7 @@ int lineloop_c::NeighboringSector() const
 		if (sec < 0)
 			continue;
 
-		int dx = L->Start()->x - L->End()->x;
-		int dy = L->Start()->y - L->End()->y;
-		int len = ComputeDist(dx, dy);
+		double len = L->CalcLength();
 
 		if (len > best_len)
 		{
@@ -530,22 +568,22 @@ int lineloop_c::DetermineSector() const
 }
 
 
-void lineloop_c::CalcBounds(int *x1, int *y1, int *x2, int *y2) const
+void lineloop_c::CalcBounds(double *x1, double *y1, double *x2, double *y2) const
 {
 	SYS_ASSERT(lines.size() > 0);
 
-	*x1 = +99999; *y1 = +99999;
-	*x2 = -99999; *y2 = -99999;
+	*x1 = +9e9; *y1 = +9e9;
+	*x2 = -9e9; *y2 = -9e9;
 
 	for (unsigned int i = 0 ; i < lines.size() ; i++)
 	{
 		const LineDef *L = LineDefs[lines[i]];
 
-		*x1 = MIN(*x1, MIN(L->Start()->x, L->End()->x));
-		*y1 = MIN(*y1, MIN(L->Start()->y, L->End()->y));
+		*x1 = MIN(*x1, MIN(L->Start()->x(), L->End()->x()));
+		*y1 = MIN(*y1, MIN(L->Start()->y(), L->End()->y()));
 
-		*x2 = MAX(*x2, MAX(L->Start()->x, L->End()->x));
-		*y2 = MAX(*y2, MAX(L->Start()->y, L->End()->y));
+		*x2 = MAX(*x2, MAX(L->Start()->x(), L->End()->x()));
+		*y2 = MAX(*y2, MAX(L->Start()->y(), L->End()->y()));
 	}
 }
 
@@ -727,8 +765,7 @@ bool lineloop_c::LookForIsland()
 	//
 
 	// calc bounding box
-	int bbox_x1, bbox_y1, bbox_x2, bbox_y2;
-
+	double bbox_x1, bbox_y1, bbox_x2, bbox_y2;
 	CalcBounds(&bbox_x1, &bbox_y1, &bbox_x2, &bbox_y2);
 
 	int count = 0;
@@ -737,10 +774,10 @@ bool lineloop_c::LookForIsland()
 	{
 		const LineDef * L = LineDefs[ld];
 
-		int x1 = L->Start()->x;
-		int y1 = L->Start()->y;
-		int x2 = L->End()->x;
-		int y2 = L->End()->y;
+		double x1 = L->Start()->x();
+		double y1 = L->Start()->y();
+		double x2 = L->End()->x();
+		double y2 = L->End()->y();
 
 		if (MAX(x1, x2) < bbox_x1 || MIN(x1, x2) > bbox_x2 ||
 		    MAX(y1, y2) < bbox_y1 || MIN(y1, y2) > bbox_y2)
@@ -843,8 +880,8 @@ void lineloop_c::Dump() const
 		DebugPrintf("  %s of line #%d : (%d %d) --> (%d %d)\n",
 		            sides[i] == SIDE_LEFT ? " LEFT" : "RIGHT",
 					lines[i],
-					L->Start()->x, L->Start()->y,
-					L->End  ()->x, L->End  ()->y);
+					L->Start()->x(), L->Start()->y(),
+					L->End  ()->x(), L->End  ()->y());
 	}
 }
 
@@ -1005,13 +1042,13 @@ static void DetermineNewTextures(lineloop_c& loop,
 //
 void DoAssignSector(int ld, int side, int new_sec,
 					int new_lower, int new_upper,
-					selection_c& flip)
+					selection_c *flip)
 {
 // DebugPrintf("DoAssignSector %d ---> line #%d, side %d\n", new_sec, ld, side);
 	const LineDef * L = LineDefs[ld];
 
-	int sd_num   = (side > 0) ? L->right : L->left;
-	int other_sd = (side > 0) ? L->left  : L->right;
+	int sd_num   = (side == SIDE_RIGHT) ? L->right : L->left;
+	int other_sd = (side == SIDE_RIGHT) ? L->left  : L->right;
 
 	if (sd_num >= 0)
 	{
@@ -1024,10 +1061,10 @@ void DoAssignSector(int ld, int side, int new_sec,
 	// Thus we don't end up with invalid lines -- i.e. ones with a
 	// left side but no right side.
 
-	if (side < 0 && other_sd < 0)
-	{
-		flip.set(ld);
-	}
+	if (side == SIDE_LEFT && other_sd < 0)
+		flip->set(ld);
+	else
+		flip->clear(ld);
 
 	SYS_ASSERT(new_lower >= 0);
 	SYS_ASSERT(new_upper >= 0);
@@ -1068,7 +1105,7 @@ void DoAssignSector(int ld, int side, int new_sec,
 }
 
 
-void lineloop_c::AssignSector(int new_sec, selection_c& flip)
+void lineloop_c::AssignSector(int new_sec, selection_c *flip)
 {
 	std::vector<int> lower_texs(lines.size());
 	std::vector<int> upper_texs(lines.size());
@@ -1088,7 +1125,7 @@ void lineloop_c::AssignSector(int new_sec, selection_c& flip)
 }
 
 
-static bool GetLoopForSpace(int map_x, int map_y, lineloop_c& loop)
+static bool GetLoopForSpace(double map_x, double map_y, lineloop_c& loop)
 {
 	selection_c seen_lines(OBJ_LINEDEFS);
 
@@ -1162,7 +1199,7 @@ static bool GetLoopForSpace(int map_x, int map_y, lineloop_c& loop)
 // the 'model' is what properties to use for a new sector, < 0 means
 // look for a neighboring sector to copy.
 //
-bool AssignSectorToSpace(int map_x, int map_y, int new_sec, int model)
+bool AssignSectorToSpace(double map_x, double map_y, int new_sec, int model)
 {
 	lineloop_c loop;
 
@@ -1192,9 +1229,9 @@ bool AssignSectorToSpace(int map_x, int map_y, int new_sec, int model)
 
 	loop.GetAllSectors(&unused);
 
-	loop.AssignSector(new_sec, flip);
+	loop.AssignSector(new_sec, &flip);
 
-	FlipLineDefGroup(flip);
+	FlipLineDefGroup(&flip);
 
 	// detect any sectors which have become unused, and delete them
 	for (int n = 0 ; n < NumLineDefs ; n++)
