@@ -93,23 +93,25 @@ public:
 	// the 'flip' parameter will contain lines that should be flipped
 	// afterwards (to ensure it has a valid right side).
 	// 'new_sec' MUST be a valid sector number.
-	void AssignSector(int new_sec, selection_c& flip);
+	void AssignSector(int new_sec, selection_c *flip);
 
 	void Dump() const;
 
 private:
 	bool LookForIsland();
 
-	void CalcBounds(int *x1, int *y1, int *x2, int *y2) const;
+	void CalcBounds(double *x1, double *y1, double *x2, double *y2) const;
 };
 
 
 bool TraceLineLoop(int ld, int side, lineloop_c& loop, bool ignore_bare = false);
 
-bool AssignSectorToSpace(int map_x, int map_y, int new_sec = -1, int model = -1);
+bool AssignSectorToSpace(double map_x, double map_y, int new_sec = -1, int model = -1);
 
 void SectorsAdjustLight(int delta);
 
+void SEC_SafeRaiseLower(int sec, int parts, int dz);
+
 
 /* commands */
 
