@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -46,8 +46,6 @@ byte raw_palette[256][3];
 byte raw_colormap[32][256];
 
 
-extern int gammatable[5][256];
-
 byte rgb555_gamma [32];
 byte rgb555_medium[32];
 
@@ -253,13 +251,50 @@ rgb_color_t ParseColor(const char *str)
 
 rgb_color_t SectorLightColor(int light)
 {
-	float lt = CLAMP(0, (light | 15), 256) / 256.0;
+	int lt = light;
 
-	lt = pow(lt, 1.5);
+	// sample three distances, produce average
+	lt = R_DoomLightingEquation(light,  60.0) +
+		 R_DoomLightingEquation(light, 160.0) +
+		 R_DoomLightingEquation(light, 560.0);
 
-	light = CLAMP(48, (int)(lt * 256), 255);
+	lt = (93 - lt) * 255 / 93;
 
-	return RGB_MAKE(light, light, light);
+	// need to gamma-correct the light level
+	if (usegamma > 0)
+		lt = gammatable[usegamma][lt];
+
+	return RGB_MAKE(lt, lt, lt);
+}
+
+
+int HashedPalColor(const char *name, const int *cols)
+{
+	// cols is array of two elements: start and end color.
+	// this returns a palette color somewhere between start
+	// and end, depending on the texture name.
+
+	int len = (int)strlen(name);
+
+	int hash = name[0]*41;
+
+	if (len >= 2) hash += name[2]*13;
+	if (len >= 4) hash += name[4]*17;
+	if (len >= 5) hash += name[5]*7;
+	if (len >= 7) hash += name[7]*3;
+
+	hash ^= (hash >> 5);
+
+	int c1 = cols[0];
+	int c2 = cols[1];
+
+	if (c1 > c2)
+		std::swap(c1, c2);
+
+	if (c1 == c2)
+		return c1;
+
+	return c1 + hash % (c2 - c1 + 1);
 }
 
 
