@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 André Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -29,6 +29,12 @@
 
 #include "im_color.h"
 
+#ifdef NO_OPENGL
+typedef unsigned int GLuint;
+#else
+#include "FL/gl.h"
+#endif
+
 // this is a 16-bit value:
 //   - when high bit is clear, it is a palette index 0-255
 //     (value 255 is used to represent fully transparent).
@@ -56,26 +62,34 @@ private:
 	int  w;  // Width
 	int  h;  // Height
 
+	// texture identifier for OpenGL, 0 if not uploaded yet
+	GLuint gl_tex;
+
 public:
 	 Img_c();
 	 Img_c(int width, int height, bool _dummy = false);
 	~Img_c();
 
-	bool is_null() const
+	inline bool is_null() const
 	{
 		return (! pixels);
 	}
 
-	int width() const
+	inline int width() const
 	{
 		return w;
 	}
 
-	int height() const
+	inline int height() const
 	{
 		return h;
 	}
 
+	GLuint gl_texture() const
+	{
+		return gl_tex;
+	}
+
 	// read access
 	const img_pixel_t *buf() const;
 
@@ -88,6 +102,10 @@ public:
 
 	void resize(int new_width, int new_height);
 
+	// paste a copy of another image into this one, but skip any
+	// transparent pixels.
+	void compose(Img_c *other, int x, int y);
+
 	Img_c * spectrify() const;
 
 	Img_c * scale_img(double scale) const;
@@ -96,6 +114,14 @@ public:
 
 	bool has_transparent() const;
 
+	// upload to OpenGL, overwriting 'gl_tex' field.
+	void load_gl();
+
+	// invalidate the 'gl_tex' field, deleting old texture if possible.
+	void unload_gl(bool can_delete);
+
+	void bind_gl();
+
 	// convert pixels to RGB mode, for testing other code
 	void test_make_RGB();
 
@@ -106,15 +132,21 @@ private:
 
 
 void IM_ResetDummyTextures();
+void IM_UnloadDummyTextures();
 
 Img_c * IM_MissingTex();
 Img_c * IM_UnknownTex();
+Img_c * IM_SpecialTex();
 Img_c * IM_UnknownFlat();
 Img_c * IM_UnknownSprite();
 
 Img_c * IM_CreateFromText(int W, int H, const char **text, const rgb_color_t *palette, int pal_size);
 Img_c * IM_CreateDogSprite();
 Img_c * IM_CreateLightSprite();
+Img_c * IM_CreateMapSpotSprite(int base_r, int base_g, int base_b);
+
+Img_c * IM_DigitFont_11x14();
+Img_c * IM_DigitFont_14x19();
 
 Img_c * IM_ConvertRGBImage(Fl_RGB_Image *src);
 Img_c * IM_ConvertTGAImage(const rgba_color_t * data, int W, int H);
