@@ -36,6 +36,7 @@
 #include "r_grid.h"
 #include "r_render.h"
 #include "e_main.h"	  // RecUsed_xxx
+#include "w_wad.h"
 
 #include "ui_window.h"  // Browser_xxx, Props_xxx
 
@@ -86,6 +87,7 @@ typedef struct
 	// '<' : print extra newline after this option (when dumping)
 	// 'v' : a real variable (preference setting)
 	// 'w' : warp hack -- accept two numeric args
+	// 'H' : hide option from --help display
 
 	const char *desc;   // Description of the option
 	const char *arg_desc;  // Description of the argument (NULL --> none or default)
@@ -222,6 +224,15 @@ static const opt_desc_t options[] =
 		&Level_name
 	},
 
+	{	"udmftest",
+		0,
+		OPT_BOOLEAN,
+		"H",
+		"Enable the unfinished UDMF support",
+		NULL,
+		&udmf_testing
+	},
+
 	/* ------------ Preferences ------------ */
 
 	{	"auto_load_recent",
@@ -260,6 +271,15 @@ static const opt_desc_t options[] =
 		&backup_max_space
 	},
 
+	{	"browser_combine_tex",
+		0,
+		OPT_BOOLEAN,
+		"v",
+		"Combine flats and textures in a single browser",
+		NULL,
+		&browser_combine_tex
+	},
+
 	{	"browser_small_tex",
 		0,
 		OPT_BOOLEAN,
@@ -467,6 +487,33 @@ static const opt_desc_t options[] =
 		&grid_hide_in_free_mode
 	},
 
+	{	"grid_ratio_high",
+		0,
+		OPT_INTEGER,
+		"v",
+		"custom grid ratio : high value (numerator)",
+		NULL,
+		&grid_ratio_high
+	},
+
+	{	"grid_ratio_low",
+		0,
+		OPT_INTEGER,
+		"v",
+		"custom grid ratio : low value (denominator)",
+		NULL,
+		&grid_ratio_low
+	},
+
+	{	"grid_snap_indicator",
+		0,
+		OPT_BOOLEAN,
+		"v",
+		"show a cross at the grid-snapped location",
+		NULL,
+		&grid_snap_indicator
+	},
+
 	{	"grid_style",
 		0,
 		OPT_INTEGER,
@@ -521,6 +568,15 @@ static const opt_desc_t options[] =
 		&gui_custom_fg
 	},
 
+	{	"highlight_line_info",
+		0,
+		OPT_INTEGER,
+		"v",
+		"Info drawn near a highlighted line (0 = nothing)",
+		NULL,
+		&highlight_line_info
+	},
+
 	{	"leave_offsets_alone",
 		0,
 		OPT_BOOLEAN,
@@ -575,15 +631,6 @@ static const opt_desc_t options[] =
 		&minimum_drag_pixels
 	},
 
-	{	"new_islands_are_void",
-		0,
-		OPT_BOOLEAN,
-		"v",
-		"Islands created inside a sector will get a void interior",
-		NULL,
-		&new_islands_are_void
-	},
-
 	{	"new_sector_size",
 		0,
 		OPT_INTEGER,
@@ -647,11 +694,20 @@ static const opt_desc_t options[] =
 		&render_pixel_aspect
 	},
 
+	{	"render_far_clip",
+		0,
+		OPT_INTEGER,
+		"v",
+		"Distance of far clip plane for 3D rendering",
+		NULL,
+		&render_far_clip
+	},
+
 	{	"render_high_detail",
 		0,
 		OPT_BOOLEAN,
 		"v",
-		"Use highest detail when rendering the 3D view",
+		"Use highest detail when rendering 3D view (software mode)",
 		NULL,
 		&render_high_detail
 	},
@@ -955,11 +1011,10 @@ static int parse_a_config_file(FILE *fp, const char *filename)
 
 static const char * default_config_file()
 {
-	static char filename[FL_PATH_MAX];
-
 	SYS_ASSERT(home_dir);
 
-	sprintf(filename, "%s/config.cfg", home_dir);
+	static char filename[FL_PATH_MAX];
+	snprintf(filename, sizeof(filename), "%s/config.cfg", home_dir);
 
 	return StringDup(filename);
 }
@@ -999,7 +1054,7 @@ int M_ParseDefaultConfigFile()
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/defaults.cfg", install_dir);
+	snprintf(filename, sizeof(filename), "%s/defaults.cfg", install_dir);
 
 	FILE * fp = fopen(filename, "r");
 
@@ -1221,7 +1276,7 @@ void M_PrintCommandLineOptions(FILE *fp)
 	{
 		int len;
 
-		if (strchr(o->flags, 'v'))
+		if (strchr(o->flags, 'v') || strchr(o->flags, 'H'))
 			continue;
 
 		if (o->long_name)
@@ -1240,7 +1295,7 @@ void M_PrintCommandLineOptions(FILE *fp)
 	for (int pass = 0 ; pass < 2 ; pass++)
 	for (o = options; o->opt_type != OPT_END; o++)
 	{
-		if (strchr(o->flags, 'v'))
+		if (strchr(o->flags, 'v') || strchr(o->flags, 'H'))
 			continue;
 
 		if ((strchr(o->flags, '1') ? 1 : 0) != pass)
@@ -1355,8 +1410,10 @@ int M_WriteConfigFile()
 //------------------------------------------------------------------------
 
 
-int M_ParseLine(const char *line, const char ** tokens, int max_tok, bool do_strings)
+int M_ParseLine(const char *line, const char ** tokens, int max_tok, int do_strings)
 {
+	// when do_strings == 2, string tokens keep their quotes.
+
 	int num_tok = 0;
 
 	char tokenbuf[256];
@@ -1396,7 +1453,9 @@ int M_ParseLine(const char *line, const char ** tokens, int max_tok, bool do_str
 			if (ch == '"' && do_strings)
 			{
 				in_string = true;
-				continue;
+
+				if (do_strings != 2)
+					continue;
 			}
 
 			// begin a normal token
@@ -1408,6 +1467,9 @@ int M_ParseLine(const char *line, const char ** tokens, int max_tok, bool do_str
 		{
 			// end of string
 			in_string = false;
+
+			if (do_strings == 2)
+				tokenbuf[tokenlen++] = ch;
 		}
 		else if (ch == 0 || ch == '\n')
 		{
@@ -1459,8 +1521,8 @@ char * PersistFilename(const crc32_c& crc)
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/cache/%08X%08X.dat", cache_dir,
-	        crc.extra, crc.raw);
+	snprintf(filename, sizeof(filename), "%s/cache/%08X%08X.dat",
+			cache_dir, crc.extra, crc.raw);
 
 	return filename;
 }
@@ -1491,7 +1553,7 @@ bool M_LoadUserState()
 
 	while (M_ReadTextLine(line, sizeof(line), fp))
 	{
-		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, true /* do_strings */);
+		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, 1 /* do_strings */);
 
 		if (num_tok == 0)
 			continue;
