@@ -39,7 +39,6 @@ extern bool begin_maximized;
 extern bool map_scroll_bars;
 
 extern bool leave_offsets_alone;
-extern bool new_islands_are_void;
 extern bool same_mode_clears_selection;
 
 extern bool swap_sidedefs;
@@ -55,6 +54,7 @@ extern rgb_color_t gui_custom_fg;
 extern int panel_gamma;
 
 extern int  minimum_drag_pixels;
+extern int  highlight_line_info;
 extern int  new_sector_size;
 extern int  sector_render_default;
 extern int   thing_render_default;
@@ -64,6 +64,9 @@ extern int  grid_default_mode;
 extern bool grid_default_snap;
 extern int  grid_default_size;
 extern bool grid_hide_in_free_mode;
+extern bool grid_snap_indicator;
+extern int  grid_ratio_high;
+extern int  grid_ratio_low;
 
 extern rgb_color_t dotty_axis_col;
 extern rgb_color_t dotty_major_col;
@@ -79,6 +82,7 @@ extern int backup_max_files;
 extern int backup_max_space;
 
 extern bool browser_small_tex;
+extern bool browser_combine_tex;
 
 extern int floor_bump_small;
 extern int floor_bump_medium;
@@ -89,6 +93,7 @@ extern int light_bump_medium;
 extern int light_bump_large;
 
 extern int  render_pixel_aspect;
+extern int  render_far_clip;
 extern bool render_high_detail;
 extern bool render_lock_gravity;
 extern bool render_missing_bright;
@@ -125,8 +130,7 @@ bool M_ReadTextLine(char *buf, size_t size, FILE *fp);
 
 
 // returns number of tokens, zero for comment, negative on error
-int M_ParseLine(const char *line, const char ** tokens, int max_tok,
-                bool do_strings);
+int M_ParseLine(const char *line, const char ** tokens, int max_tok, int do_strings);
 void M_FreeLine(const char ** tokens, int num_tok);
 
 // user state persistence (stuff like camera pos, grid settings, ...)
