@@ -94,21 +94,6 @@ void Editor_SetAction(editor_action_e  new_action)
 }
 
 
-// FIXME : this was being called by CMD_Scroll, but smells very hacky
-#if 0
-void Editor_UpdateFromScroll()
-{
-	if (edit.action == ACT_SELBOX || edit.action == ACT_DRAW_LINE ||
-		edit.action == ACT_DRAG)
-	{
-		int mod = Fl::event_state() & MOD_ALL_MASK;
-
-		EV_MouseMotion(Fl::event_x(), Fl::event_y(), mod, 0, 0);
-	}
-}
-#endif
-
-
 void Editor_Zoom(int delta, int mid_x, int mid_y)
 {
     float S1 = grid.Scale;
@@ -120,12 +105,12 @@ void Editor_Zoom(int delta, int mid_x, int mid_y)
 
 
 // this is only used for mouse scrolling
-void Editor_ScrollMap(int mode, int dx, int dy)
+void Editor_ScrollMap(int mode, int dx, int dy, keycode_t mod)
 {
 	// started?
 	if (mode < 0)
 	{
-		edit.is_scrolling = true;
+		edit.is_panning = true;
 		main_win->SetCursor(FL_CURSOR_HAND);
 		return;
 	}
@@ -133,49 +118,69 @@ void Editor_ScrollMap(int mode, int dx, int dy)
 	// finished?
 	if (mode > 0)
 	{
-		edit.is_scrolling = false;
+		edit.is_panning = false;
 		main_win->SetCursor(FL_CURSOR_DEFAULT);
 		return;
 	}
 
-	float speed = edit.scroll_speed / grid.Scale;
+	if (! edit.panning_lax)
+		mod = 0;
 
-	//??		 if (mod & MOD_COMMAND) speed *= 2.0;
-	//??	else if (mod & MOD_SHIFT)   speed *= 0.5;
+	if (dx == 0 && dy == 0)
+		return;
 
-	double delta_x = ((double) -dx * speed);
-	double delta_y = ((double)  dy * speed);
+	if (edit.render3d)
+	{
+		Render3D_ScrollMap(dx, dy, mod);
+	}
+	else
+	{
+		float speed = edit.panning_speed / grid.Scale;
 
-	grid.Scroll(delta_x, delta_y);
+		double delta_x = ((double) -dx * speed);
+		double delta_y = ((double)  dy * speed);
+
+		grid.Scroll(delta_x, delta_y);
+	}
 }
 
 
 void Editor_ClearNav()
 {
-	edit.nav_scroll_left  = 0;
-	edit.nav_scroll_right = 0;
-	edit.nav_scroll_up    = 0;
-	edit.nav_scroll_down  = 0;
+	edit.nav_left  = 0;
+	edit.nav_right = 0;
+	edit.nav_up    = 0;
+	edit.nav_down  = 0;
+
+	edit.nav_fwd    = 0;
+	edit.nav_back   = 0;
+	edit.nav_turn_L = 0;
+	edit.nav_turn_R = 0;
+
+	edit.nav_lax = false;
 }
 
 
-static void Editor_Navigate()
+static void Navigate2D()
 {
 	float delay_ms = Nav_TimeDiff();
 
 	delay_ms = delay_ms / 1000.0;
 
-	keycode_t mod = M_ReadLaxModifiers();
+	keycode_t mod = 0;
+
+	if (edit.nav_lax)
+		mod = M_ReadLaxModifiers();
 
 	float mod_factor = 1.0;
 	if (mod & MOD_SHIFT)   mod_factor = 0.5;
 	if (mod & MOD_COMMAND) mod_factor = 2.0;
 
-	if (edit.nav_scroll_left || edit.nav_scroll_right ||
-		edit.nav_scroll_up   || edit.nav_scroll_down)
+	if (edit.nav_left || edit.nav_right ||
+		edit.nav_up   || edit.nav_down)
 	{
-		float delta_x = (edit.nav_scroll_right - edit.nav_scroll_left);
-		float delta_y = (edit.nav_scroll_up    - edit.nav_scroll_down);
+		float delta_x = (edit.nav_right - edit.nav_left);
+		float delta_y = (edit.nav_up    - edit.nav_down);
 
 		delta_x = delta_x * mod_factor * delay_ms;
 		delta_y = delta_y * mod_factor * delay_ms;
@@ -215,8 +220,7 @@ static unsigned int nav_time;
 
 void Nav_Clear()
 {
-	  Editor_ClearNav();
-	Render3D_ClearNav();
+	Editor_ClearNav();
 
 	memset(nav_actives, 0, sizeof(nav_actives));
 
@@ -229,7 +233,7 @@ void Nav_Navigate()
 	if (edit.render3d)
 		Render3D_Navigate();
 	else
-		Editor_Navigate();
+		Navigate2D();
 }
 
 
@@ -241,7 +245,8 @@ bool Nav_SetKey(keycode_t key, nav_release_func_t func)
 
 	keycode_t lax_mod = 0;
 
-	if (Exec_HasFlag("/LAX"))
+	edit.nav_lax = Exec_HasFlag("/LAX");
+	if (edit.nav_lax)
 		lax_mod = MOD_SHIFT | MOD_COMMAND;
 
 	edit.is_navigating = true;
@@ -423,7 +428,10 @@ unsigned int Nav_TimeDiff()
 int wheel_dx;
 int wheel_dy;
 
+static bool in_operation_menu;
+
 extern void CheckBeginDrag();
+extern void Transform_Update();
 
 
 static void EV_EnterWindow()
@@ -441,9 +449,6 @@ static void EV_EnterWindow()
 	// restore keyboard focus to the canvas
 	Fl_Widget * foc = main_win->canvas;
 
-	if (edit.render3d)
-		foc = main_win->render;
-
 	if (Fl::focus() != foc)
 		foc->take_focus();
 
@@ -453,6 +458,11 @@ static void EV_EnterWindow()
 
 static void EV_LeaveWindow()
 {
+	// ignore FL_LEAVE event when doing a popup operation menu,
+	// otherwise we lose the highlight and kill drawing mode.
+	if (in_operation_menu)
+		return;
+
 	edit.pointer_in_window = false;
 
 	// this offers a handy way to get out of drawing mode
@@ -464,6 +474,23 @@ static void EV_LeaveWindow()
 }
 
 
+void EV_EscapeKey()
+{
+	Nav_Clear();
+	ClearStickyMod();
+	Editor_ClearAction();
+	Status_Clear();
+
+	edit.clicked.clear();
+	edit.dragged.clear();
+	edit.split_line.clear();
+	edit.draw_from.clear();
+
+	UpdateHighlight();
+	RedrawMap();
+}
+
+
 void EV_MouseMotion(int x, int y, keycode_t mod, int dx, int dy)
 {
 	// this is probably redundant, as we generally shouldn't get here
@@ -474,9 +501,9 @@ void EV_MouseMotion(int x, int y, keycode_t mod, int dx, int dy)
 
 //  fprintf(stderr, "MOUSE MOTION: (%d %d)  map: (%1.2f %1.2f)\n", x, y, edit.map_x, edit.map_y);
 
-	if (edit.is_scrolling)
+	if (edit.is_panning)
 	{
-		Editor_ScrollMap(0, dx, dy);
+		Editor_ScrollMap(0, dx, dy, mod);
 		return;
 	}
 
@@ -484,33 +511,48 @@ void EV_MouseMotion(int x, int y, keycode_t mod, int dx, int dy)
 
 	if (edit.action == ACT_TRANSFORM)
 	{
-		main_win->canvas->TransformUpdate(edit.map_x, edit.map_y);
+		Transform_Update();
 		return;
 	}
 
 	if (edit.action == ACT_DRAW_LINE)
 	{
+		// this calls UpdateHighlight() which updates edit.draw_to_x/y
 		RedrawMap();
 		return;
 	}
 
 	if (edit.action == ACT_SELBOX)
 	{
-		main_win->canvas->SelboxUpdate(edit.map_x, edit.map_y);
+		edit.selbox_x2 = edit.map_x;
+		edit.selbox_y2 = edit.map_y;
+
+		main_win->canvas->redraw();
 		return;
 	}
 
 	if (edit.action == ACT_DRAG)
 	{
-		main_win->canvas->DragUpdate(edit.map_x, edit.map_y);
+		edit.drag_screen_dx = x - edit.click_screen_x;
+		edit.drag_screen_dy = y - edit.click_screen_y;
 
-		// if dragging a single vertex, update the possible split_line
-		UpdateHighlight();
+		edit.drag_cur_x = edit.map_x;
+		edit.drag_cur_y = edit.map_y;
+
+		// if dragging a single vertex, update the possible split_line.
+		// Note: ratio-lock is handled in UI_Canvas::DragDelta
+		if (edit.mode == OBJ_VERTICES && edit.dragged.valid())
+			UpdateHighlight();
+
+		main_win->canvas->redraw();
 		return;
 	}
 
 	// begin dragging?
-	CheckBeginDrag();
+	if (edit.action == ACT_CLICK)
+	{
+		CheckBeginDrag();
+	}
 
 	// in general, just update the highlight, split-line (etc)
 	UpdateHighlight();
@@ -760,8 +802,7 @@ int EV_HandleEvent(int event)
 
 static bool no_operation_cfg;
 
-// the value of edit.highlight before the menu was opened
-static Objid op_saved_highlight;
+static std::map< std::string, Fl_Menu_Button* > op_all_menus;
 
 
 typedef struct
@@ -773,39 +814,33 @@ typedef struct
 } operation_command_t;
 
 
-static void operation_callback_func(Fl_Widget *w, void *data)
-{
-	operation_command_t *info = (operation_command_t *)data;
-
-	// restore the highlight object
-	edit.highlight = op_saved_highlight;
-
-	Render3D_RestoreHighlight();
-
-	// TODO : support more than 4 parameters
-
-	ExecuteCommand(info->cmd, info->param[0], info->param[1],
-				   info->param[2], info->param[3]);
-}
-
-
 static void ParseOperationLine(const char ** tokens, int num_tok,
 							   Fl_Menu_Button *menu)
 {
-	if (num_tok < 2)
-		FatalError("Bad operations menu : missing description.\n");
-
-	// just a spacer?
-	if (tokens[1][0] == '_')
+	// just a divider?
+	if (y_stricmp(tokens[0], "divider") == 0)
 	{
 		menu->add("", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE);
 		return;
 	}
 
-	if (num_tok < 3)
-		FatalError("Bad operations menu : missing command name.\n");
+	// parse the key
+	int shortcut = 0;
+
+	if (y_stricmp(tokens[0], "UNBOUND") != 0)
+	{
+		keycode_t key = M_ParseKeyString(tokens[0]);
+		if (key != 0)
+			shortcut = M_KeyToShortcut(key);
+	}
 
 	// parse the command and its parameters...
+	if (num_tok < 2)
+		FatalError("operations.cfg: entry missing description.\n");
+
+	if (num_tok < 3)
+		FatalError("operations.cfg: entry missing command name.\n");
+
 	const editor_command_t *cmd = FindEditorCommand(tokens[2]);
 
 	if (! cmd)
@@ -825,72 +860,15 @@ static void ParseOperationLine(const char ** tokens, int num_tok,
 		if (num_tok >= 4 + p)
 			strncpy(info->param[p], tokens[3 + p], MAX_BIND_LENGTH-1);
 
-	menu->add(tokens[1], 0 /* shortcut */, &operation_callback_func,
-			  (void *)info, 0 /* flags */);
+	menu->add(tokens[1], shortcut, 0 /* callback */, (void *)info, 0 /* flags */);
 }
 
 
-#define MAX_TOKENS  30
-
-static void M_ParseOperationFile(const char *context, Fl_Menu_Button *menu)
+static void M_AddOperationMenu(std::string context, Fl_Menu_Button *menu)
 {
-	// open the file and build the menu from all line whose first
-	// keyword matches the given 'context'.
-
-	static char filename[FL_PATH_MAX];
-
-	// look in user's $HOME directory first
-	sprintf(filename, "%s/operations.cfg", home_dir);
-
-	FILE *fp = fopen(filename, "r");
-
-	// otherwise load it from the installation directory
-	if (! fp)
-	{
-		sprintf(filename, "%s/operations.cfg", install_dir);
-
-		fp = fopen(filename, "r");
-	}
-
-	if (! fp)
-	{
-		no_operation_cfg = true;
-		return;
-	}
-
-	menu->clear();
-
-
-	// parse each line
-
-	static char line[FL_PATH_MAX];
-	const  char * tokens[MAX_TOKENS];
-
-	while (M_ReadTextLine(line, sizeof(line), fp))
-	{
-		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, true /* do_strings */);
-		if (num_tok == 0)
-			continue;
-
-		if (num_tok < 0)
-		{
-			LogPrintf("operations.cfg: failed parsing a line\n");
-			continue;
-		}
-
-		// first word is the context, require a match
-		if (y_stricmp(tokens[0], context) != 0)
-			continue;
-
-		ParseOperationLine(tokens, num_tok, menu);
-	}
-
-	fclose(fp);
-
-
 	if (menu->size() < 2)
 	{
-		FatalError("Bad operations menu : no %s items.\n", context);
+		FatalError("operations.cfg: no %s items.\n", context.c_str());
 		return;
 	}
 
@@ -904,11 +882,96 @@ static void M_ParseOperationFile(const char *context, Fl_Menu_Button *menu)
 	// the boxtype MUST be FL_NO_BOX
 	menu->box(FL_NO_BOX);
 
-	menu->textsize(KF_fonth);
+	menu->textsize(16);
 
 	// NOTE: we cannot show() this menu, as that will interfere with
 	// mouse motion events [ canvas will get FL_LEAVE when the mouse
 	// enters this menu's bbox ]
+
+	menu->hide();
+
+	op_all_menus[context] = menu;
+
+	main_win->add(menu);
+}
+
+
+#define MAX_TOKENS  30
+
+static bool M_ParseOperationFile()
+{
+	// open the file and build all the menus it contains.
+
+	static char filename[FL_PATH_MAX];
+
+	// look in user's $HOME directory first
+	snprintf(filename, sizeof(filename), "%s/operations.cfg", home_dir);
+
+	FILE *fp = fopen(filename, "r");
+
+	// otherwise load it from the installation directory
+	if (! fp)
+	{
+		snprintf(filename, sizeof(filename), "%s/operations.cfg", install_dir);
+
+		fp = fopen(filename, "r");
+	}
+
+	if (! fp)
+		return false;
+
+
+	// parse each line
+
+	static char line[FL_PATH_MAX];
+	const  char * tokens[MAX_TOKENS];
+
+	Fl_Menu_Button *menu = NULL;
+
+	std::string context;
+
+	while (M_ReadTextLine(line, sizeof(line), fp))
+	{
+		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, 1 /* do_strings */);
+		if (num_tok == 0)
+			continue;
+
+		if (num_tok < 0)
+		{
+			LogPrintf("operations.cfg: failed parsing a line\n");
+			continue;
+		}
+
+		if (y_stricmp(tokens[0], "menu") == 0)
+		{
+			if (num_tok < 3)
+			{
+				LogPrintf("operations.cfg: bad menu line\n");
+				continue;
+			}
+
+			if (menu != NULL)
+				M_AddOperationMenu(context, menu);
+
+			// create new menu
+			menu = new Fl_Menu_Button(0, 0, 99, 99, "");
+			menu->copy_label(tokens[2]);
+			menu->clear();
+
+			context = tokens[1];
+			continue;
+		}
+
+		if (menu != NULL)
+			ParseOperationLine(tokens, num_tok, menu);
+	}
+
+	fclose(fp);
+
+	if (menu != NULL)
+		M_AddOperationMenu(context, menu);
+
+	return true;
 }
 
 
@@ -916,16 +979,11 @@ void M_LoadOperationMenus()
 {
 	LogPrintf("Loading Operation menus...\n");
 
-	no_operation_cfg = false;
-
-	M_ParseOperationFile("thing",  main_win->op_thing);
-	M_ParseOperationFile("line",   main_win->op_line);
-	M_ParseOperationFile("sector", main_win->op_sector);
-	M_ParseOperationFile("vertex", main_win->op_vertex);
-	M_ParseOperationFile("render", main_win->op_render);
-
-	if (no_operation_cfg)
+	if (! M_ParseOperationFile())
+	{
+		no_operation_cfg = true;
 		DLG_Notify("Installation problem: cannot find \"operaitons.cfg\" file!");
+	}
 }
 
 
@@ -934,25 +992,33 @@ void CMD_OperationMenu()
 	if (no_operation_cfg)
 		return;
 
-	Fl_Menu_Button *menu = NULL;
+	const char *context = EXEC_Param[0];
 
-	if (edit.render3d)
+	// if no context given, pick one based on editing mode
+	if (! context[0])
 	{
-		menu = main_win->op_render;
-	}
-	else
-	{
-		switch (edit.mode)
+		if (edit.render3d)
 		{
-			case OBJ_THINGS:	menu = main_win->op_thing;  break;
-			case OBJ_LINEDEFS:	menu = main_win->op_line;   break;
-			case OBJ_SECTORS:	menu = main_win->op_sector; break;
-			case OBJ_VERTICES:	menu = main_win->op_vertex; break;
-
-			default:
-				Beep("a strange case indeed!");
-				return;
+			context = "render";
 		}
+		else
+		{
+			switch (edit.mode)
+			{
+			case OBJ_LINEDEFS:	context = "line";   break;
+			case OBJ_SECTORS:	context = "sector"; break;
+			case OBJ_VERTICES:	context = "vertex"; break;
+			default: context = "thing";
+			}
+		}
+	}
+
+	Fl_Menu_Button *menu = op_all_menus[context];
+
+	if (menu == NULL)
+	{
+		Beep("no such menu: %s", context);
+		return;
 	}
 
 	SYS_ASSERT(menu);
@@ -962,13 +1028,23 @@ void CMD_OperationMenu()
 	// especially if the last command was destructive.
 	menu->value((const Fl_Menu_Item *)NULL);
 
-	// save the highlight object, because the pop-up menu will cause
-	// an FL_LEAVE event on the canvas which resets the highlight.
-	op_saved_highlight = edit.highlight;
+	// the pop-up menu will cause an FL_LEAVE event on the canvas,
+	// which we need to ignore to prevent forgetting the current
+	// highlight or killing line drawing mode.
+	in_operation_menu = true;
 
-	Render3D_SaveHighlight();
+	const Fl_Menu_Item *item = menu->popup();
 
-	menu->popup();
+	in_operation_menu = false;
+
+	if (item)
+	{
+		operation_command_t *info = (operation_command_t *)item->user_data();
+		SYS_ASSERT(info);
+
+		ExecuteCommand(info->cmd, info->param[0], info->param[1],
+					   info->param[2], info->param[3]);
+	}
 }
 
 
