@@ -367,15 +367,17 @@ public:
 		}
 	}
 
-	void Format(char *buffer, int index) const
+	std::string Format(int index) const
 	{
 		SYS_ASSERT(index < size);
 
 		const char *name = fl_filename_name(filenames[index]);
-		// const char *map  = map_names[index];
 
-		sprintf(buffer, "%s%s%d:  %-.42s", (index < 9) ? "  " : "",
+		char buffer[256];
+		snprintf(buffer, sizeof(buffer), "%s%s%d:  %-.42s", (index < 9) ? "  " : "",
 				(index < 9) ? "&" : "", 1+index, name);
+
+		return std::string(buffer);
 	}
 
 	void Lookup(int index, const char ** file_v, const char ** map_v)
@@ -457,7 +459,7 @@ void M_LoadRecent()
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/misc.cfg", home_dir);
+	snprintf(filename, sizeof(filename), "%s/misc.cfg", home_dir);
 
 	FILE *fp = fopen(filename, "r");
 
@@ -483,7 +485,7 @@ void M_SaveRecent()
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/misc.cfg", home_dir);
+	snprintf(filename, sizeof(filename), "%s/misc.cfg", home_dir);
 
 	FILE *fp = fopen(filename, "w");
 
@@ -513,9 +515,9 @@ int M_RecentCount()
 	return recent_files.getSize();
 }
 
-void M_RecentShortName(int index, char *name_buf)
+std::string M_RecentShortName(int index)
 {
-	recent_files.Format(name_buf, index);
+	return recent_files.Format(index);
 }
 
 void * M_RecentData(int index)
@@ -649,7 +651,7 @@ static const char * SearchDirForIWAD(const char *dir_name, const char *game)
 {
 	char name_buf[FL_PATH_MAX];
 
-	sprintf(name_buf, "%s/%s.wad", dir_name, game);
+	snprintf(name_buf, sizeof(name_buf), "%s/%s.wad", dir_name, game);
 
 	DebugPrintf("  trying: %s\n", name_buf);
 
@@ -1086,7 +1088,7 @@ static const char *Backup_Name(const char *dir_name, int slot)
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/%d.wad", dir_name, slot);
+	snprintf(filename, sizeof(filename), "%s/%d.wad", dir_name, slot);
 
 	return filename;
 }
@@ -1122,7 +1124,7 @@ void M_BackupWad(Wad_file *wad)
 
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/backups/%s", cache_dir, fl_filename_name(wad->PathName()));
+	snprintf(filename, sizeof(filename), "%s/backups/%s", cache_dir, fl_filename_name(wad->PathName()));
 
 	char * dir_name = ReplaceExtension(filename, NULL);
 
