@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2013-2018 Andrew Apted
+//  Copyright (C) 2013-2019 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -40,15 +40,14 @@ keycode_t EXEC_CurKey;
 static std::vector< editor_command_t * > all_commands;
 
 
-static key_context_e ContextFromName(const char *name)
+static key_context_e RequiredContextFromName(const char *name)
 {
 	if (strncmp(name, "LIN_", 4) == 0) return KCTX_Line;
 	if (strncmp(name, "SEC_", 4) == 0) return KCTX_Sector;
 	if (strncmp(name, "TH_",  3) == 0) return KCTX_Thing;
 	if (strncmp(name, "VT_",  3) == 0) return KCTX_Vertex;
-	if (strncmp(name, "3D_",  3) == 0) return KCTX_Render;
 
-	// we don't need anything for KCTX_Browser
+	// we don't need anything for KCTX_Browser or KCTX_Render
 
 	return KCTX_NONE;
 }
@@ -73,25 +72,6 @@ static const char * CalcGroupName(const char *given, key_context_e ctx)
 }
 
 
-//
-// this should only be called during startup
-//
-void M_RegisterCommand(const char *name, command_func_t func, const char *group_name)
-{
-	editor_command_t *cmd = new editor_command_t;
-
-	cmd->name = name;
-	cmd->func = func;
-	cmd->flag_list = NULL;
-	cmd->keyword_list = NULL;
-
-	cmd->req_context = ContextFromName(name);
-	cmd->group_name  = CalcGroupName(group_name, cmd->req_context);
-
-	all_commands.push_back(cmd);
-}
-
-
 //
 // this should only be called during startup
 //
@@ -101,7 +81,7 @@ void M_RegisterCommandList(editor_command_t * list)
 
 	for ( ; list->name ; list++)
 	{
-		list->req_context = ContextFromName(list->name);
+		list->req_context = RequiredContextFromName(list->name);
 		list->group_name  = CalcGroupName(list->group_name, list->req_context);
 
 		all_commands.push_back(list);
@@ -111,6 +91,18 @@ void M_RegisterCommandList(editor_command_t * list)
 
 const editor_command_t * FindEditorCommand(const char *name)
 {
+	// backwards compatibility
+	if (y_stricmp(name, "GRID_Step") == 0)
+		name = "GRID_Bump";
+	else if (y_stricmp(name, "Check") == 0)
+		name = "MapCheck";
+	else if (y_stricmp(name, "3D_Click") == 0)
+		name = "ACT_Click";
+	else if (y_stricmp(name, "3D_NAV_MouseMove") == 0)
+		name = "NAV_MouseScroll";
+	else if (y_stricmp(name, "OperationMenu") == 0)
+		name = "OpMenu";
+
 	for (unsigned int i = 0 ; i < all_commands.size() ; i++)
 		if (y_stricmp(all_commands[i]->name, name) == 0)
 			return all_commands[i];
@@ -142,6 +134,7 @@ typedef struct
 static const key_mapping_t key_map[] =
 {
 	{ ' ',			"SPACE" },
+	{ '"',			"DBLQUOTE" },
 	{ FL_BackSpace,	"BS" },
 	{ FL_Tab,		"TAB" },
 	{ FL_Enter,		"ENTER" },
@@ -282,7 +275,7 @@ static const char * BareKeyName(keycode_t key)
 {
 	static char buffer[200];
 
-	if (key < 127 && key > 32 && isprint(key))
+	if (key < 127 && key > 32 && isprint(key) && key != '"')
 	{
 		buffer[0] = (char) key;
 		buffer[1] = 0;
@@ -292,13 +285,13 @@ static const char * BareKeyName(keycode_t key)
 
 	if (FL_F < key && key <= FL_F_Last)
 	{
-		sprintf(buffer, "F%d", key - FL_F);
+		snprintf(buffer, sizeof(buffer), "F%d", key - FL_F);
 		return buffer;
 	}
 
 	if (is_mouse_button(key))
 	{
-		sprintf(buffer, "MOUSE%d", key - FL_Button);
+		snprintf(buffer, sizeof(buffer), "MOUSE%d", key - FL_Button);
 		return buffer;
 	}
 
@@ -309,14 +302,13 @@ static const char * BareKeyName(keycode_t key)
 
 	if (FL_KP + 33 <= key && key <= FL_KP_Last)
 	{
-		sprintf(buffer, "KP_%c", (char)(key & 127));
+		snprintf(buffer, sizeof(buffer), "KP_%c", (char)(key & 127));
 		return buffer;
 	}
 
 	// fallback : hex code
 
-	sprintf(buffer, "0x%04x", key);
-
+	snprintf(buffer, sizeof(buffer), "0x%04x", key);
 	return buffer;
 }
 
@@ -364,13 +356,12 @@ const char * M_KeyToString(keycode_t key)
 		(key & FL_KEY_MASK)  <  127 &&
 		isalpha(key & FL_KEY_MASK))
 	{
-		sprintf(buffer, "%c", toupper(key & FL_KEY_MASK));
+		snprintf(buffer, sizeof(buffer), "%c", toupper(key & FL_KEY_MASK));
 		return buffer;
 	}
 
-	strcpy(buffer, ModName_Dash(key));
-
-	strcat(buffer, BareKeyName(key & FL_KEY_MASK));
+	snprintf(buffer, sizeof(buffer), "%s%s", ModName_Dash(key),
+		BareKeyName(key & FL_KEY_MASK));
 
 	return buffer;
 }
@@ -529,18 +520,8 @@ fprintf(stderr, "REMOVED BINDING key:%04x (%s)\n", temp.key, tokens[0]);
 		return;
 	}
 
-
 	temp.cmd = FindEditorCommand(tokens[2]);
 
-	// backwards compatibility
-	if (! temp.cmd)
-	{
-		if (y_stricmp(tokens[2], "GRID_Step") == 0)
-			temp.cmd = FindEditorCommand("GRID_Bump");
-		else if (y_stricmp(tokens[2], "Check") == 0)
-			temp.cmd = FindEditorCommand("MapCheck");
-	}
-
 	if (! temp.cmd)
 	{
 		LogPrintf("bindings.cfg: unknown function: %s\n", tokens[2]);
@@ -575,7 +556,7 @@ static bool LoadBindingsFromPath(const char *path, bool required)
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/bindings.cfg", path);
+	snprintf(filename, sizeof(filename), "%s/bindings.cfg", path);
 
 	FILE *fp = fopen(filename, "r");
 
@@ -602,7 +583,7 @@ static bool LoadBindingsFromPath(const char *path, bool required)
 
 		StringRemoveCRLF(line);
 
-		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, false /* do_strings */);
+		int num_tok = M_ParseLine(line, tokens, MAX_TOKENS, 2 /* do_strings, keep quotes */);
 
 		if (num_tok == 0)
 			continue;
@@ -688,7 +669,7 @@ void M_SaveBindings()
 {
 	static char filename[FL_PATH_MAX];
 
-	sprintf(filename, "%s/bindings.cfg", home_dir);
+	snprintf(filename, sizeof(filename), "%s/bindings.cfg", home_dir);
 
 	FILE *fp = fopen(filename, "w");
 
@@ -889,7 +870,7 @@ void M_DetectConflictingBinds()
 
 const char * M_StringForFunc(int index)
 {
-	static char buffer[300];
+	static char buffer[2048];
 
 	key_binding_t& bind = pref_binds[index];
 
@@ -942,7 +923,7 @@ const char * M_StringForBinding(int index, bool changing_key)
 		tempk = toupper(tempk & FL_KEY_MASK);
 	}
 
-	sprintf(buffer, "%s%6.6s%-10.10s %-9.9s %.32s",
+	snprintf(buffer, sizeof(buffer), "%s%6.6s%-10.10s %-9.9s %.32s",
 			bind.is_duplicate ? "@C1" : "",
 			changing_key ? "<?"     : ModName_Space(tempk),
 			changing_key ? "\077?>" : BareKeyName(tempk & FL_KEY_MASK),
@@ -988,7 +969,7 @@ static const char * DoParseBindingFunc(key_binding_t& bind, const char * func_st
 
 	const char * tokens[MAX_TOKENS];
 
-	int num_tok = M_ParseLine(buffer, tokens, MAX_TOKENS, false /* do_strings */);
+	int num_tok = M_ParseLine(buffer, tokens, MAX_TOKENS, 2 /* do_strings, keep quotes */);
 
 	if (num_tok <= 0)
 		return "Missing function name";
@@ -997,7 +978,7 @@ static const char * DoParseBindingFunc(key_binding_t& bind, const char * func_st
 
 	if (! cmd)
 	{
-		sprintf(error_msg, "Unknown function name: %s", tokens[0]);
+		snprintf(error_msg, sizeof(error_msg), "Unknown function name: %s", tokens[0]);
 		return error_msg;
 	}
 
@@ -1008,7 +989,7 @@ static const char * DoParseBindingFunc(key_binding_t& bind, const char * func_st
 	{
 		char *mode = StringUpper(M_KeyContextString(cmd->req_context));
 
-		sprintf(error_msg, "%s can only be used in %s mode",
+		snprintf(error_msg, sizeof(error_msg), "%s can only be used in %s mode",
 		        tokens[0], mode);
 
 		StringFree(mode);
@@ -1123,6 +1104,18 @@ keycode_t M_TranslateKey(int key, int state)
 }
 
 
+int M_KeyToShortcut(keycode_t key)
+{
+	int shortcut = key & FL_KEY_MASK;
+
+	     if (key & MOD_COMMAND) shortcut |= MOD_COMMAND;
+	else if (key & MOD_ALT)     shortcut |= MOD_ALT;
+	else if (key & MOD_SHIFT)   shortcut |= MOD_SHIFT;
+
+	return shortcut;
+}
+
+
 key_context_e M_ModeToKeyContext(obj_type_e mode)
 {
 	switch (mode)
@@ -1167,19 +1160,9 @@ static void DoExecuteCommand(const editor_command_t *cmd)
 }
 
 
-bool ExecuteKey(keycode_t key, key_context_e context)
+static int FindBinding(keycode_t key, key_context_e context, bool lax_only)
 {
-	Status_Clear();
-
-	for (int p = 0 ; p < MAX_EXEC_PARAM ; p++)
-	{
-		EXEC_Param[p] = "";
-		EXEC_Flags[p] = "";
-	}
-
-	EXEC_Errno = 0;
-
-	for (unsigned int i = 0 ; i < all_bindings.size() ; i++)
+	for (int i = 0 ; i < (int)all_bindings.size() ; i++)
 	{
 		key_binding_t& bind = all_bindings[i];
 
@@ -1191,6 +1174,9 @@ bool ExecuteKey(keycode_t key, key_context_e context)
 		// match modifiers "loosely" for certain commands (esp. NAV_xxx)
 		bool is_lax = (bind.key & MOD_LAX_SHIFTCTRL) ? true : false;
 
+		if (lax_only != is_lax)
+			continue;
+
 		keycode_t key1 = key;
 		keycode_t key2 = bind.key;
 
@@ -1203,37 +1189,65 @@ bool ExecuteKey(keycode_t key, key_context_e context)
 		if (key1 != key2)
 			continue;
 
-		// found a match!
-
-		int p_idx = 0;
-		int f_idx = 0;
-
-		for (int p = 0 ; p < MAX_EXEC_PARAM ; p++)
-		{
-			if (! bind.param[p][0])
-				break;
-
-			// separate flags from normal parameters
-			if (bind.param[p][0] == '/')
-				EXEC_Flags[f_idx++] = bind.param[p];
-			else
-				EXEC_Param[p_idx++] = bind.param[p];
-		}
-
-		EXEC_CurKey = key;
-
-		// commands can test for LAX mode via a special flag
-		if (is_lax && f_idx < MAX_EXEC_PARAM)
-		{
-			EXEC_Flags[f_idx++] = "/LAX";
-		}
-
-		DoExecuteCommand(bind.cmd);
-
-		return true;
+		// found a match
+		return i;
 	}
 
-	return false;
+	// not found
+	return -1;
+}
+
+
+bool ExecuteKey(keycode_t key, key_context_e context)
+{
+	for (int p = 0 ; p < MAX_EXEC_PARAM ; p++)
+	{
+		EXEC_Param[p] = "";
+		EXEC_Flags[p] = "";
+	}
+
+	EXEC_Errno = 0;
+
+	// this logic means we only use a LAX binding if an exact match
+	// could not be found.
+	int idx = FindBinding(key, context, false);
+
+	if (idx < 0)
+		idx = FindBinding(key, context, true);
+
+	if (idx < 0)
+		return false;
+
+	key_binding_t& bind = all_bindings[idx];
+
+	int p_idx = 0;
+	int f_idx = 0;
+
+	for (int p = 0 ; p < MAX_EXEC_PARAM ; p++)
+	{
+		if (! bind.param[p][0])
+			break;
+
+		// separate flags from normal parameters
+		if (bind.param[p][0] == '/')
+			EXEC_Flags[f_idx++] = bind.param[p];
+		else
+			EXEC_Param[p_idx++] = bind.param[p];
+	}
+
+	EXEC_CurKey = key;
+
+	// commands can test for LAX mode via a special flag
+	bool is_lax = (bind.key & MOD_LAX_SHIFTCTRL) ? true : false;
+
+	if (is_lax && f_idx < MAX_EXEC_PARAM)
+	{
+		EXEC_Flags[f_idx++] = "/LAX";
+	}
+
+	DoExecuteCommand(bind.cmd);
+
+	return true;
 }
 
 
@@ -1241,8 +1255,6 @@ bool ExecuteCommand(const editor_command_t *cmd,
 					const char *param1, const char *param2,
                     const char *param3, const char *param4)
 {
-	Status_Clear();
-
 	for (int p = 0 ; p < MAX_EXEC_PARAM ; p++)
 	{
 		EXEC_Param[p] = "";
@@ -1302,18 +1314,12 @@ void Beep(const char *fmt, ...)
 
 	buffer[MSG_BUF_LEN-1] = 0;
 
-	if (buffer[0])
-	{
-		Status_Set("%s", buffer);
-
-		LogPrintf("BEEP: %s\n", buffer);
-	}
-	else
-		Status_Set("Problem occurred");
-
-	EXEC_Errno = 1;
+	Status_Set("%s", buffer);
+	LogPrintf("BEEP: %s\n", buffer);
 
 	fl_beep();
+
+	EXEC_Errno = 1;
 }
 
 //--- editor settings ---
