@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2018 Andrew Apted
+//  Copyright (C) 2001-2020 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -37,6 +37,7 @@
 #include "e_main.h"
 #include "m_events.h"
 #include "r_render.h"
+#include "r_subdiv.h"
 
 #include "w_rawdef.h"
 #include "w_texture.h"
@@ -62,6 +63,7 @@
 
 bool want_quit = false;
 bool app_has_focus = false;
+bool in_fatal_error = false;
 
 const char *config_file = NULL;
 const char *log_file;
@@ -82,6 +84,7 @@ const char *Port_name;
 const char *Level_name;
 
 map_format_e Level_format;
+std::string  Udmf_namespace;
 
 
 //
@@ -141,6 +144,19 @@ void FatalError(const char *fmt, ...)
 
 	buffer[MSG_BUF_LEN-1] = 0;
 
+	// re-entered here? ouch!
+	if (in_fatal_error)
+	{
+		fprintf(stderr, "\nERROR LOOP DETECTED!\n");
+		fflush(stderr);
+
+		LogClose();
+		exit(4);
+	}
+
+	// minimise chance of a infinite loop of errors
+	in_fatal_error = true;
+
 	if (init_progress < 1 || Quiet || log_file)
 	{
 		fprintf(stderr, "\nFATAL ERROR: %s", buffer);
@@ -429,7 +445,7 @@ static void DeterminePort()
 		return;
 	}
 
-	const char *var_game = M_VariantForGame(Game_name);
+	const char *base_game = M_GetBaseGame(Game_name);
 
 	// ensure the 'default_port' value is OK
 	if (! (default_port && default_port[0]))
@@ -443,7 +459,7 @@ static void DeterminePort()
 				default_port);
 		default_port = "vanilla";
 	}
-	else if (! M_CheckPortSupportsGame(var_game, default_port))
+	else if (! M_CheckPortSupportsGame(base_game, default_port))
 	{
 		LogPrintf("WARNING: Default port '%s' not compatible with '%s'\n",
 				default_port, Game_name);
@@ -508,7 +524,7 @@ static const char * DetermineLevel()
 }
 
 
-// this is only to prevent ESCAPE key from quitting
+// this is mainly to prevent ESCAPE key from quitting
 int Main_key_handler(int event)
 {
 	if (event != FL_SHORTCUT)
@@ -516,7 +532,7 @@ int Main_key_handler(int event)
 
 	if (Fl::event_key() == FL_Escape)
 	{
-		fl_beep();
+		EV_EscapeKey();
 		return 1;
 	}
 
@@ -564,6 +580,10 @@ static void Main_SetupFLTK()
 {
 	Fl::visual(FL_DOUBLE | FL_RGB);
 
+#ifndef NO_OPENGL
+	Fl::gl_visual(FL_RGB);
+#endif
+
 	// disable keyboard navigation, as it often interferes with our
 	// user interface, especially TAB key for toggling the 3D view.
 	Fl::option(Fl::OPTION_VISIBLE_FOCUS, false);
@@ -614,15 +634,6 @@ static void Main_SetupFLTK()
 	int screen_h = Fl::h();
 
 	LogPrintf("Detected Screen Size: %dx%d\n", screen_w, screen_h);
-
-	KF = 1;
-#if 0  // TODO
-	KF = 0;
-	if (screen_w >= 1000) KF = 1;
-	if (screen_w >= 1180) KF = 2;
-#endif
-
-	KF_fonth = (14 + KF * 2);
 }
 
 
@@ -644,7 +655,9 @@ static void Main_OpenWindow()
 		argv[1] = NULL;
 
 		main_win->show(argc, argv);
-
+#ifndef NO_OPENGL
+		main_win->canvas->show();  // needed for OpenGL
+#endif
 		app_has_focus = true;
 	}
 
@@ -695,7 +708,7 @@ bool Main_ConfirmQuit(const char *action)
 
 	char buttons[200];
 
-	sprintf(buttons, "Cancel|&%s", action);
+	snprintf(buttons, sizeof(buttons), "Cancel|&%s", action);
 
 	// convert action string like "open a new map" to a simple "Open"
 	// string for the yes choice.
@@ -768,7 +781,6 @@ void Main_Loop()
 		// TODO: handle these in a better way
 		main_win->UpdateTitle(MadeChanges ? '*' : 0);
 
-		main_win->scroll->UpdateRenderMode();
 		main_win->scroll->UpdateBounds();
 
 		if (edit.Selected->empty())
@@ -829,10 +841,10 @@ static void ReadPortInfo()
 
 	SYS_ASSERT(Port_name);
 
-	const char *var_game = M_VariantForGame(Game_name);
+	const char *base_game = M_GetBaseGame(Game_name);
 
 	// warn user if this port is incompatible with the game
-	if (! M_CheckPortSupportsGame(var_game, Port_name))
+	if (! M_CheckPortSupportsGame(base_game, Port_name))
 	{
 		LogPrintf("WARNING: the port '%s' is not compatible with the game '%s'\n",
 				Port_name, Game_name);
@@ -857,11 +869,11 @@ static void ReadPortInfo()
 	M_LoadDefinitions("ports", Port_name);
 
 	// prevent UI weirdness if the port is forced to BOOM / MBF
-	if (game_info.strife_flags)
+	if (Features.strife_flags)
 	{
-		game_info.pass_through = 0;
-		game_info.coop_dm_flags = 0;
-		game_info.friend_flag = 0;
+		Features.pass_through = 0;
+		Features.coop_dm_flags = 0;
+		Features.friend_flag = 0;
 	}
 }
 
@@ -878,8 +890,10 @@ void Main_LoadResources()
 
 	M_ClearAllDefinitions();
 
-	ReadGameInfo();
+	// clear the parse variables, pre-set a few vars
+	M_PrepareConfigVariables();
 
+	ReadGameInfo();
 	ReadPortInfo();
 
 	// reset the master directory
@@ -910,8 +924,15 @@ void Main_LoadResources()
 	LogPrintf("--- DONE ---\n");
 	LogPrintf("\n");
 
+	// reset sector info (for slopes and 3D floors)
+	Subdiv_InvalidateAll();
+
 	if (main_win)
 	{
+		// kill all loaded OpenGL images
+		if (main_win->canvas)
+			main_win->canvas->DeleteContext();
+
 		main_win->UpdateGameInfo();
 
 		main_win->browser->Populate();
@@ -927,7 +948,7 @@ void Main_LoadResources()
 static void ShowHelp()
 {
 	printf(	"\n"
-			"*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2018 Andrew Apted, et al ***\n"
+			"*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2020 Andrew Apted, et al ***\n"
 			"\n");
 
 	printf(	"Eureka is free software, under the terms of the GNU General\n"
@@ -1011,7 +1032,7 @@ int main(int argc, char *argv[])
 
 
 	LogPrintf("\n");
-	LogPrintf("*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2018 Andrew Apted, et al ***\n");
+	LogPrintf("*** " EUREKA_TITLE " v" EUREKA_VERSION " (C) 2020 Andrew Apted, et al ***\n");
 	LogPrintf("\n");
 
 	// sanity checks type sizes (useful when porting)
@@ -1107,28 +1128,25 @@ int main(int argc, char *argv[])
 
 	DeterminePort();
 
-
 	// temporarily load the iwad, the following few functions need it.
-	// it gets loaded again in Main_LoadResources().
+	// it will get loaded again in Main_LoadResources().
 	Main_LoadIWAD();
 
-	Level_name = DetermineLevel();
-
-	// config file parsing can depend on the map format, so get it now
-	GetLevelFormat(edit_wad ? edit_wad : game_wad, Level_name);
-
-	Main_LoadResources();
-
 
 	// load the initial level
+	Level_name = DetermineLevel();
+
 	LogPrintf("Loading initial map : %s\n", Level_name);
 
 	LoadLevel(edit_wad ? edit_wad : game_wad, Level_name);
 
+	// do this *after* loading the level, since config file parsing
+	// can depend on the map format and UDMF namespace.
+	Main_LoadResources();
+
 
 	Main_Loop();
 
-
 quit:
 	/* that's all folks! */
 
