@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2018 Andrew Apted
+//  Copyright (C) 2001-2020 Andrew Apted
 //  Copyright (C) 1997-2003 André Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
 
 #define EUREKA_TITLE  "Eureka DOOM Editor"
 
-#define EUREKA_VERSION  "1.24"
+#define EUREKA_VERSION  "1.27b"
 
 #define EUREKA_LUMP  "__EUREKA"
 
@@ -68,6 +68,7 @@
 
 #include <algorithm>
 #include <vector>
+#include <string>
 
 
 /*
@@ -109,7 +110,8 @@ typedef enum
 	MAPF_INVALID = 0,
 
 	MAPF_Doom,
-	MAPF_Hexen
+	MAPF_Hexen,
+	MAPF_UDMF
 
 } map_format_e;
 
@@ -125,6 +127,7 @@ typedef int map_format_bitset_t;
 extern int  init_progress;
 extern bool want_quit;
 extern bool app_has_focus;
+extern bool in_fatal_error;
 
 extern const char *install_dir;  // install dir (e.g. /usr/share/eureka)
 extern const char *home_dir;     // home dir (e.g. $HOME/.eureka)
@@ -134,7 +137,8 @@ extern const char *Game_name;   // Name of game "doom", "doom2", "heretic", ...
 extern const char *Port_name;   // Name of source port "vanilla", "boom", ...
 extern const char *Level_name;  // Name of map lump we are editing
 
-extern map_format_e Level_format; // format of current map
+extern map_format_e Level_format;   // format of current map
+extern std::string  Udmf_namespace; // for UDMF, the current namespace
 
 extern const char *config_file; // Name of the configuration file, or NULL
 extern const char *log_file;    // Name of log file, or NULL
@@ -160,10 +164,6 @@ extern int   show_help;		// Print usage message and exit.
 extern int   show_version;	// Print version info and exit.
 
 
-extern int KF;  // Kromulent Factor
-extern int KF_fonth;  // default font size
-
-
 extern int MadeChanges;
 
 
