@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 André Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -40,12 +40,17 @@ int  grid_default_mode = 0;  // off
 
 int  grid_style;  // 0 = squares, 1 = dotty
 bool grid_hide_in_free_mode = false;
+bool grid_snap_indicator = true;
+
+int  grid_ratio_high = 3;  // custom ratio (high must be >= low)
+int  grid_ratio_low  = 1;  // (low must be > 0)
 
 
 Grid_State_c::Grid_State_c() :
-	 step(16 /* dummy */),
-	 snap(true), shown(true),
-     orig_x(0.0), orig_y(0.0), Scale(1.0)
+	step(64 /* dummy */), snap(true),
+	ratio(0), shown(true),
+	orig_x(0.0), orig_y(0.0),
+	Scale(1.0)
 { }
 
 Grid_State_c::~Grid_State_c()
@@ -54,13 +59,13 @@ Grid_State_c::~Grid_State_c()
 
 void Grid_State_c::Init()
 {
-	step = grid_default_size - 1;
+	step = grid_default_size;
 
 	if (step < 1)
 		step = 1;
 
-	if (step >= grid_values[1])
-		step =  grid_values[1] - 1;
+	if (step > grid_values[0])
+		step = grid_values[0];
 
 	shown = true;  // prevent a beep in AdjustStep
 
@@ -71,7 +76,7 @@ void Grid_State_c::Init()
 		shown = false;
 
 		if (main_win)
-			main_win->info_bar->SetGrid(0);
+			main_win->info_bar->SetGrid(-1);
 	}
 	else
 	{
@@ -111,24 +116,18 @@ void Grid_State_c::Scroll(double delta_x, double delta_y)
 }
 
 
-int Grid_State_c::ForceSnapX(int map_x) const
+int Grid_State_c::ForceSnapX(double map_x) const
 {
-	if (map_x >= 0)
-		return grid.step * ((map_x + grid.step / 2) / grid.step);
-	else
-		return grid.step * ((map_x - grid.step / 2) / grid.step);
+	return grid.step * round(map_x / (double)grid.step);
 }
 
-int Grid_State_c::ForceSnapY(int map_y) const
+int Grid_State_c::ForceSnapY(double map_y) const
 {
-	if (map_y >= 0)
-		return grid.step * ((map_y + grid.step / 2) / grid.step);
-	else
-		return grid.step * ((map_y - grid.step / 2) / grid.step);
+	return grid.step * round(map_y / (double)grid.step);
 }
 
 
-int Grid_State_c::SnapX(int map_x) const
+double Grid_State_c::SnapX(double map_x) const
 {
 	if (! snap || grid.step == 0)
 		return map_x;
@@ -136,7 +135,7 @@ int Grid_State_c::SnapX(int map_x) const
 	return ForceSnapX(map_x);
 }
 
-int Grid_State_c::SnapY(int map_y) const
+double Grid_State_c::SnapY(double map_y) const
 {
 	if (! snap || grid.step == 0)
 		return map_y;
@@ -145,7 +144,180 @@ int Grid_State_c::SnapY(int map_y) const
 }
 
 
-int Grid_State_c::QuantSnapX(int map_x, bool want_furthest, int *dir) const
+void Grid_State_c::RatioSnapXY(double& var_x, double& var_y,
+							   double start_x, double start_y) const
+{
+	// snap first, otherwise we lose the ratio
+	var_x = grid.SnapX(var_x);
+	var_y = grid.SnapY(var_y);
+
+	double dx = var_x - start_x;
+	double dy = var_y - start_y;
+
+	double len = MAX(abs(dx), abs(dy));
+
+	int sign_x = (dx >= 0) ? +1 : -1;
+	int sign_y = (dy >= 0) ? +1 : -1;
+
+	double custom;
+
+	switch (ratio)
+	{
+	case 0: // unlocked
+		break;
+
+	case 1: // 1:1 (45 degrees) + axis aligned
+		if (fabs(dx) * 2 < fabs(dy))
+		{
+			var_x = start_x;
+		}
+		else if (fabs(dy) * 2 < fabs(dx))
+		{
+			var_y = start_y;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len;
+		}
+		break;
+
+	case 2: // 2:1 + axis aligned
+		if (fabs(dx) * 4 < fabs(dy))
+		{
+			var_x = start_x;
+		}
+		else if (fabs(dy) * 4 < fabs(dx))
+		{
+			var_y = start_y;
+		}
+		else if (fabs(dx) < fabs(dy))
+		{
+			var_x = start_x + sign_x * len * 0.5;
+			var_y = start_y + sign_y * len;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len * 0.5;
+		}
+		break;
+
+	case 3: // 4:1 + axis aligned
+		if (fabs(dx) * 8 < fabs(dy))
+		{
+			var_x = start_x;
+		}
+		else if (fabs(dy) * 8 < fabs(dx))
+		{
+			var_y = start_y;
+		}
+		else if (fabs(dx) < fabs(dy))
+		{
+			var_x = start_x + sign_x * len * 0.25;
+			var_y = start_y + sign_y * len;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len * 0.25;
+		}
+		break;
+
+	case 4: // 8:1 + axis aligned
+		if (fabs(dx) * 16 < fabs(dy))
+		{
+			var_x = start_x;
+		}
+		else if (fabs(dy) * 16 < fabs(dx))
+		{
+			var_y = start_y;
+		}
+		else if (fabs(dx) < fabs(dy))
+		{
+			var_x = start_x + sign_x * len * 0.125;
+			var_y = start_y + sign_y * len;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len * 0.125;
+		}
+		break;
+
+	case 5: // 5:4 + axis aligned
+		if (fabs(dx) * 3 < fabs(dy))
+		{
+			var_x = start_x;
+		}
+		else if (fabs(dy) * 3 < fabs(dx))
+		{
+			var_y = start_y;
+		}
+		else if (fabs(dx) < fabs(dy))
+		{
+			var_x = start_x + sign_x * len * 0.8;
+			var_y = start_y + sign_y * len;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len * 0.8;
+		}
+		break;
+
+	case 6: // 7:4 + axis aligned
+		if (fabs(dx) * 3 < fabs(dy))
+		{
+			var_x = start_x;
+		}
+		else if (fabs(dy) * 3 < fabs(dx))
+		{
+			var_y = start_y;
+		}
+		else if (fabs(dx) < fabs(dy))
+		{
+			var_x = start_x + sign_x * len * 4 / 7;
+			var_y = start_y + sign_y * len;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len * 4 / 7;
+		}
+		break;
+
+	default: // USER SETTING
+		if (grid_ratio_low < 1)
+			grid_ratio_low = 1;
+		if (grid_ratio_high < grid_ratio_low)
+			grid_ratio_high = grid_ratio_low;
+
+		custom = (double)grid_ratio_low / (double)grid_ratio_high;
+
+		if (custom > 0.1 && fabs(dx) < fabs(dy) * custom * 0.3)
+		{
+			var_x = start_x;
+		}
+		else if (custom > 0.1 && fabs(dy) < fabs(dx) * custom * 0.3)
+		{
+			var_y = start_y;
+		}
+		else if (fabs(dx) < fabs(dy))
+		{
+			var_x = start_x + sign_x * len * custom;
+			var_y = start_y + sign_y * len;
+		}
+		else
+		{
+			var_x = start_x + sign_x * len;
+			var_y = start_y + sign_y * len * custom;
+		}
+	}
+}
+
+
+int Grid_State_c::QuantSnapX(double map_x, bool want_furthest, int *dir) const
 {
 	if (OnGridX(map_x))
 	{
@@ -173,7 +345,7 @@ int Grid_State_c::QuantSnapX(int map_x, bool want_furthest, int *dir) const
 		return ForceSnapX(map_x - (step - 1));
 }
 
-int Grid_State_c::QuantSnapY(int map_y, bool want_furthest, int *dir) const
+int Grid_State_c::QuantSnapY(double map_y, bool want_furthest, int *dir) const
 {
 	// this is sufficient since the grid is always square
 
@@ -181,31 +353,58 @@ int Grid_State_c::QuantSnapY(int map_y, bool want_furthest, int *dir) const
 }
 
 
-bool Grid_State_c::OnGridX(int map_x) const
+void Grid_State_c::NaturalSnapXY(double& var_x, double& var_y) const
+{
+	// this is only used by UI_Canvas::PointerPos()
+
+	double nat_step = 1.0;
+
+	while (nat_step * 2.0 <= Scale)
+		nat_step = nat_step * 2.0;
+
+	while (nat_step * 0.5 >= Scale)
+		nat_step = nat_step * 0.5;
+
+	var_x = round(var_x * nat_step) / nat_step;
+	var_y = round(var_y * nat_step) / nat_step;
+}
+
+
+bool Grid_State_c::OnGridX(double map_x) const
 {
 	if (map_x < 0)
 		map_x = -map_x;
 
-	return (map_x % step) == 0;
+	int map_x2 = (int)map_x;
+
+	if (map_x != (double)map_x2)
+		return false;
+
+	return (map_x2 % step) == 0;
 }
 
-bool Grid_State_c::OnGridY(int map_y) const
+bool Grid_State_c::OnGridY(double map_y) const
 {
 	if (map_y < 0)
 		map_y = -map_y;
 
-	return (map_y % step) == 0;
+	int map_y2 = (int)map_y;
+
+	if (map_y != (double)map_y2)
+		return false;
+
+	return (map_y2 % step) == 0;
 }
 
-bool Grid_State_c::OnGrid(int map_x, int map_y) const
+bool Grid_State_c::OnGrid(double map_x, double map_y) const
 {
 	return OnGridX(map_x) && OnGridY(map_y);
 }
 
 
-void Grid_State_c::RefocusZoom(int map_x, int map_y, float before_Scale)
+void Grid_State_c::RefocusZoom(double map_x, double map_y, float before_Scale)
 {
-	float dist_factor = (1.0 - before_Scale / Scale);
+	double dist_factor = (1.0 - before_Scale / Scale);
 
 	orig_x += (map_x - orig_x) * dist_factor;
 	orig_y += (map_y - orig_y) * dist_factor;
@@ -235,29 +434,16 @@ const int Grid_State_c::digit_scales[] =
 
 const int Grid_State_c::grid_values[] =
 {
-	-1 /* OFF */,
+	1024, 512, 256, 192, 128, 64, 32, 16, 8, 4, 2,
 
-	1024, 512, 256, 192, 128, 64, 32, 16, 8, 4, 2
+	-1 /* OFF */,
 };
 
 #define NUM_SCALE_VALUES  18
 #define NUM_GRID_VALUES   12
 
 
-const char *Grid_State_c::scale_options()
-{
-	return  "x 32|x 16|x 8|x 6|x 4|x 3|x 2|x 1.5|"
-			" 100% |"
-			"/ 1.5|/ 2|/ 3|/ 4|/ 6|/ 8|/ 16|/ 32|/ 64";
-}
-
-const char *Grid_State_c::grid_options()
-{
-	return "OFF|1024|512|256|192|128| 64| 32| 16|  8|  4|  2";
-}
-
-
-void Grid_State_c::ScaleFromWidget(int i)
+void Grid_State_c::RawSetScale(int i)
 {
 	SYS_ASSERT(0 <= i && i < NUM_SCALE_VALUES);
 
@@ -268,52 +454,30 @@ void Grid_State_c::ScaleFromWidget(int i)
 
 	main_win->scroll->AdjustPos();
 	main_win->canvas->PointerPos();
+	main_win->info_bar->SetScale(Scale);
 
 	RedrawMap();
 }
 
 
-void Grid_State_c::SetStep(int new_step)
-{
-	step = new_step;
-
-	shown = true;
-
-	// find closest value in info bar
-	int best = 0;
-	int best_dist = 999999;
-
-	for (int i = 1 ; i < NUM_GRID_VALUES ; i++)
-	{
-		int dist = abs(grid_values[i] - new_step);
-
-		if (dist < best_dist)
-		{
-			best = i;
-			best_dist = dist;
-		}
-	}
-
-	if (main_win)
-		main_win->info_bar->SetGrid(best);
-
-	RedrawMap();
-}
-
-
-void Grid_State_c::StepFromWidget(int i)
+void Grid_State_c::RawSetStep(int i)
 {
 	SYS_ASSERT(0 <= i && i < NUM_GRID_VALUES);
 
-	if (i == 0)  /* OFF */
+	if (i == NUM_GRID_VALUES-1)  /* OFF */
 	{
 		shown = false;
+
+		if (main_win)
+			main_win->info_bar->SetGrid(-1);
 	}
 	else
 	{
 		shown = true;
+		step  = grid_values[i];
 
-		step = grid_values[i];
+		if (main_win)
+			main_win->info_bar->SetGrid(step);
 	}
 
 	if (grid_hide_in_free_mode)
@@ -323,13 +487,28 @@ void Grid_State_c::StepFromWidget(int i)
 }
 
 
+void Grid_State_c::ForceStep(int new_step)
+{
+	step  = new_step;
+	shown = true;
+
+	if (main_win)
+		main_win->info_bar->SetGrid(step);
+
+	if (grid_hide_in_free_mode)
+		SetSnap(shown);
+
+	RedrawMap();
+}
+
+
 void Grid_State_c::StepFromScale()
 {
 	int pixels_min = 16;
 
-	int result = 1;
+	int result = 0;
 
-	for (int i = 1 ; i < NUM_GRID_VALUES ; i++)
+	for (int i = 0 ; i < NUM_GRID_VALUES-1 ; i++)
 	{
 		result = i;
 
@@ -358,7 +537,7 @@ void Grid_State_c::AdjustStep(int delta)
 
 	if (delta > 0)
 	{
-		for (int i = NUM_GRID_VALUES-1 ; i >= 1 ; i--)
+		for (int i = NUM_GRID_VALUES-2 ; i >= 0 ; i--)
 		{
 			if (grid_values[i] > step)
 			{
@@ -369,7 +548,7 @@ void Grid_State_c::AdjustStep(int delta)
 	}
 	else // (delta < 0)
 	{
-		for (int i = 1 ; i < NUM_GRID_VALUES ; i++)
+		for (int i = 0 ; i < NUM_GRID_VALUES-1 ; i++)
 		{
 			if (grid_values[i] < step)
 			{
@@ -383,10 +562,7 @@ void Grid_State_c::AdjustStep(int delta)
 	if (result < 0)
 		return;
 
-	StepFromWidget(result);
-
-	if (main_win)
-		main_win->info_bar->SetGrid(result);
+	RawSetStep(result);
 }
 
 
@@ -421,43 +597,11 @@ void Grid_State_c::AdjustScale(int delta)
 	if (result < 0)
 		return;
 
-	if (main_win)
-		main_win->info_bar->SetScale(result);
-
-	ScaleFromWidget(result);
+	RawSetScale(result);
 }
 
 
-void Grid_State_c::DoSetScale(double new_scale)
-{
-	int index = 10;  // meh
-
-	for (int i = NUM_SCALE_VALUES-1 ; i >= 0 ; i--)
-	{
-		float ratio = scale_values[i] / new_scale;
-
-		if (ratio > 0.99 && ratio < 1.01)
-		{
-			index = i;
-			break;
-		}
-	}
-
-	Scale = scale_values[index];
-
-	if (main_win)
-	{
-		main_win->info_bar->SetScale(index);
-
-		main_win->scroll->AdjustPos();
-		main_win->canvas->PointerPos();
-
-		RedrawMap();
-	}
-}
-
-
-void Grid_State_c::DoSetShown(bool new_value)
+void Grid_State_c::RawSetShown(bool new_value)
 {
 	shown = new_value;
 
@@ -466,20 +610,13 @@ void Grid_State_c::DoSetShown(bool new_value)
 
 	if (! shown)
 	{
-		main_win->info_bar->SetGrid(0);
+		main_win->info_bar->SetGrid(-1);
 		RedrawMap();
 		return;
 	}
 
 	// update the info-bar
-	for (int i = 1 ; i < NUM_GRID_VALUES ; i++)
-	{
-		if (grid_values[i] == step)
-		{
-			main_win->info_bar->SetGrid(i);
-			break;
-		}
-	}
+	main_win->info_bar->SetGrid(step);
 
 	RedrawMap();
 }
@@ -487,7 +624,7 @@ void Grid_State_c::DoSetShown(bool new_value)
 
 void Grid_State_c::SetShown(bool enable)
 {
-	DoSetShown(enable);
+	RawSetShown(enable);
 
 	if (grid_hide_in_free_mode)
 		SetSnap(enable);
@@ -533,10 +670,7 @@ void Grid_State_c::NearestScale(double want_scale)
 			break;
 	}
 
-	if (main_win)
-		main_win->info_bar->SetScale(best);
-
-	ScaleFromWidget(best);
+	RawSetScale(best);
 }
 
 
@@ -551,7 +685,7 @@ bool Grid_ParseUser(const char ** tokens, int num_tok)
 
 		double new_scale = atof(tokens[3]);
 
-		grid.DoSetScale(new_scale);
+		grid.NearestScale(new_scale);
 
 		RedrawMap();
 		return true;
@@ -565,7 +699,7 @@ bool Grid_ParseUser(const char ** tokens, int num_tok)
 
 		// tokens[2] was grid.mode, currently unused
 
-		grid.DoSetShown(t_shown);
+		grid.RawSetShown(t_shown);
 
 		RedrawMap();
 
