@@ -33,6 +33,9 @@ public:
 	// if true, new and moved objects are forced to be on the grid
 	bool snap;
 
+	// if non-zero, new lines will be forced to have a certain ratio
+	int ratio;
+
 	// whether the grid is being displayed or not.
 	bool shown;
 
@@ -42,7 +45,7 @@ public:
 
 	// scale for drawing map
 	// (multiply a map coordinate by this to get a screen coord)
-	float Scale;
+	double Scale;
 
 public:
 	Grid_State_c();
@@ -70,17 +73,13 @@ public:
 
 	// move the origin so that the focus point of the last zoom
 	// operation (scale change) is map_x/y.
-	void RefocusZoom(int map_x, int map_y, float before_Scale);
+	void RefocusZoom(double map_x, double map_y, float before_Scale);
 
 	// choose the scale nearest to (and less than) the wanted one
 	void NearestScale(double want_scale);
 
-	void ScaleFromWidget(int i);
-
-	// set grid stepping size
-	void SetStep(int new_step);
-
-	void StepFromWidget(int i);
+	// force grid stepping size to arbitrary value
+	void ForceStep(int new_step);
 
 	// compute new grid step from current scale
 	void StepFromScale();
@@ -92,31 +91,37 @@ public:
 
 	// return X/Y coordinate snapped to grid
 	// (or unchanged is the 'snap' flag is off)
-	int SnapX(int map_x) const;
-	int SnapY(int map_y) const;
+	double SnapX(double map_x) const;
+	double SnapY(double map_y) const;
 
 	// return X/Y coordinate snapped to grid (always)
-	int ForceSnapX(int map_x) const;
-	int ForceSnapY(int map_y) const;
+	int ForceSnapX(double map_x) const;
+	int ForceSnapY(double map_y) const;
+
+	// snap X/Y coordinate to ratio lock
+	// (unchanged is the ratio snapping is off)
+	void RatioSnapXY(double& var_x, double& var_y, double start_x, double start_y) const;
 
 	// quantization snap, can pick coordinate on other side
-	int QuantSnapX(int map_x, bool want_furthest, int *dir = NULL) const;
-	int QuantSnapY(int map_y, bool want_furthest, int *dir = NULL) const;
+	int QuantSnapX(double map_x, bool want_furthest, int *dir = NULL) const;
+	int QuantSnapY(double map_y, bool want_furthest, int *dir = NULL) const;
+
+	// snap to the natural resolution of canvas
+	void NaturalSnapXY(double& var_x, double& var_y) const;
 
 	// check if the X/Y coordinate is on a grid point
-	bool OnGridX(int map_x) const;
-	bool OnGridY(int map_y) const;
+	bool OnGridX(double map_x) const;
+	bool OnGridY(double map_y) const;
 
-	bool OnGrid(int map_x, int map_y) const;
-
-	// interface with UI_Infobar widget...
-	static const char *scale_options();
-	static const char * grid_options();
+	bool OnGrid(double map_x, double map_y) const;
 
 private:
-	void DoSetShown(bool   new_shown);
 	void DoSetScale(double new_scale);
 
+	void RawSetStep(int i);
+	void RawSetScale(int i);
+	void RawSetShown(bool new_shown);
+
 private:
 	static const double scale_values[];
 	static const int digit_scales[];
