@@ -22,6 +22,7 @@
 #include "ui_window.h"
 
 #include "e_main.h"
+#include "e_cutpaste.h"
 #include "m_config.h"	// gui_scheme
 #include "m_game.h"
 #include "r_render.h"
@@ -46,13 +47,10 @@ UI_DefaultProps::UI_DefaultProps(int X, int Y, int W, int H) :
 
 	Fl_Box *title = new Fl_Box(X + 60, Y + 10, W - 70, 30, "Default Properties");
 	title->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
-	title->labelsize(18+KF*4);
+	title->labelsize(22);
 
-	Y += 35;
-	H -= 35;
-
-	X += 6;
-	W -= 12;
+	Y += 35; H -= 35;
+	X += 6;  W -= 12;
 
 
 	// ---- LINEDEF TEXTURES ------------
@@ -186,7 +184,7 @@ void UI_DefaultProps::hide_callback(Fl_Widget *w, void *data)
 void UI_DefaultProps::SetIntVal(Fl_Int_Input *w, int value)
 {
 	char buffer[64];
-	sprintf(buffer, "%d", value);
+	snprintf(buffer, sizeof(buffer), "%d", value);
 	w->value(buffer);
 }
 
@@ -200,36 +198,6 @@ void UI_DefaultProps::UpdateThingDesc()
 }
 
 
-void UI_DefaultProps::SetTexture(const char *name, int e_state)
-{
-	w_tex->value(name);
-	w_tex->do_callback();
-}
-
-
-void UI_DefaultProps::SetFlat(const char *name, int e_state)
-{
-	// same logic as in UI_SectorBox::SetFlat()
-
-	int sel_pics =	(f_pic->Selected() ? 1 : 0) |
-					(c_pic->Selected() ? 2 : 0);
-
-	if (sel_pics == 0)
-		sel_pics = (e_state & FL_BUTTON3) ? 2 : 1;
-
-	if (sel_pics & 1)
-	{
-		f_tex->value(name);
-		f_tex->do_callback();
-	}
-	if (sel_pics & 2)
-	{
-		c_tex->value(name);
-		c_tex->do_callback();
-	}
-}
-
-
 void UI_DefaultProps::SetThing(int number)
 {
 	default_thing = number;
@@ -240,21 +208,6 @@ void UI_DefaultProps::SetThing(int number)
 }
 
 
-void UI_DefaultProps::UnselectPicSet(char what /* 'f' or 't' */)
-{
-	if (what == 'f')
-	{
-		f_pic->Selected(false);
-		c_pic->Selected(false);
-	}
-
-	if (what == 't')
-	{
-		w_pic->Selected(false);
-	}
-}
-
-
 const char * UI_DefaultProps::Normalize_and_Dup(UI_DynInput *w)
 {
 	const char *normalized = StringDup(NormalizeTex(w->value()));
@@ -275,13 +228,9 @@ void UI_DefaultProps::tex_callback(Fl_Widget *w, void *data)
 
 		pic->Selected(! pic->Selected());
 
-		Render3D_ClearSelection();
-
 		if (pic->Selected())
-		{
-			box->UnselectPicSet('f');
 			main_win->BrowserMode('T');
-		}
+
 		return;
 	}
 
@@ -305,13 +254,9 @@ void UI_DefaultProps::flat_callback(Fl_Widget *w, void *data)
 
 		pic->Selected(! pic->Selected());
 
-		Render3D_ClearSelection();
-
 		if (pic->Selected())
-		{
-			box->UnselectPicSet('t');
 			main_win->BrowserMode('F');
-		}
+
 		return;
 	}
 
@@ -450,9 +395,9 @@ void UI_DefaultProps::CB_Copy(int sel_pics)
 	}
 
 	if (sel_pics & 4)
-		r_clipboard.SetTex(name);
+		Texboard_SetTex(name);
 	else
-		r_clipboard.SetFlat(name);
+		Texboard_SetFlat(name);
 }
 
 
@@ -460,19 +405,19 @@ void UI_DefaultProps::CB_Paste(int sel_pics)
 {
 	if (sel_pics & 1)
 	{
-		f_tex->value(BA_GetString(r_clipboard.GetFlatNum()));
+		f_tex->value(BA_GetString(Texboard_GetFlatNum()));
 		f_tex->do_callback();
 	}
 
 	if (sel_pics & 2)
 	{
-		c_tex->value(BA_GetString(r_clipboard.GetFlatNum()));
+		c_tex->value(BA_GetString(Texboard_GetFlatNum()));
 		c_tex->do_callback();
 	}
 
 	if (sel_pics & 4)
 	{
-		w_tex->value(BA_GetString(r_clipboard.GetTexNum()));
+		w_tex->value(BA_GetString(Texboard_GetTexNum()));
 		w_tex->do_callback();
 	}
 }
@@ -484,13 +429,13 @@ void UI_DefaultProps::CB_Delete(int sel_pics)
 
 	if (sel_pics & 1)
 	{
-		f_tex->value(game_info.sky_flat);
+		f_tex->value(Misc_info.sky_flat);
 		f_tex->do_callback();
 	}
 
 	if (sel_pics & 2)
 	{
-		c_tex->value(game_info.sky_flat);
+		c_tex->value(Misc_info.sky_flat);
 		c_tex->do_callback();
 	}
 }
@@ -537,23 +482,55 @@ bool UI_DefaultProps::ClipboardOp(char op)
 
 void UI_DefaultProps::BrowsedItem(char kind, int number, const char *name, int e_state)
 {
-	switch (kind)
+	if (kind == 'O')
 	{
-		case 'T': SetTexture(name, e_state); break;
-		case 'F': SetFlat   (name, e_state); break;
-		case 'O': SetThing(number); break;
+		SetThing(number);
+		return;
+	}
 
-		default:
-			fl_beep();
-			break;
+	if (! (kind == 'T' || kind == 'F'))
+		return;
+
+	int sel_pics =	(f_pic->Selected() ? 1 : 0) |
+					(c_pic->Selected() ? 2 : 0) |
+					(w_pic->Selected() ? 4 : 0);
+
+	if (sel_pics == 0)
+	{
+		if (kind == 'T')
+			sel_pics = 4;
+		else
+			sel_pics = (e_state & FL_BUTTON3) ? 2 : 1;
+	}
+
+
+	if (sel_pics & 1)
+	{
+		f_tex->value(name);
+		f_tex->do_callback();
+	}
+	if (sel_pics & 2)
+	{
+		c_tex->value(name);
+		c_tex->do_callback();
+	}
+	if (sel_pics & 4)
+	{
+		w_tex->value(name);
+		w_tex->do_callback();
 	}
 }
 
 
 void UI_DefaultProps::UnselectPics()
 {
-	UnselectPicSet('f');
-	UnselectPicSet('t');
+	w_pic->Unhighlight();
+	f_pic->Unhighlight();
+	c_pic->Unhighlight();
+
+	w_pic->Selected(false);
+	f_pic->Selected(false);
+	c_pic->Selected(false);
 }
 
 
