@@ -22,6 +22,7 @@
 #include "ui_window.h"
 
 #include "e_checks.h"
+#include "e_cutpaste.h"
 #include "e_linedef.h"
 #include "e_main.h"
 #include "e_things.h"
@@ -241,19 +242,16 @@ void UI_LineBox::type_callback(Fl_Widget *w, void *data)
 	// support hexadecimal
 	int new_type = (int)strtol(box->type->value(), NULL, 0);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited type of", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
 			BA_ChangeLD(*it, LineDef::F_TYPE, new_type);
 		}
 
-		BA_MessageForSel("edited type of", &list);
 		BA_End();
 	}
 
@@ -288,36 +286,35 @@ void UI_LineBox::dyntype_callback(Fl_Widget *w, void *data)
 }
 
 
-void UI_LineBox::SetTexOnLine(int ld, int new_tex, int e_state,
-                              int front_pics, int back_pics)
+void UI_LineBox::SetTexOnLine(int ld, int new_tex, int e_state, int parts)
 {
 	bool opposite = (e_state & FL_SHIFT);
 
 	LineDef *L = LineDefs[ld];
 
 	// handle the selected texture boxes
-	if (front_pics || back_pics)
+	if (parts != 0)
 	{
 		if (L->OneSided())
 		{
-			if (front_pics & 1) BA_ChangeSD(L->right, SideDef::F_MID_TEX,   new_tex);
-			if (front_pics & 2) BA_ChangeSD(L->right, SideDef::F_UPPER_TEX, new_tex);
+			if (parts & PART_RT_LOWER) BA_ChangeSD(L->right, SideDef::F_MID_TEX,   new_tex);
+			if (parts & PART_RT_UPPER) BA_ChangeSD(L->right, SideDef::F_UPPER_TEX, new_tex);
 
 			return;
 		}
 
 		if (L->Right())
 		{
-			if (front_pics & 1) BA_ChangeSD(L->right, SideDef::F_LOWER_TEX, new_tex);
-			if (front_pics & 2) BA_ChangeSD(L->right, SideDef::F_UPPER_TEX, new_tex);
-			if (front_pics & 4) BA_ChangeSD(L->right, SideDef::F_MID_TEX,   new_tex);
+			if (parts & PART_RT_LOWER) BA_ChangeSD(L->right, SideDef::F_LOWER_TEX, new_tex);
+			if (parts & PART_RT_UPPER) BA_ChangeSD(L->right, SideDef::F_UPPER_TEX, new_tex);
+			if (parts & PART_RT_RAIL)  BA_ChangeSD(L->right, SideDef::F_MID_TEX,   new_tex);
 		}
 
 		if (L->Left())
 		{
-			if (back_pics & 1) BA_ChangeSD(L->left, SideDef::F_LOWER_TEX, new_tex);
-			if (back_pics & 2) BA_ChangeSD(L->left, SideDef::F_UPPER_TEX, new_tex);
-			if (back_pics & 4) BA_ChangeSD(L->left, SideDef::F_MID_TEX,   new_tex);
+			if (parts & PART_LF_LOWER) BA_ChangeSD(L->left, SideDef::F_LOWER_TEX, new_tex);
+			if (parts & PART_LF_UPPER) BA_ChangeSD(L->left, SideDef::F_UPPER_TEX, new_tex);
+			if (parts & PART_LF_RAIL)  BA_ChangeSD(L->left, SideDef::F_MID_TEX,   new_tex);
 		}
 		return;
 	}
@@ -394,31 +391,32 @@ void UI_LineBox::SetTexOnLine(int ld, int new_tex, int e_state,
 }
 
 
-void UI_LineBox::SetTexture(const char *tex_name, int e_state)
+void UI_LineBox::SetTexture(const char *tex_name, int e_state, int parts)
 {
 	int new_tex = BA_InternaliseString(tex_name);
 
-	int front_pics = front->GetSelectedPics();
-	int  back_pics =  back->GetSelectedPics();
-
 	// this works a bit differently than other ways, we don't modify a
 	// widget and let it update the map, instead we update the map and
 	// let the widget(s) get updated.  That's because we do different
 	// things depending on whether the line is one-sided or two-sided.
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited texture on", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
-			SetTexOnLine(*it, new_tex, e_state, front_pics, back_pics);
+			int p2 = edit.Selected->get_ext(*it);
+
+			// only use parts explicitly selected in 3D view when no
+			// parts in the linedef panel are selected.
+			if (! (parts == 0 && p2 > 1))
+				p2 = parts;
+
+			SetTexOnLine(*it, new_tex, e_state, p2);
 		}
 
-		BA_MessageForSel("edited texture on", &list);
 		BA_End();
 	}
 
@@ -438,30 +436,27 @@ void UI_LineBox::SetLineType(int new_type)
 	}
 
 	char buffer[64];
-
-	sprintf(buffer, "%d", new_type);
+	snprintf(buffer, sizeof(buffer), "%d", new_type);
 
 	type->value(buffer);
 	type->do_callback();
 }
 
 
-void UI_LineBox::CB_Copy()
+void UI_LineBox::CB_Copy(int parts)
 {
 	// determine which sidedef texture to grab from
 	const char *name = NULL;
 
-	bool use_hl = (front->GetSelectedPics() == 0 && back->GetSelectedPics() == 0);
-
 	for (int pass = 0 ; pass < 2 ; pass++)
 	{
 		UI_SideBox *SD = (pass == 0) ? front : back;
 
-		int sel_pics = use_hl ? SD->GetHighlightedPics() : SD->GetSelectedPics();
-
 		for (int b = 0 ; b < 3 ; b++)
 		{
-			if ((sel_pics & (1 << b)) == 0)
+			int try_part = PART_RT_LOWER << (b + pass * 4);
+
+			if ((parts & try_part) == 0)
 				continue;
 
 			const char *b_name = (b == 0) ? SD->l_tex->value() :
@@ -479,60 +474,52 @@ void UI_LineBox::CB_Copy()
 		}
 	}
 
-	r_clipboard.SetTex(name);
+	Texboard_SetTex(name);
 
-	Status_Set("Copied %s", name);
+	Status_Set("copied %s", name);
 }
 
 
-void UI_LineBox::CB_Paste(int new_tex)
+void UI_LineBox::CB_Paste(int parts, int new_tex)
 {
-	bool use_hl = (front->GetSelectedPics() == 0 && back->GetSelectedPics() == 0);
-
 	// iterate over selected linedefs
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	if (edit.Selected->empty())
 		return;
 
 	BA_Begin();
+	BA_Message("pasted %s", BA_GetString(new_tex));
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
 		for (int pass = 0 ; pass < 2 ; pass++)
 		{
 			int sd = (pass == 0) ? L->right : L->left;
-
 			if (sd < 0)
 				continue;
 
-			UI_SideBox *SD = (pass == 0) ? front : back;
-
-			int sel_pics = use_hl ? SD->GetHighlightedPics() : SD->GetSelectedPics();
+			int parts2 = pass ? (parts >> 4) : parts;
 
 			if (L->TwoSided())
 			{
-				if (sel_pics & 1)
+				if (parts2 & PART_RT_LOWER)
 					BA_ChangeSD(sd, SideDef::F_LOWER_TEX, new_tex);
 
-				if (sel_pics & 2)
+				if (parts2 & PART_RT_UPPER)
 					BA_ChangeSD(sd, SideDef::F_UPPER_TEX, new_tex);
 
-				if (sel_pics & 4)
+				if (parts2 & PART_RT_RAIL)
 					BA_ChangeSD(sd, SideDef::F_MID_TEX, new_tex);
 			}
 			else  // one-sided line
 			{
-				if (sel_pics & 1)
+				if (parts2 & PART_RT_LOWER)
 					BA_ChangeSD(sd, SideDef::F_MID_TEX, new_tex);
 			}
 		}
 	}
 
-	BA_Message("Pasted %s", BA_GetString(new_tex));
 	BA_End();
 
 	UpdateField();
@@ -547,28 +534,30 @@ bool UI_LineBox::ClipboardOp(char op)
 	if (obj < 0)
 		return false;
 
-	if (front->GetSelectedPics() == 0 && front->GetHighlightedPics() == 0 &&
-		 back->GetSelectedPics() == 0 &&  back->GetHighlightedPics() == 0)
-	{
+	int parts = front->GetSelectedPics() | (back->GetSelectedPics() << 4);
+
+	if (parts == 0)
+		parts = front->GetHighlightedPics() | (back->GetHighlightedPics() << 4);
+
+	if (parts == 0)
 		return false;
-	}
 
 	switch (op)
 	{
 		case 'c':
-			CB_Copy();
+			CB_Copy(parts);
 			break;
 
 		case 'v':
-			CB_Paste(r_clipboard.GetTexNum());
+			CB_Paste(parts, Texboard_GetTexNum());
 			break;
 
 		case 'x':	// Cut
-			CB_Paste(BA_InternaliseString(default_wall_tex));
+			CB_Paste(parts, BA_InternaliseString(default_wall_tex));
 			break;
 
 		case 'd': // Delete
-			CB_Paste(BA_InternaliseString("-"));
+			CB_Paste(parts, BA_InternaliseString("-"));
 			break;
 	}
 
@@ -578,9 +567,15 @@ bool UI_LineBox::ClipboardOp(char op)
 
 void UI_LineBox::BrowsedItem(char kind, int number, const char *name, int e_state)
 {
-	if (kind == 'T')
+	if (kind == 'T' || kind == 'F')
 	{
-		SetTexture(name, e_state);
+		int front_pics = front->GetSelectedPics();
+		int  back_pics =  back->GetSelectedPics();
+
+		// this can be zero, invoking special behavior (based on mouse button)
+		int parts = front_pics | (back_pics << 4);
+
+		SetTexture(name, e_state, parts);
 	}
 	else if (kind == 'L')
 	{
@@ -595,7 +590,8 @@ void UI_LineBox::tag_callback(Fl_Widget *w, void *data)
 
 	int new_tag = atoi(box->tag->value());
 
-	Tags_ApplyNewValue(new_tag);
+	if (! edit.Selected->empty())
+		Tags_ApplyNewValue(new_tag);
 }
 
 
@@ -608,14 +604,12 @@ void UI_LineBox::flags_callback(Fl_Widget *w, void *data)
 	int mask = l_f_c->mask;
 	int new_flags = box->CalcFlags();
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited flags of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			const LineDef *L = LineDefs[*it];
 
@@ -624,7 +618,6 @@ void UI_LineBox::flags_callback(Fl_Widget *w, void *data)
 			BA_ChangeLD(*it, LineDef::F_FLAGS, (L->flags & ~mask) | (new_flags & mask));
 		}
 
-		BA_MessageForSel("edited flags of", &list);
 		BA_End();
 	}
 }
@@ -641,19 +634,16 @@ void UI_LineBox::args_callback(Fl_Widget *w, void *data)
 
 	new_value = CLAMP(0, new_value, 255);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited args of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			BA_ChangeLD(*it, LineDef::F_TAG + arg_idx, new_value);
 		}
 
-		BA_MessageForSel("edited args of", &list);
 		BA_End();
 	}
 }
@@ -739,7 +729,7 @@ void UI_LineBox::UpdateField(int field)
 
 			const linetype_t *info = M_GetLineType(L->type);
 
-			if (Level_format == MAPF_Hexen)
+			if (Level_format != MAPF_Doom)
 			{
 				for (int a = 0 ; a < 5 ; a++)
 				{
@@ -855,9 +845,8 @@ void UI_LineBox::CalcLength()
 
 	float len_f = LineDefs[n]->CalcLength();
 
-	char buffer[300];
-
-	sprintf(buffer, "%1.0f", len_f);
+	char buffer[128];
+	snprintf(buffer, sizeof(buffer), "%1.0f", len_f);
 
 	length->value(buffer);
 }
@@ -866,7 +855,7 @@ void UI_LineBox::CalcLength()
 void UI_LineBox::FlagsFromInt(int lineflags)
 {
 	// compute activation
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		int new_act = (lineflags & MLF_Activation) >> 9;
 
@@ -922,7 +911,7 @@ int UI_LineBox::CalcFlags() const
 	if (f_mons->value())  lineflags |= MLF_BlockMonsters;
 	if (f_sound->value()) lineflags |= MLF_SoundBlock;
 
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		int actval = actkind->value();
 		if (actval >= 12) actval = 0;
@@ -931,13 +920,13 @@ int UI_LineBox::CalcFlags() const
 	}
 	else
 	{
-		if (game_info.pass_through && f_passthru->value())
+		if (Features.pass_through && f_passthru->value())
 			lineflags |= MLF_Boom_PassThru;
 
-		if (game_info.midtex_3d && f_3dmidtex->value())
+		if (Features.midtex_3d && f_3dmidtex->value())
 			lineflags |= MLF_Eternity_3DMidTex;
 
-		if (game_info.strife_flags)
+		if (Features.strife_flags)
 		{
 			if (f_jumpover->value())
 				lineflags |= MLF_Strife_JumpOver;
@@ -998,7 +987,7 @@ void UI_LineBox::UpdateGameInfo()
 {
 	choose->label("Choose");
 
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		tag->hide();
 		length->hide();
@@ -1018,17 +1007,17 @@ void UI_LineBox::UpdateGameInfo()
 		actkind->hide();
 		desc->resize(type->x(), desc->y(), w()-78, desc->h());
 
-		if (game_info.pass_through)
+		if (Features.pass_through)
 			f_passthru->show();
 		else
 			f_passthru->hide();
 
-		if (game_info.midtex_3d)
+		if (Features.midtex_3d)
 			f_3dmidtex->show();
 		else
 			f_3dmidtex->hide();
 
-		if (game_info.strife_flags)
+		if (Features.strife_flags)
 		{
 			f_jumpover->show();
 			f_flyers->show();
@@ -1043,7 +1032,7 @@ void UI_LineBox::UpdateGameInfo()
 			f_trans2->hide();
 		}
 
-		if (game_info.gen_types)
+		if (Features.gen_types)
 			gen->show();
 		else
 			gen->hide();
@@ -1051,7 +1040,7 @@ void UI_LineBox::UpdateGameInfo()
 
 	for (int a = 0 ; a < 5 ; a++)
 	{
-		if (Level_format == MAPF_Hexen)
+		if (Level_format != MAPF_Doom)
 			args[a]->show();
 		else
 			args[a]->hide();
@@ -1063,7 +1052,7 @@ void UI_LineBox::UpdateGameInfo()
 
 const char * UI_LineBox::GeneralizedDesc(int type_num)
 {
-	if (! game_info.gen_types)
+	if (! Features.gen_types)
 		return NULL;
 
 	static char desc_buffer[256];
@@ -1080,7 +1069,7 @@ const char * UI_LineBox::GeneralizedDesc(int type_num)
 
 			const char *trigger = info->fields[0].keywords[type_num & 7];
 
-			sprintf(desc_buffer, "%s GENTYPE: %s", trigger, info->name);
+			snprintf(desc_buffer, sizeof(desc_buffer), "%s GENTYPE: %s", trigger, info->name);
 			return desc_buffer;
 		}
 	}
