@@ -154,7 +154,7 @@ static void edit_do_last_sel(Fl_Widget *w, void * data)
 
 static void edit_do_op_menu(Fl_Widget *w, void * data)
 {
-	ExecuteCommand("OperationMenu");
+	ExecuteCommand("OpMenu");
 }
 
 static void edit_do_move(Fl_Widget *w, void * data)
@@ -232,24 +232,6 @@ static void view_do_gamma(Fl_Widget *w, void * data)
 	ExecuteCommand("Toggle", "gamma");
 }
 
-static void view_do_sector_render(Fl_Widget *w, void * data)
-{
-	const Fl_Menu_Item *item = ((Fl_Menu_*)w)->mvalue();
-
-	if (strcmp(item->text, "Floors") == 0)
-		edit.sector_render_mode = SREND_Floor;
-	else if (strcmp(item->text, "Ceilings") == 0)
-		edit.sector_render_mode = SREND_Ceiling;
-	else if (strcmp(item->text, "Lighting") == 0)
-		edit.sector_render_mode = SREND_Lighting;
-	else if (strcmp(item->text, "Sound") == 0)
-		edit.sector_render_mode = SREND_SoundProp;
-	else
-		edit.sector_render_mode = SREND_Nothing;
-
-	main_win->redraw();
-}
-
 static void view_do_default_props(Fl_Widget *w, void * data)
 {
 	ExecuteCommand("DefaultProps");
@@ -405,6 +387,11 @@ static void tools_do_view_logs(Fl_Widget *w, void * data)
 	ExecuteCommand("LogViewer");
 }
 
+static void tools_do_recalc_sectors(Fl_Widget *w, void * data)
+{
+	ExecuteCommand("RecalcSectors");
+}
+
 
 //------------------------------------------------------------------------
 //  HELP MENU
@@ -412,7 +399,7 @@ static void tools_do_view_logs(Fl_Widget *w, void * data)
 
 static void help_do_online_docs(Fl_Widget *w, void * data)
 {
-	ExecuteCommand("Documentation");
+	ExecuteCommand("OnlineDocs");
 }
 
 static void help_do_about(Fl_Widget *w, void * data)
@@ -430,6 +417,10 @@ static void help_do_about(Fl_Widget *w, void * data)
 #undef FCAL
 #define FCAL  (Fl_Callback *)
 
+// Note that the spaces at end of some menu strings are there to
+// prevent FLTK drawing the key binding hard against the item's
+// text.
+
 static Fl_Menu_Item menu_items[] =
 {
 	{ "&File", 0, 0, 0, FL_SUBMENU },
@@ -445,8 +436,6 @@ static Fl_Menu_Item menu_items[] =
 		{ M_RECENT_FILES, 0, 0, 0, FL_SUBMENU|FL_MENU_INACTIVE },
 			{ 0 },
 
-		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
-
 		{ "&Save Map",    FL_COMMAND + 's', FCAL file_do_save },
 		{ "&Export Map",  FL_COMMAND + 'e', FCAL file_do_export },
 
@@ -499,23 +488,12 @@ static Fl_Menu_Item menu_items[] =
 		// Note: FL_Tab cannot be used as a shortcut here, as it
 		//       invokes FLTK's hard-coded navigation stuff.
 
-		{ "Toggle &3D View",           0, FCAL view_do_toggle_3d },
 		{ "Toggle S&prites",     FL_F+10, FCAL view_do_sprites },
 		{ "Toggle &Gamma",       FL_F+11, FCAL view_do_gamma },
 		{ "Toggle Object Nums",  FL_F+12, FCAL view_do_object_nums },
 
 		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
 
-		{ "Sector Rendering", 0, 0, 0, FL_SUBMENU },
-			{ "Nothing",   0, FCAL view_do_sector_render },
-			{ "Floors",    0, FCAL view_do_sector_render },
-			{ "Ceilings",  0, FCAL view_do_sector_render },
-			{ "Lighting",  0, FCAL view_do_sector_render },
-			{ "Sound",     0, FCAL view_do_sector_render },
-			{ 0 },
-
-		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
-
 		{ "Zoom &In",      0, FCAL view_do_zoom_in },
 		{ "Zoom &Out",     0, FCAL view_do_zoom_out },
 		{ "&Whole Map",    0, FCAL view_do_whole_map },
@@ -524,6 +502,7 @@ static Fl_Menu_Item menu_items[] =
 		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
 
 		{ "&Default Props  ",  FL_COMMAND + 'd', FCAL view_do_default_props },
+		{ "Toggle &3D View",  0, FCAL view_do_toggle_3d },
 
 		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
 
@@ -576,16 +555,16 @@ static Fl_Menu_Item menu_items[] =
 
 	{ "&Tools", 0, 0, 0, FL_SUBMENU },
 
+#ifndef __APPLE__	// for macOS it will be in the app menu
 		{ "&Preferences",        FL_COMMAND + 'p', FCAL tools_do_preferences },
+#endif
 		{ "&View Logs",          0,  FCAL tools_do_view_logs },
+		{ "&Recalc Sectors",     0,  FCAL tools_do_recalc_sectors },
 
 		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
 
 		{ "&Test in Game",       FL_COMMAND + 't', FCAL tools_do_test_map },
 		{ "&Build All Nodes  ",  FL_COMMAND + 'b', FCAL tools_do_build_nodes },
-
-		{ "", 0, 0, 0, FL_MENU_DIVIDER|FL_MENU_INACTIVE },
-
 		{ "&Edit Text Lump  ",    0, FCAL tools_do_lump_editor },
 		{ "&Add BEHAVIOR Lump  ", 0, FCAL tools_do_add_behavior },
 		{ 0 },
@@ -607,7 +586,7 @@ static Fl_Menu_Item menu_items[] =
 
 //
 // allow the hard-coded menu shortcuts to be bind to other functions
-// by the user, hence we remove those shortcuts when this happens. 
+// by the user, hence we remove those shortcuts when this happens.
 //
 static void Menu_RemovedBoundKeys(Fl_Menu_Item *items)
 {
@@ -635,6 +614,51 @@ static void Menu_RemovedBoundKeys(Fl_Menu_Item *items)
 }
 
 
+//
+// remove the blank menu items with FL_MENU_DIVIDER flag, and
+// setting the flag on the previous item.  those blank lines
+// make for nicer looking menus in Windows/Linux, but are not
+// needed for the MacOS system menu bar.
+//
+void Menu_PackForMac(Fl_Menu_Item *src)
+{
+	int depth = 0;
+
+	Fl_Menu_Item *dest = src;
+
+	for (;;)
+	{
+		if (src->text == NULL)
+		{
+			*dest++ = *src++;
+
+			if (depth == 0)
+				break;
+
+			depth--;
+			continue;
+		}
+
+		if (src->flags & FL_SUBMENU)
+		{
+			*dest++ = *src++;
+
+			depth++;
+			continue;
+		}
+
+		if (src->text[0] == 0 && (src->flags & FL_MENU_DIVIDER))
+		{
+			dest[-1].flags |= FL_MENU_DIVIDER;
+			src++;
+			continue;
+		}
+
+		*dest++ = *src++;
+	}
+}
+
+
 static int Menu_FindItem(const Fl_Menu_Item *items, const char *text)
 {
 	int total = items[0].size();  // includes {0} at end
@@ -748,13 +772,11 @@ static Fl_Menu_Item * Menu_PopulateRecentFiles(Fl_Menu_Item *items, Fl_Callback
 
 	for (int k = 0 ; k < count ; k++)
 	{
-		char name_buf[256];
-
-		M_RecentShortName(k, name_buf);
+		std::string name = M_RecentShortName(k);
 
 		void *data = M_RecentData(k);
 
-		Menu_AddItem(pos, StringDup(name_buf), cb, data, 0);
+		Menu_AddItem(pos, StringDup(name.c_str()), cb, data, 0);
 	}
 
 	for ( ; menu_pos < total ; menu_pos++)
@@ -768,12 +790,14 @@ Fl_Sys_Menu_Bar * Menu_Create(int x, int y, int w, int h)
 {
 	Fl_Sys_Menu_Bar *bar = new Fl_Sys_Menu_Bar(x, y, w, h);
 
-#ifndef __APPLE__
-	bar->textsize(KF_fonth);
-#endif
-
 	Fl_Menu_Item *items = menu_items;
 
+#ifdef __APPLE__
+	Menu_PackForMac(items);
+#else
+	bar->textsize(16);
+#endif
+
 	Menu_RemovedBoundKeys(items);
 
 	items = Menu_PopulateGivenFiles(items);
@@ -781,6 +805,14 @@ Fl_Sys_Menu_Bar * Menu_Create(int x, int y, int w, int h)
 
 	bar->menu(items);
 
+	// for macOS, the preferences shall be in the app menu
+#ifdef __APPLE__
+	static const Fl_Menu_Item macPreferencesItem = {
+		"&Preferences\u2026", FL_COMMAND + ',', FCAL tools_do_preferences
+	};
+	Fl_Mac_App_Menu::custom_application_menu_items(&macPreferencesItem);
+#endif
+
 	return bar;
 }
 
