@@ -113,6 +113,21 @@ void UI_Pic::MarkMissing()
 	redraw();
 }
 
+void UI_Pic::MarkSpecial()
+{
+	Clear();
+
+	special = SP_Special;
+
+	color(fl_rgb_color(192, 0, 192));
+	labelcolor(FL_WHITE);
+	labelsize(40);
+
+	label("?");
+
+	redraw();
+}
+
 
 void UI_Pic::GetFlat(const char * fname)
 {
@@ -124,7 +139,12 @@ void UI_Pic::GetFlat(const char * fname)
 
 void UI_Pic::GetTex(const char * tname)
 {
-	if (is_null_tex(tname))
+	if (is_special_tex(tname))
+	{
+		MarkSpecial();
+		return;
+	}
+	else if (is_null_tex(tname))
 	{
 		Clear();
 		return;
@@ -138,8 +158,6 @@ void UI_Pic::GetTex(const char * tname)
 
 void UI_Pic::GetSprite(int type, Fl_Color back_color)
 {
-	//  color(FL_GRAY0 + 2);
-
 	Clear();
 
 	Img_c *img = W_GetSprite(type);
@@ -316,9 +334,12 @@ int UI_Pic::handle(int event)
 			return 1;
 
 		case FL_LEAVE:
-			main_win->SetCursor(FL_CURSOR_DEFAULT);
-			highlighted = false;
-			redraw();
+			if (highlighted)
+			{
+				main_win->SetCursor(FL_CURSOR_DEFAULT);
+				highlighted = false;
+				redraw();
+			}
 			return 1;
 
 		case FL_PUSH:
@@ -382,6 +403,12 @@ void UI_Pic::Selected(bool _val)
 }
 
 
+void UI_Pic::Unhighlight()
+{
+	handle(FL_LEAVE);
+}
+
+
 //------------------------------------------------------------------------
 
 
