@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2012-2018 Andrew Apted
+//  Copyright (C) 2012-2019 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -74,6 +74,7 @@ private:
 
 		key_name->color(FL_YELLOW, FL_YELLOW);
 		key_name->value("<\077\077\077>");
+		key_name->textcolor(FL_FOREGROUND_COLOR);
 		grab_but->deactivate();
 
 		Fl::focus(this);
@@ -106,6 +107,9 @@ private:
 				if (key)
 					key_name->value(M_KeyToString(key));
 
+				// if previous key was invalid, need to re-enable OK button
+				validate_callback(this, this);
+
 				return 1;
 			}
 
@@ -122,6 +126,9 @@ private:
 					key = new_key;
 					key_name->value(M_KeyToString(key));
 
+					// if previous key was invalid, need to re-enable OK button
+					validate_callback(this, this);
+
 					return 1;
 				}
 			}
@@ -290,7 +297,7 @@ private:
 
 		const char * tokens[32];
 
-		int num_tok = M_ParseLine(list, tokens, 32, false);
+		int num_tok = M_ParseLine(list, tokens, 32, 0 /* do_strings */);
 
 		if (num_tok < 1)	// shouldn't happen
 		{
@@ -661,13 +668,16 @@ public:
 	Fl_Input  *edit_def_port;
 	Fl_Choice *edit_def_mode;
 
-	Fl_Check_Button *edit_newislands;
 	Fl_Check_Button *edit_samemode;
 	Fl_Check_Button *edit_autoadjustX;
 	Fl_Check_Button *edit_add_del;
+	Fl_Check_Button *edit_full_1S;
 	Fl_Int_Input    *edit_sectorsize;
+	Fl_Input        *edit_userratio;
+	Fl_Choice       *edit_lineinfo;
 
 	Fl_Check_Button *brow_smalltex;
+	Fl_Check_Button *brow_combo;
 
 	/* Grid Tab */
 
@@ -681,6 +691,7 @@ public:
 	Fl_Check_Button *grid_hide_free;
 	Fl_Check_Button *grid_flatrender;
 	Fl_Check_Button *grid_spriterend;
+	Fl_Check_Button *grid_indicator;
 
 	Fl_Button *dotty_axis;
 	Fl_Button *dotty_major;
@@ -699,6 +710,8 @@ public:
 	Fl_Check_Button *rend_high_detail;
 	Fl_Check_Button *rend_lock_grav;
 
+	Fl_Choice *rend_far_clip;
+
 	/* Nodes Tab */
 
 	Fl_Check_Button *nod_on_save;
@@ -889,16 +902,21 @@ UI_Preferences::UI_Preferences() :
 		  edit_def_mode->align(FL_ALIGN_LEFT);
 		  edit_def_mode->add("Things|Linedefs|Sectors|Vertices");
 		}
-		{ edit_newislands = new Fl_Check_Button(50, 120, 265, 30, " new islands have void interior");
-		}
 		{ edit_autoadjustX = new Fl_Check_Button(50, 150, 260, 30, " auto-adjust X offsets");
 		}
 		{ edit_samemode = new Fl_Check_Button(50, 180, 270, 30, " same mode key will clear selection");
 		}
 		{ edit_add_del = new Fl_Check_Button(50, 210, 270, 30, " enable sidedef ADD / DEL buttons");
 		}
+		{ edit_full_1S = new Fl_Check_Button(50, 240, 270, 30, " show all textures on a one-sided linedef");
+		}
 		{ edit_sectorsize = new Fl_Int_Input(440, 120, 105, 25, "new sector size:");
 		}
+		{ edit_userratio = new Fl_Input(440, 150, 105, 25, "user ratio:");
+		}
+		{ edit_lineinfo = new Fl_Choice(440, 180, 105, 25, "line info:");
+		  edit_lineinfo->add("NONE|Length|Angle|Ratio|Len+Ang|Len+Ratio");
+		}
 
 		{ Fl_Box* o = new Fl_Box(25, 295, 355, 30, "Browser Options");
 		  o->labelfont(FL_BOLD);
@@ -906,6 +924,8 @@ UI_Preferences::UI_Preferences() :
 		}
 		{ brow_smalltex = new Fl_Check_Button(50, 330, 265, 30, " smaller textures");
 		}
+		{ brow_combo = new Fl_Check_Button(50, 360, 265, 30, " combine flats and textures in a single browser");
+		}
 		o->end();
 	  }
 
@@ -925,18 +945,20 @@ UI_Preferences::UI_Preferences() :
 		}
 		{ grid_enabled = new Fl_Check_Button(50, 125, 95, 25, " default grid to ON");
 		}
-		{ grid_snap = new Fl_Check_Button(50, 155, 235, 25, " default SNAP mode");
+		{ grid_snap = new Fl_Check_Button(50, 155, 235, 25, " default snap to ON");
 		}
-		{ grid_flatrender = new Fl_Check_Button(50, 185, 270, 25, " default sector rendering to ON");
+		{ grid_flatrender = new Fl_Check_Button(50, 185, 270, 25, " default sector-render to ON");
 		}
-		{ grid_spriterend = new Fl_Check_Button(50, 215, 270, 25, " default sprite rendering to ON");
+		{ grid_spriterend = new Fl_Check_Button(50, 215, 270, 25, " default sprites to ON");
 		}
-		{ grid_size = new Fl_Choice(400, 90, 95, 25, "default grid size ");
+		{ grid_size = new Fl_Choice(420, 90, 95, 25, "default grid size ");
 		  grid_size->add("1024|512|256|192|128|64|32|16|8|4|2");
 		}
-		{ gen_scrollbars = new Fl_Check_Button(277, 125, 245, 25, " enable scroll-bars for map view");
+		{ gen_scrollbars = new Fl_Check_Button(300, 125, 245, 25, " enable scroll-bars for map view");
 		}
-		{ grid_hide_free = new Fl_Check_Button(277, 155, 245, 25, " hide grid in FREE mode");
+		{ grid_indicator = new Fl_Check_Button(300, 155, 245, 25, " enable snap-pos indicator");
+		}
+		{ grid_hide_free = new Fl_Check_Button(300, 185, 245, 25, " hide grid in FREE mode");
 		}
 
 		{ Fl_Box* o = new Fl_Box(25, 270, 355, 30, "Grid Colors");
@@ -1013,9 +1035,18 @@ UI_Preferences::UI_Preferences() :
 		  Fl_Box* o = new Fl_Box(300, 90, 150, 25, "(higher is wider, default is 0.83)");
 		  o->align(Fl_Align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE));
 		}
-		{ rend_high_detail = new Fl_Check_Button(50, 125, 360, 30, " High detail -- slower but looks better");
+		{ rend_lock_grav = new Fl_Check_Button(50, 125, 360, 30, " Locked gravity -- cannot move up or down");
 		}
-		{ rend_lock_grav = new Fl_Check_Button(50, 155, 360, 30, " Locked gravity -- cannot move up or down");
+		{ rend_high_detail = new Fl_Check_Button(50, 155, 360, 30, " High detail -- slower but looks better");
+#ifndef NO_OPENGL
+		  rend_high_detail->hide();
+#endif
+		}
+		{ rend_far_clip = new Fl_Choice(195, 160, 100, 30, "Far clip distance: ");
+		  rend_far_clip->add("32768|16384|8192|4096|2048|1024");
+#ifdef NO_OPENGL
+		  rend_far_clip->hide();
+#endif
 		}
 
 		o->end();
@@ -1423,14 +1454,20 @@ void UI_Preferences::LoadValues()
 
 	edit_def_port->value(default_port);
 	edit_def_mode->value(CLAMP(0, default_edit_mode, 3));
+	edit_lineinfo->value(CLAMP(0, highlight_line_info, 5));
 
 	edit_sectorsize->value(Int_TmpStr(new_sector_size));
-	edit_newislands->value(new_islands_are_void ? 1 : 0);
 	edit_samemode->value(same_mode_clears_selection ? 1 : 0);
 	edit_add_del->value(sidedef_add_del_buttons ? 1 : 0);
+	edit_full_1S->value(show_full_one_sided ? 1 : 0);
 	edit_autoadjustX->value(leave_offsets_alone ? 0 : 1);
 
 	brow_smalltex->value(browser_small_tex ? 1 : 0);
+	brow_combo->value(browser_combine_tex ? 1 : 0);
+
+	char ratio_buf[256];
+	snprintf(ratio_buf, sizeof(ratio_buf), "%d:%d", grid_ratio_high, grid_ratio_low);
+	edit_userratio->value(ratio_buf);
 
 	/* Grid Tab */
 
@@ -1447,6 +1484,7 @@ void UI_Preferences::LoadValues()
 	grid_hide_free ->value(grid_hide_in_free_mode ? 1 : 0);
 	grid_flatrender->value(sector_render_default ? 1 : 0);
 	grid_spriterend->value(thing_render_default ? 1 : 0);
+	grid_indicator->value(grid_snap_indicator ? 1 : 0);
 
 	gen_scrollbars ->value(map_scroll_bars ? 1 : 0);
 
@@ -1465,12 +1503,25 @@ void UI_Preferences::LoadValues()
 	render_pixel_aspect = CLAMP(25, render_pixel_aspect, 400);
 
 	char aspect_buf[64];
-	sprintf(aspect_buf, "%1.2f", render_pixel_aspect / 100.0);
+	snprintf(aspect_buf, sizeof(aspect_buf), "%1.2f", render_pixel_aspect / 100.0);
 	rend_aspect->value(aspect_buf);
 
 	rend_high_detail->value(render_high_detail ? 1 : 0);
 	rend_lock_grav->value(render_lock_gravity ? 1 : 0);
 
+	if (render_far_clip > 24000)
+		rend_far_clip->value(0);
+	else if (render_far_clip > 12000)
+		rend_far_clip->value(1);
+	else if (render_far_clip > 6000)
+		rend_far_clip->value(2);
+	else if (render_far_clip > 3000)
+		rend_far_clip->value(3);
+	else if (render_far_clip > 1500)
+		rend_far_clip->value(4);
+	else
+		rend_far_clip->value(5);
+
 	/* Nodes Tab */
 
 	nod_on_save->value(bsp_on_save ? 1 : 0);
@@ -1545,23 +1596,39 @@ void UI_Preferences::SaveValues()
 
 	default_port = StringDup(edit_def_port->value());
 	default_edit_mode = edit_def_mode->value();
+	highlight_line_info = edit_lineinfo->value();
 
 	new_sector_size = atoi(edit_sectorsize->value());
 	new_sector_size = CLAMP(4, new_sector_size, 8192);
 
-	new_islands_are_void = edit_newislands->value() ? true : false;
 	same_mode_clears_selection = edit_samemode->value() ? true : false;
 	sidedef_add_del_buttons = edit_add_del->value() ? true : false;
+	show_full_one_sided = edit_full_1S->value() ? true : false;
 	leave_offsets_alone = edit_autoadjustX->value() ? false : true;
 
 	// changing this requires re-populating the browser
 	bool new_small_tex = brow_smalltex->value() ? true : false;
-	if (new_small_tex != browser_small_tex)
+	bool new_combo = brow_combo->value() ? true : false;
+
+	if (new_small_tex != browser_small_tex || new_combo != browser_combine_tex)
 	{
 		browser_small_tex = new_small_tex;
+		browser_combine_tex = new_combo;
+
 		main_win->browser->Populate();
 	}
 
+	// decode the user ratio
+	grid_ratio_low = grid_ratio_high = -1;
+	sscanf(edit_userratio->value(), "%d:%d", &grid_ratio_high, &grid_ratio_low);
+	if (grid_ratio_high < 1) grid_ratio_high = 3;
+	if (grid_ratio_low  < 1) grid_ratio_low  = 1;
+
+	if (grid_ratio_low > grid_ratio_high)
+		std::swap(grid_ratio_low, grid_ratio_high);
+
+	main_win->info_bar->UpdateRatio();
+
 	/* Grid Tab */
 
 	grid_style        = grid_cur_style->value();
@@ -1569,6 +1636,7 @@ void UI_Preferences::SaveValues()
 	grid_default_snap = grid_snap->value() ? true : false;
 	grid_default_size = atoi(grid_size->mvalue()->text);
 	grid_hide_in_free_mode = grid_hide_free ->value() ? true : false;
+	grid_snap_indicator    = grid_indicator ->value() ? true : false;
 	sector_render_default  = grid_flatrender->value() ? 1 : 0;
 	thing_render_default   = grid_spriterend->value() ? 1 : 0;
 
@@ -1609,7 +1677,7 @@ void UI_Preferences::SaveValues()
 
 	render_high_detail  = rend_high_detail->value() ? true : false;
 	render_lock_gravity = rend_lock_grav->value() ? true : false;
-
+	render_far_clip     = atoi(rend_far_clip->mvalue()->text);
 }
 
 
