@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2015-2018 Andrew Apted
+//  Copyright (C) 2015-2020 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -21,6 +21,7 @@
 #include "main.h"
 #include "ui_window.h"
 
+#include "e_cutpaste.h"  // Texboard_XX
 #include "e_path.h"  // GoToObject
 #include "e_main.h"  // Selection_Clear
 #include "m_config.h"	// gui_scheme
@@ -272,9 +273,9 @@ public:
 
 UI_FindAndReplace::UI_FindAndReplace(int X, int Y, int W, int H) :
 	Fl_Group(X, Y, W, H, NULL),
+	cur_obj(),
 	find_numbers(new number_group_c),
-	 tag_numbers(new number_group_c),
-	cur_obj(OBJ_THINGS, -1)
+	 tag_numbers(new number_group_c)
 {
 	box(FL_FLAT_BOX);
 
@@ -294,7 +295,7 @@ UI_FindAndReplace::UI_FindAndReplace(int X, int Y, int W, int H) :
 
 		Fl_Box *title = new Fl_Box(X + 60, Y + 10, W - 70, 30, "Find and Replace");
 		title->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
-		title->labelsize(18+KF*4);
+		title->labelsize(22);
 
 
 		what = new Fl_Choice(X+60, Y+46, W - 120, 33);
@@ -314,7 +315,8 @@ UI_FindAndReplace::UI_FindAndReplace(int X, int Y, int W, int H) :
 
 		find_pic = new UI_Pic(X+225, Y+95, 64, 64, "Choose");
 		find_pic->callback((Fl_Callback *)choose_callback, this);
-///		find_pic->AllowHighlight(true);
+		// NOTE: no point allowing highlight when copy'n'paste don't work
+		// find_pic->AllowHighlight(true);
 
 		find_but = new Fl_Button(X+50, Y+165, 90, 30, "Find");
 		find_but->labelfont(FL_HELVETICA_BOLD);
@@ -339,7 +341,7 @@ UI_FindAndReplace::UI_FindAndReplace(int X, int Y, int W, int H) :
 
 		rep_pic = new UI_Pic(X+225, Y+230, 64, 64, "Choose");
 		rep_pic->callback((Fl_Callback *)choose_callback, this);
-///		rep_pic->AllowHighlight(true);
+		// rep_pic->AllowHighlight(true);
 
 		apply_but = new Fl_Button(X+45, Y+300, 90, 30, "Replace");
 		apply_but->labelfont(FL_HELVETICA_BOLD);
@@ -449,7 +451,7 @@ void UI_FindAndReplace::UpdateWhatFilters()
 	if (test) (w)->show(); else (w)->hide();
 
 	// common stuff
-	if (x == 0 && Level_format != MAPF_Hexen)
+	if (x == 0 && Level_format == MAPF_Doom)
 	{
 		tag_input->deactivate();
 		tag_input->value("");
@@ -484,7 +486,7 @@ void UI_FindAndReplace::UpdateWhatFilters()
 #undef SHOW_WIDGET_IF
 
 	// vanilla DOOM : always hide SP and COOP flags
-	if (x == 0 && ! game_info.coop_dm_flags && Level_format != MAPF_Hexen)
+	if (x == 0 && ! Features.coop_dm_flags && Level_format == MAPF_Doom)
 	{
 		  o_sp->hide();
 		o_coop->hide();
@@ -831,8 +833,8 @@ bool UI_FindAndReplace::CheckInput(Fl_Input *w, Fl_Output *desc, UI_Pic *pic, nu
 
 		case 4: // Sectors by Type
 		{
-			int mask = (game_info.gen_sectors == 2) ? 255 :
-						(game_info.gen_sectors) ? 31 : 65535;
+			int mask = (Features.gen_sectors == 2) ? 255 :
+						(Features.gen_sectors) ? 31 : 65535;
 
 			const sectortype_t * info = M_GetSectorType(type_num & mask);
 			desc->value(info->desc);
@@ -903,18 +905,135 @@ char UI_FindAndReplace::GetKind()
 
 bool UI_FindAndReplace::ClipboardOp(char op)
 {
-	// hmmm, review this
-	fl_beep();
+	// NOTE: the logic below is disabled since texture copy'n'paste
+	//       simply does not work properly when an input widget has
+	//       the focus (especially paste).
+
+#if 1
+	return false;
+#else
+	// current mode must be Line Textures or Sector Flats
+	if (what->value() < 1 || what->value() > 2)
+		return false;
+
+	bool is_replace = false;
+
+	// a highlighted pic overrides any selected one
+	if (find_pic->Highlighted())
+		is_replace = false;
+	else if (rep_pic->Highlighted())
+		is_replace = true;
+	else if (find_pic->Selected())
+		is_replace = false;
+	else if (rep_pic->Selected())
+		is_replace = true;
+	else
+		return false;
+
+	switch (op)
+	{
+	case 'c':
+		CB_Copy(is_replace);
+		break;
+
+	case 'v':
+		CB_Paste(is_replace);
+		break;
+
+	case 'x':
+		Beep("cannot cut that");
+		break;
+
+	case 'd':
+		CB_Delete(is_replace);
+		break;
+	}
+
 	return true;
+#endif
+}
+
+
+void UI_FindAndReplace::CB_Copy(bool is_replace)
+{
+	const char *tex_name = is_replace ? rep_value->value() : find_match->value();
+
+	// check for a valid, single name
+	if (strchr(tex_name, ',') != NULL)
+		tex_name = "";
+
+	if (is_null_tex(tex_name))
+		tex_name = "";
+
+	if (tex_name[0])
+	{
+		bool is_known = (what->value() == 1) ?
+			W_TextureIsKnown(tex_name) : W_FlatIsKnown(tex_name);
+
+		if (!is_known)
+			tex_name = "";
+	}
+
+	if (tex_name[0] == 0)
+	{
+		Beep("invalid texture name");
+		return;
+	}
+
+	if (what->value() == 1)
+		Texboard_SetTex(tex_name);
+	else
+		Texboard_SetFlat(tex_name);
+}
+
+
+void UI_FindAndReplace::CB_Paste(bool is_replace)
+{
+	Fl_Input *inp = is_replace ? rep_value : find_match;
+
+	int tex_num = (what->value() == 1) ?
+		Texboard_GetTexNum() : Texboard_GetFlatNum();
+
+	const char *tex_name = BA_GetString(tex_num);
+
+	InsertName(inp, false /* append */, tex_name);
+}
+
+
+void UI_FindAndReplace::CB_Delete(bool is_replace)
+{
+	if (!is_replace)
+	{
+		find_match->value("");
+		find_pic->Selected(false);
+
+		find_match_callback(find_match, this);
+	}
+	else
+	{
+		rep_value->value("");
+		rep_pic->Selected(false);
+
+		rep_value_callback(rep_value, this);
+	}
 }
 
 
 void UI_FindAndReplace::BrowsedItem(char kind, int number, const char *name, int e_state)
 {
+	if (kind == 'F')
+		kind = 'T';
+
 	if (kind != GetKind())
 	{
-		fl_beep();
-		return;
+		if (kind == 'T' && GetKind() == 'F')
+		{
+			/* ok */
+		}
+		else
+		{
+			return;
+		}
 	}
 
 	// determine which box the user intended
@@ -928,10 +1047,7 @@ void UI_FindAndReplace::BrowsedItem(char kind, int number, const char *name, int
 	}
 
 	if (sel_pics == 0)
-	{
-		fl_beep();
-		return;
-	}
+		sel_pics = 1;
 
 	bool is_replace = (sel_pics == 2);
 
@@ -995,9 +1111,8 @@ void UI_FindAndReplace::InsertName(Fl_Input *inp, char append, const char *name)
 
 void UI_FindAndReplace::InsertNumber(Fl_Input *inp, char append, int number)
 {
-	char buf[256];
-
-	sprintf(buf, "%d", number);
+	char buf[64];
+	snprintf(buf, sizeof(buf), "%d", number);
 
 	InsertName(inp, append, buf);
 }
@@ -1074,8 +1189,6 @@ void UI_FindAndReplace::choose_callback(UI_Pic *w, void *data)
 			box->find_pic->Selected(false);
 		}
 	}
-
-	Render3D_ClearSelection();
 }
 
 
@@ -1142,7 +1255,7 @@ bool UI_FindAndReplace::FindNext()
 
 			GoToObject(cur_obj);
 
-			Status_Set("Found #%d", idx);
+			Status_Set("found #%d", idx);
 			return true;
 		}
 	}
@@ -1183,11 +1296,10 @@ void UI_FindAndReplace::DoReplace()
 	int replace_tex_id = BA_InternaliseString(NormalizeTex(rep_value->value()));
 
 	BA_Begin();
+	BA_Message("replacement in %s #%d", NameForObjectType(cur_obj.type), cur_obj.num);
 
 	ApplyReplace(cur_obj.num, replace_tex_id);
 
-	BA_Message("replacement in %s #%d", NameForObjectType(cur_obj.type), cur_obj.num);
-
 	BA_End();
 
 	// move onto next object
@@ -1311,12 +1423,11 @@ void UI_FindAndReplace::DoAll(bool replace)
 	if (count == 0)
 		Beep("Nothing found");
 	else
-		Status_Set("Found %d objects", count);
+		Status_Set("found %d objects", count);
 
 	if (replace)
 	{
 		BA_MessageForSel("replacement in", edit.Selected);
-
 		BA_End();
 	}
 
@@ -1443,8 +1554,8 @@ bool UI_FindAndReplace::Match_SectorType(int idx)
 {
 	const Sector *SEC = Sectors[idx];
 
-	int mask = (game_info.gen_sectors == 2) ? 255 :
-				(game_info.gen_sectors) ? 31 : 65535;
+	int mask = (Features.gen_sectors == 2) ? 255 :
+				(Features.gen_sectors) ? 31 : 65535;
 
 	if (! find_numbers->get(SEC->type & mask))
 		return false;
@@ -1519,7 +1630,7 @@ void UI_FindAndReplace::ComputeFlagMask()
 	FLAG_FROM_WIDGET(o_medium, 1, MTF_Medium);
 	FLAG_FROM_WIDGET(  o_hard, 1, MTF_Hard);
 
-	if (game_info.coop_dm_flags)
+	if (Features.coop_dm_flags)
 	{
 		FLAG_FROM_WIDGET(  o_sp, -1, MTF_Not_SP);
 		FLAG_FROM_WIDGET(o_coop, -1, MTF_Not_COOP);
@@ -1662,8 +1773,8 @@ void UI_FindAndReplace::Replace_LineType(int idx)
 
 void UI_FindAndReplace::Replace_SectorType(int idx)
 {
-	int mask = (game_info.gen_sectors == 2) ? 255 :
-				(game_info.gen_sectors) ? 31 : 65535;
+	int mask = (Features.gen_sectors == 2) ? 255 :
+				(Features.gen_sectors) ? 31 : 65535;
 
 	int old_type = Sectors[idx]->type;
 	int new_type = atoi(rep_value->value());
