@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2007-2018 Andrew Apted
+//  Copyright (C) 2007-2019 Andrew Apted
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include "ui_window.h"
 
 #include "e_checks.h"
+#include "e_cutpaste.h"
 #include "e_main.h"
 #include "e_sector.h"
 #include "e_things.h"
@@ -253,14 +254,16 @@ void UI_SectorBox::height_callback(Fl_Widget *w, void *data)
 	f_h = CLAMP(-32767, f_h, 32767);
 	c_h = CLAMP(-32767, c_h, 32767);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		if (w == box->floor_h)
+			BA_MessageForSel("edited floor of", edit.Selected);
+		else
+			BA_MessageForSel("edited ceiling of", edit.Selected);
+
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			if (w == box->floor_h)
 				BA_ChangeSEC(*it, Sector::F_FLOORH, f_h);
@@ -268,11 +271,6 @@ void UI_SectorBox::height_callback(Fl_Widget *w, void *data)
 				BA_ChangeSEC(*it, Sector::F_CEILH, c_h);
 		}
 
-		if (w == box->floor_h)
-			BA_MessageForSel("edited floor of", &list);
-		else
-			BA_MessageForSel("edited ceiling of", &list);
-
 		BA_End();
 
 		box-> floor_h->value(Int_TmpStr(f_h));
@@ -296,14 +294,12 @@ void UI_SectorBox::headroom_callback(Fl_Widget *w, void *data)
 		}
 	}
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited headroom of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			int new_h = Sectors[*it]->floorh + room;
 
@@ -312,7 +308,6 @@ void UI_SectorBox::headroom_callback(Fl_Widget *w, void *data)
 			BA_ChangeSEC(*it, Sector::F_CEILH, new_h);
 		}
 
-		BA_MessageForSel("edited headroom of", &list);
 		BA_End();
 
 		box->UpdateField();
@@ -330,14 +325,11 @@ void UI_SectorBox::tex_callback(Fl_Widget *w, void *data)
 	bool is_pic   = (w == box->f_pic || w == box->c_pic);
 	bool is_floor = (w == box->f_pic || w == box->f_tex);
 
-	int new_tex;
-
 	// MMB on ceiling flat image sets to sky
 	if (w == box->c_pic && Fl::event_button() == 2)
 	{
-		new_tex = BA_InternaliseString(game_info.sky_flat);
-
-		goto change_it;
+		box->SetFlat(Misc_info.sky_flat, PART_CEIL);
+		return;
 	}
 
 	// LMB on the flat image just selects/unselects it (red border)
@@ -347,49 +339,52 @@ void UI_SectorBox::tex_callback(Fl_Widget *w, void *data)
 
 		pic->Selected(! pic->Selected());
 
-		Render3D_ClearSelection();
-
 		if (pic->Selected())
 			main_win->BrowserMode('F');
 		return;
 	}
 
+	const char *new_flat;
+
 	// right click sets to default value
 	// [ Note the 'is_pic' check prevents a bug when using RMB in browser ]
 	if (is_pic && Fl::event_button() == 3)
-	{
-		new_tex = BA_InternaliseString(is_floor ? default_floor_tex : default_ceil_tex);
-	}
+		new_flat = is_floor ? default_floor_tex : default_ceil_tex;
 	else if (is_floor)
-	{
-		new_tex = BA_InternaliseString(NormalizeTex(box->f_tex->value()));
-	}
+		new_flat = NormalizeTex(box->f_tex->value());
 	else
-	{
-		new_tex = BA_InternaliseString(NormalizeTex(box->c_tex->value()));
-	}
+		new_flat = NormalizeTex(box->c_tex->value());
 
-change_it:
-	selection_c list;
-	selection_iterator_c it;
+	box->InstallFlat(new_flat, is_floor ? PART_FLOOR : PART_CEIL);
+}
 
-	if (GetCurrentObjects(&list))
+
+void UI_SectorBox::InstallFlat(const char *name, int filter_parts)
+{
+	int tex_num = BA_InternaliseString(name);
+
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited texture on", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
-			if (is_floor)
-				BA_ChangeSEC(*it, Sector::F_FLOOR_TEX, new_tex);
-			else
-				BA_ChangeSEC(*it, Sector::F_CEIL_TEX, new_tex);
+			int parts = edit.Selected->get_ext(*it);
+			if (parts == 1)
+				parts = filter_parts;
+
+			if (parts & filter_parts & PART_FLOOR)
+				BA_ChangeSEC(*it, Sector::F_FLOOR_TEX, tex_num);
+
+			if (parts & filter_parts & parts & PART_CEIL)
+				BA_ChangeSEC(*it, Sector::F_CEIL_TEX, tex_num);
 		}
 
-		BA_MessageForSel("edited texture on", &list);
 		BA_End();
-
-		box->UpdateField();
 	}
+
+	UpdateField();
 }
 
 
@@ -413,27 +408,15 @@ void UI_SectorBox::dyntex_callback(Fl_Widget *w, void *data)
 }
 
 
-void UI_SectorBox::SetFlat(const char *name, int e_state)
+void UI_SectorBox::SetFlat(const char *name, int parts)
 {
-	if (obj < 0)
-		return;
-
-	int sel_pics = GetSelectedPics();
-
-	if (sel_pics == 0)
-		sel_pics = (e_state & FL_BUTTON3) ? 2 : 1;
-
-	if (sel_pics & 1)
-	{
+	if (parts & PART_FLOOR)
 		f_tex->value(name);
-		f_tex->do_callback();
-	}
 
-	if (sel_pics & 2)
-	{
+	if (parts & PART_CEIL)
 		c_tex->value(name);
-		c_tex->do_callback();
-	}
+
+	InstallFlat(name, parts);
 }
 
 
@@ -444,7 +427,7 @@ void UI_SectorBox::type_callback(Fl_Widget *w, void *data)
 	int mask  = 65535;
 	int value = atoi(box->type->value());
 
-	int gen_mask = (game_info.gen_sectors == 2) ? 255 : 31;
+	int gen_mask = (Features.gen_sectors == 2) ? 255 : 31;
 
 	// when generalize sectors active, typing a low value (which does
 	// not touch any bitflags) should just update the TYPE part, and
@@ -456,11 +439,10 @@ void UI_SectorBox::type_callback(Fl_Widget *w, void *data)
 		// update the WHOLE sector type.  If generalized sectors are
 		// active, then the panel will reinterpret the typed value.
 	}
-	else if (game_info.gen_sectors)
+	else if (Features.gen_sectors)
 	{
 		// Boom and ZDoom generalized sectors
 
-		// we fix this shortly...
 		mask = 0;
 
 		if (w == box->bm_damage)
@@ -488,7 +470,7 @@ void UI_SectorBox::type_callback(Fl_Widget *w, void *data)
 		{
 			mask = gen_mask;
 		}
-		else if (game_info.gen_sectors == 2)
+		else if (Features.gen_sectors == 2)
 		{
 			// for ZDoom in Hexen mode, shift up 3 bits
 			mask  <<= 3;
@@ -496,28 +478,31 @@ void UI_SectorBox::type_callback(Fl_Widget *w, void *data)
 		}
 	}
 
+	box->InstallSectorType(mask, value);
+}
+
+
+void UI_SectorBox::InstallSectorType(int mask, int value)
+{
 	value &= mask;
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited type of", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
 			int old_type = Sectors[*it]->type;
 
 			BA_ChangeSEC(*it, Sector::F_TYPE, (old_type & ~mask) | value);
 		}
 
-		BA_MessageForSel("edited type of", &list);
 		BA_End();
 	}
 
 	// update the description
-	box->UpdateField(Sector::F_TYPE);
+	UpdateField(Sector::F_TYPE);
 }
 
 
@@ -533,9 +518,9 @@ void UI_SectorBox::dyntype_callback(Fl_Widget *w, void *data)
 	// when generalize sectors in effect, the name should just
 	// show the TYPE part of the sector type.
 
-	if (game_info.gen_sectors)
+	if (Features.gen_sectors)
 	{
-		int gen_mask = (game_info.gen_sectors == 2) ? 255 : 31;
+		int gen_mask = (Features.gen_sectors == 2) ? 255 : 31;
 
 		value &= gen_mask;
 	}
@@ -552,11 +537,14 @@ void UI_SectorBox::SetSectorType(int new_type)
 		return;
 
 	char buffer[64];
-
-	sprintf(buffer, "%d", new_type);
+	snprintf(buffer, sizeof(buffer), "%d", new_type);
 
 	type->value(buffer);
-	type->do_callback();
+
+	int mask = (Features.gen_sectors == 2) ? 255 :
+			   (Features.gen_sectors == 1) ? 31  : 65535;
+
+	InstallSectorType(mask, new_type);
 }
 
 
@@ -573,19 +561,16 @@ void UI_SectorBox::light_callback(Fl_Widget *w, void *data)
 	if (new_lt < 0)
 		new_lt = 0;
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited light of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			BA_ChangeSEC(*it, Sector::F_LIGHT, new_lt);
 		}
 
-		BA_MessageForSel("edited light of", &list);
 		BA_End();
 	}
 }
@@ -599,7 +584,8 @@ void UI_SectorBox::tag_callback(Fl_Widget *w, void *data)
 
 	new_tag = CLAMP(-32767, new_tag, 32767);
 
-	Tags_ApplyNewValue(new_tag);
+	if (! edit.Selected->empty())
+		Tags_ApplyNewValue(new_tag);
 }
 
 
@@ -620,7 +606,8 @@ void UI_SectorBox::FreshTag()
 		return;
 	}
 
-	Tags_ApplyNewValue(new_tag);
+	if (! edit.Selected->empty())
+		Tags_ApplyNewValue(new_tag);
 }
 
 
@@ -766,17 +753,17 @@ void UI_SectorBox::UpdateField(int field)
 		if (is_sector(obj))
 		{
 			int value = Sectors[obj]->type;
-			int mask  = (game_info.gen_sectors == 2) ? 255 :
-						(game_info.gen_sectors) ? 31 : 65535;
+			int mask  = (Features.gen_sectors == 2) ? 255 :
+						(Features.gen_sectors) ? 31 : 65535;
 
 			type->value(Int_TmpStr(value & mask));
 
 			const sectortype_t *info = M_GetSectorType(value & mask);
 			desc->value(info->desc);
 
-			if (game_info.gen_sectors)
+			if (Features.gen_sectors)
 			{
-				if (game_info.gen_sectors == 2)
+				if (Features.gen_sectors == 2)
 					value >>= 3;
 
 				bm_damage->value((value >> 5) & 3);
@@ -811,20 +798,20 @@ void UI_SectorBox::UpdateField(int field)
 
 int UI_SectorBox::GetSelectedPics() const
 {
-	return	(f_pic->Selected() ? 1 : 0) |
-			(c_pic->Selected() ? 2 : 0);
+	return	(f_pic->Selected() ? PART_FLOOR : 0) |
+			(c_pic->Selected() ? PART_CEIL  : 0);
 }
 
 int UI_SectorBox::GetHighlightedPics() const
 {
-	return	(f_pic->Highlighted() ? 1 : 0) |
-			(c_pic->Highlighted() ? 2 : 0);
+	return	(f_pic->Highlighted() ? PART_FLOOR : 0) |
+			(c_pic->Highlighted() ? PART_CEIL  : 0);
 }
 
 
-void UI_SectorBox::CB_Copy()
+void UI_SectorBox::CB_Copy(int parts)
 {
-	if (GetSelectedPics() == 3)
+	if (parts == (PART_FLOOR | PART_CEIL))
 	{
 		Beep("multiple textures");
 		return;
@@ -832,78 +819,57 @@ void UI_SectorBox::CB_Copy()
 
 	const char *name = NULL;
 
-	if (GetSelectedPics() == 2 ||
-		(GetSelectedPics() == 0 && c_pic->Highlighted()))
-	{
+	if (parts == PART_CEIL)
 		name = c_tex->value();
-	}
 	else
-	{
 		name = f_tex->value();
-	}
 
-	r_clipboard.SetFlat(name);
+	Texboard_SetFlat(name);
 
-	Status_Set("Copied %s", name);
+	Status_Set("copied %s", name);
 }
 
 
-void UI_SectorBox::CB_Paste(int new_tex)
+void UI_SectorBox::CB_Paste(int parts, int new_tex)
 {
-	int sel_pics = GetSelectedPics();
-
-	if (sel_pics == 0)
-		sel_pics = GetHighlightedPics();
-
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
+	if (edit.Selected->empty())
 		return;
 
 	BA_Begin();
+	BA_Message("pasted %s", BA_GetString(new_tex));
 
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
-		if (sel_pics & 1) BA_ChangeSEC(*it, Sector::F_FLOOR_TEX, new_tex);
-		if (sel_pics & 2) BA_ChangeSEC(*it, Sector::F_CEIL_TEX,  new_tex);
+		if (parts & PART_FLOOR) BA_ChangeSEC(*it, Sector::F_FLOOR_TEX, new_tex);
+		if (parts & PART_CEIL)  BA_ChangeSEC(*it, Sector::F_CEIL_TEX,  new_tex);
 	}
 
-	BA_Message("Pasted %s", BA_GetString(new_tex));
 	BA_End();
 
 	UpdateField();
 }
 
 
-void UI_SectorBox::CB_Cut()
+void UI_SectorBox::CB_Cut(int parts)
 {
-	int sel_pics = GetSelectedPics();
-
-	if (sel_pics == 0)
-		sel_pics = GetHighlightedPics();
-
 	int new_floor = BA_InternaliseString(default_floor_tex);
 	int new_ceil  = BA_InternaliseString(default_ceil_tex);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (! GetCurrentObjects(&list))
-		return;
-
-	BA_Begin();
-
-	for (list.begin(&it) ; !it.at_end() ; ++it)
+	if (! edit.Selected->empty())
 	{
-		if (sel_pics & 1) BA_ChangeSEC(*it, Sector::F_FLOOR_TEX, new_floor);
-		if (sel_pics & 2) BA_ChangeSEC(*it, Sector::F_CEIL_TEX,  new_ceil);
+		BA_Begin();
+		BA_MessageForSel("cut texture on", edit.Selected);
+
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
+		{
+			if (parts & PART_FLOOR) BA_ChangeSEC(*it, Sector::F_FLOOR_TEX, new_floor);
+			if (parts & PART_CEIL)  BA_ChangeSEC(*it, Sector::F_CEIL_TEX,  new_ceil);
+		}
+
+		BA_End();
+
+		UpdateField();
 	}
-
-	BA_MessageForSel("cut texture on", &list);
-	BA_End();
-
-	UpdateField();
 }
 
 
@@ -912,29 +878,31 @@ bool UI_SectorBox::ClipboardOp(char op)
 	if (obj < 0)
 		return false;
 
-	if (!  (f_pic->Selected() || f_pic->Highlighted() ||
-			c_pic->Selected() || c_pic->Highlighted()))
-	{
+	int parts = GetSelectedPics();
+
+	if (parts == 0)
+		parts = GetHighlightedPics();
+
+	if (parts == 0)
 		return false;
-	}
 
 	switch (op)
 	{
 		case 'c':
-			CB_Copy();
+			CB_Copy(parts);
 			break;
 
 		case 'v':
-			CB_Paste(r_clipboard.GetFlatNum());
+			CB_Paste(parts, Texboard_GetFlatNum());
 			break;
 
 		case 'x':
-			CB_Cut();
+			CB_Cut(parts);
 			break;
 
 		case 'd':
-			// we abuse the delete function to turn sector ceilings into sky
-			CB_Paste(BA_InternaliseString(game_info.sky_flat));
+			// abuse the delete function to turn sector ceilings into sky
+			CB_Paste(parts, BA_InternaliseString(Misc_info.sky_flat));
 			break;
 	}
 
@@ -944,9 +912,26 @@ bool UI_SectorBox::ClipboardOp(char op)
 
 void UI_SectorBox::BrowsedItem(char kind, int number, const char *name, int e_state)
 {
-	if (kind == 'F')
+	if (obj < 0)
+		return;
+
+	if (kind == 'F' || kind == 'T')
 	{
-		SetFlat(name, e_state);
+		int parts = GetSelectedPics();
+
+		if (parts == 0)
+		{
+			if (edit.render3d)
+				parts = PART_FLOOR | PART_CEIL;
+			else if (edit.sector_render_mode == SREND_Ceiling)
+				parts = PART_CEIL;
+			else if (e_state & FL_BUTTON3)
+				parts = PART_CEIL;
+			else
+				parts = PART_FLOOR;
+		}
+
+		SetFlat(name, parts);
 	}
 	else if (kind == 'S')
 	{
@@ -957,6 +942,9 @@ void UI_SectorBox::BrowsedItem(char kind, int number, const char *name, int e_st
 
 void UI_SectorBox::UnselectPics()
 {
+	f_pic->Unhighlight();
+	c_pic->Unhighlight();
+
 	f_pic->Selected(false);
 	c_pic->Selected(false);
 }
@@ -971,9 +959,9 @@ void UI_SectorBox::UpdateTotal()
 
 void UI_SectorBox::UpdateGameInfo()
 {
-	if (game_info.gen_sectors)
+	if (Features.gen_sectors)
 	{
-		if (game_info.gen_sectors == 2)
+		if (Features.gen_sectors == 2)
 			bm_title->label("ZDoom flags:");
 		else
 			bm_title->label("Boom flags:");
