@@ -162,8 +162,6 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 
 		pic->Selected(! pic->Selected());
 
-		Render3D_ClearSelection();
-
 		if (pic->Selected())
 			main_win->BrowserMode('T');
 		return;
@@ -190,14 +188,12 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 	}
 
 	// iterate over selected linedefs
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited texture on", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
 			const LineDef *L = LineDefs[*it];
 
@@ -227,7 +223,6 @@ void UI_SideBox::tex_callback(Fl_Widget *w, void *data)
 			}
 		}
 
-		BA_MessageForSel("edited texture on", &list);
 		BA_End();
 
 		box->UpdateField();
@@ -270,7 +265,6 @@ void UI_SideBox::add_callback(Fl_Widget *w, void *data)
 		return;
 
 	// iterate over selected linedefs
-	selection_iterator_c it;
 
 	int field = box->is_front ? LineDef::F_RIGHT : LineDef::F_LEFT;
 
@@ -284,8 +278,7 @@ void UI_SideBox::add_callback(Fl_Widget *w, void *data)
 		Sectors[new_sec]->SetDefaults();
 	}
 
-
-	for (edit.Selected->begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
@@ -333,11 +326,9 @@ void UI_SideBox::delete_callback(Fl_Widget *w, void *data)
 		return;
 
 	// iterate over selected linedefs
-	selection_iterator_c it;
-
 	BA_Begin();
 
-	for (edit.Selected->begin(&it) ; !it.at_end() ; ++it)
+	for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 	{
 		const LineDef *L = LineDefs[*it];
 
@@ -367,14 +358,16 @@ void UI_SideBox::offset_callback(Fl_Widget *w, void *data)
 	int new_y_ofs = atoi(box->y_ofs->value());
 
 	// iterate over selected linedefs
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		if (w == box->x_ofs)
+			BA_MessageForSel("edited X offset on", edit.Selected);
+		else
+			BA_MessageForSel("edited Y offset on", edit.Selected);
+
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			const LineDef *L = LineDefs[*it];
 
@@ -389,11 +382,6 @@ void UI_SideBox::offset_callback(Fl_Widget *w, void *data)
 			}
 		}
 
-		if (w == box->x_ofs)
-			BA_MessageForSel("edited X offset on", &list);
-		else
-			BA_MessageForSel("edited Y offset on", &list);
-
 		BA_End();
 	}
 }
@@ -408,14 +396,12 @@ void UI_SideBox::sector_callback(Fl_Widget *w, void *data)
 	new_sec = CLAMP(0, new_sec, NumSectors-1);
 
 	// iterate over selected linedefs
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited sector-ref on", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			const LineDef *L = LineDefs[*it];
 
@@ -425,7 +411,6 @@ void UI_SideBox::sector_callback(Fl_Widget *w, void *data)
 				BA_ChangeSD(sd, SideDef::F_SECTOR, new_sec);
 		}
 
-		BA_MessageForSel("edited sector-ref on", &list);
 		BA_End();
 	}
 }
@@ -490,6 +475,10 @@ void UI_SideBox::UpdateField()
 		if ((what_is_solid & SOLID_UPPER) && is_null_tex(upper))
 			u_pic->MarkMissing();
 
+		if (is_special_tex(lower)) l_pic->MarkSpecial();
+		if (is_special_tex(upper)) u_pic->MarkSpecial();
+		if (is_special_tex(rail))  r_pic->MarkSpecial();
+
 		l_pic->AllowHighlight(true);
 		u_pic->AllowHighlight(true);
 		r_pic->AllowHighlight(true);
@@ -525,7 +514,7 @@ void UI_SideBox::UpdateLabel()
 
 	char buffer[200];
 
-	sprintf(buffer, "   %s Sidedef: #%d\n",
+	snprintf(buffer, sizeof(buffer), "   %s Sidedef: #%d\n",
 			is_front ? "Front" : "Back", obj);
 
 	copy_label(buffer);
@@ -592,6 +581,12 @@ void UI_SideBox::UpdateHiding()
 
 			u_pic->hide();
 			r_pic->hide();
+
+			u_pic->Unhighlight();
+			r_pic->Unhighlight();
+
+			u_pic->Selected(false);
+			r_pic->Selected(false);
 		}
 	}
 }
@@ -601,23 +596,27 @@ int UI_SideBox::GetSelectedPics() const
 {
 	if (obj < 0) return 0;
 
-	return	(l_pic->Selected() ? 1 : 0) |
-			(u_pic->Selected() ? 2 : 0) |
-			(r_pic->Selected() ? 4 : 0);
+	return	(l_pic->Selected() ? PART_RT_LOWER : 0) |
+			(u_pic->Selected() ? PART_RT_UPPER : 0) |
+			(r_pic->Selected() ? PART_RT_RAIL  : 0);
 }
 
 int UI_SideBox::GetHighlightedPics() const
 {
 	if (obj < 0) return 0;
 
-	return	(l_pic->Highlighted() ? 1 : 0) |
-			(u_pic->Highlighted() ? 2 : 0) |
-			(r_pic->Highlighted() ? 4 : 0);
+	return	(l_pic->Highlighted() ? PART_RT_LOWER : 0) |
+			(u_pic->Highlighted() ? PART_RT_UPPER : 0) |
+			(r_pic->Highlighted() ? PART_RT_RAIL  : 0);
 }
 
 
 void UI_SideBox::UnselectPics()
 {
+	l_pic->Unhighlight();
+	u_pic->Unhighlight();
+	r_pic->Unhighlight();
+
 	l_pic->Selected(false);
 	u_pic->Selected(false);
 	r_pic->Selected(false);
