@@ -360,19 +360,16 @@ void UI_ThingBox::type_callback(Fl_Widget *w, void *data)
 	box->desc->value(info->desc);
 	box->sprite->GetSprite(new_type, FL_DARK2);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited type of", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
 			BA_ChangeTH(*it, Thing::F_TYPE, new_type);
 		}
 
-		BA_MessageForSel("edited type of", &list);
 		BA_End();
 	}
 }
@@ -407,19 +404,16 @@ void UI_ThingBox::spec_callback(Fl_Widget *w, void *data)
 	else
 		box->spec_desc->value(info->desc);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited special of", edit.Selected);
 
-		for (list.begin(&it) ; !it.at_end() ; ++it)
+		for (sel_iter_c it(edit.Selected) ; !it.done() ; it.next())
 		{
 			BA_ChangeTH(*it, Thing::F_SPECIAL, new_type);
 		}
 
-		BA_MessageForSel("edited special of", &list);
 		BA_End();
 	}
 }
@@ -429,7 +423,7 @@ void UI_ThingBox::dynspec_callback(Fl_Widget *w, void *data)
 {
 	UI_ThingBox *box = (UI_ThingBox *)data;
 
-	if (box->obj < 0 || Level_format != MAPF_Hexen)
+	if (box->obj < 0 || Level_format == MAPF_Doom)
 		return;
 
 	int value = atoi(box->spec_type->value());
@@ -452,8 +446,7 @@ void UI_ThingBox::SetThingType(int new_type)
 		return;
 
 	char buffer[64];
-
-	sprintf(buffer, "%d", new_type);
+	snprintf(buffer, sizeof(buffer), "%d", new_type);
 
 	type->value(buffer);
 	type->do_callback();
@@ -466,8 +459,7 @@ void UI_ThingBox::SetSpecialType(int new_type)
 		return;
 
 	char buffer[64];
-
-	sprintf(buffer, "%d", new_type);
+	snprintf(buffer, sizeof(buffer), "%d", new_type);
 
 	spec_type->value(buffer);
 	spec_type->do_callback();
@@ -486,7 +478,7 @@ void UI_ThingBox::BrowsedItem(char kind, int number, const char *name, int e_sta
 	{
 		SetThingType(number);
 	}
-	else if (kind == 'L' && Level_format == MAPF_Hexen)
+	else if (kind == 'L' && Level_format != MAPF_Doom)
 	{
 		SetSpecialType(number);
 	}
@@ -499,19 +491,16 @@ void UI_ThingBox::angle_callback(Fl_Widget *w, void *data)
 
 	int new_ang = atoi(box->angle->value());
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited angle of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			BA_ChangeTH(*it, Thing::F_ANGLE, new_ang);
 		}
 
-		BA_MessageForSel("edited angle of", &list);
 		BA_End();
 	}
 }
@@ -524,19 +513,16 @@ void UI_ThingBox::tid_callback(Fl_Widget *w, void *data)
 
 	int new_tid = atoi(box->tid->value());
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited TID of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			BA_ChangeTH(*it, Thing::F_TID, new_tid);
 		}
 
-		BA_MessageForSel("edited TID of", &list);
 		BA_End();
 	}
 }
@@ -548,17 +534,14 @@ void UI_ThingBox::x_callback(Fl_Widget *w, void *data)
 
 	int new_x = atoi(box->pos_x->value());
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited X of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
-			BA_ChangeTH(*it, Thing::F_X, new_x);
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
+			BA_ChangeTH(*it, Thing::F_X, MakeValidCoord(new_x));
 
-		BA_MessageForSel("edited X of", &list);
 		BA_End();
 	}
 }
@@ -569,17 +552,14 @@ void UI_ThingBox::y_callback(Fl_Widget *w, void *data)
 
 	int new_y = atoi(box->pos_y->value());
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited Y of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
-			BA_ChangeTH(*it, Thing::F_Y, new_y);
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
+			BA_ChangeTH(*it, Thing::F_Y, MakeValidCoord(new_y));
 
-		BA_MessageForSel("edited Y of", &list);
 		BA_End();
 	}
 }
@@ -588,19 +568,16 @@ void UI_ThingBox::z_callback(Fl_Widget *w, void *data)
 {
 	UI_ThingBox *box = (UI_ThingBox *)data;
 
-	int new_z = atoi(box->pos_z->value());
+	int new_h = atoi(box->pos_z->value());
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited Z of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
-			BA_ChangeTH(*it, Thing::F_Z, new_z);
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
+			BA_ChangeTH(*it, Thing::F_H, INT_TO_COORD(new_h));
 
-		BA_MessageForSel("edited Z of", &list);
 		BA_End();
 	}
 }
@@ -615,14 +592,12 @@ void UI_ThingBox::option_callback(Fl_Widget *w, void *data)
 	int mask = ocb->mask;
 	int new_opts = box->CalcOptions();
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited flags of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			const Thing *T = Things[*it];
 
@@ -631,7 +606,6 @@ void UI_ThingBox::option_callback(Fl_Widget *w, void *data)
 			BA_ChangeTH(*it, Thing::F_OPTIONS, (T->options & ~mask) | (new_opts & mask));
 		}
 
-		BA_MessageForSel("edited flags of", &list);
 		BA_End();
 	}
 }
@@ -658,9 +632,8 @@ void UI_ThingBox::button_callback(Fl_Widget *w, void *data)
 	{
 		if (w == box->ang_buts[i])
 		{
-			char buffer[20];
-
-			sprintf(buffer, "%d", i * 45);
+			char buffer[64];
+			snprintf(buffer, sizeof(buffer), "%d", i * 45);
 
 			box->angle->value(buffer);
 
@@ -681,19 +654,16 @@ void UI_ThingBox::args_callback(Fl_Widget *w, void *data)
 
 	new_value = CLAMP(0, new_value, 255);
 
-	selection_c list;
-	selection_iterator_c it;
-
-	if (GetCurrentObjects(&list))
+	if (! edit.Selected->empty())
 	{
 		BA_Begin();
+		BA_MessageForSel("edited args of", edit.Selected);
 
-		for (list.begin(&it); !it.at_end(); ++it)
+		for (sel_iter_c it(edit.Selected); !it.done(); it.next())
 		{
 			BA_ChangeTH(*it, Thing::F_ARG1 + arg_idx, new_value);
 		}
 
-		BA_MessageForSel("edited args of", &list);
 		BA_End();
 	}
 }
@@ -726,7 +696,7 @@ void UI_ThingBox::OptionsFromInt(int options)
 
 	o_ambush->value((options & MTF_Ambush) ? 1 : 0);
 
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		o_sp  ->value((options & MTF_Hexen_SP)   ? 1 : 0);
 		o_coop->value((options & MTF_Hexen_COOP) ? 1 : 0);
@@ -747,7 +717,7 @@ void UI_ThingBox::OptionsFromInt(int options)
 		o_vanilla_dm->value((options & MTF_Not_SP) ? 1 : 0);
 	}
 
-	if (Level_format != MAPF_Hexen)
+	if (Level_format == MAPF_Doom)
 	{
 		o_friend->value((options & MTF_Friend) ? 1 : 0);
 
@@ -757,7 +727,7 @@ void UI_ThingBox::OptionsFromInt(int options)
 			exfloor->value("");
 	}
 
-	if (game_info.strife_flags)
+	if (Features.strife_flags)
 	{
 		o_sf_ambush->value((options & MTF_Strife_Ambush) ? 1 : 0);
 		o_sf_friend->value((options & MTF_Strife_Friend) ? 1 : 0);
@@ -776,7 +746,7 @@ int UI_ThingBox::CalcOptions() const
 	if (o_medium->value()) options |= MTF_Medium;
 	if (o_hard  ->value()) options |= MTF_Hard;
 
-	if (game_info.strife_flags)
+	if (Features.strife_flags)
 	{
 		if (o_sf_ambush->value()) options |= MTF_Strife_Ambush;
 		if (o_sf_friend->value()) options |= MTF_Strife_Friend;
@@ -789,7 +759,7 @@ int UI_ThingBox::CalcOptions() const
 		if (o_ambush->value()) options |= MTF_Ambush;
 	}
 
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		if (o_sp  ->value()) options |= MTF_Hexen_SP;
 		if (o_coop->value()) options |= MTF_Hexen_COOP;
@@ -801,7 +771,7 @@ int UI_ThingBox::CalcOptions() const
 
 		if (o_dormant->value()) options |= MTF_Hexen_Dormant;
 	}
-	else if (game_info.coop_dm_flags)
+	else if (Features.coop_dm_flags)
 	{
 		if (0 == o_sp  ->value()) options |= MTF_Not_SP;
 		if (0 == o_coop->value()) options |= MTF_Not_COOP;
@@ -812,9 +782,9 @@ int UI_ThingBox::CalcOptions() const
 		if (o_vanilla_dm->value()) options |= MTF_Not_SP;
 	}
 
-	if (Level_format != MAPF_Hexen)
+	if (Level_format == MAPF_Doom)
 	{
-		if (game_info.friend_flag && o_friend->value())
+		if (Features.friend_flag && o_friend->value())
 			options |= MTF_Friend;
 
 #if 0
@@ -833,17 +803,19 @@ int UI_ThingBox::CalcOptions() const
 
 void UI_ThingBox::UpdateField(int field)
 {
-	if (field < 0 || field == Thing::F_X || field == Thing::F_Y
-		|| field == Thing::F_Z)
+	if (field < 0 ||
+		field == Thing::F_X ||
+		field == Thing::F_Y ||
+		field == Thing::F_H)
 	{
 		if (is_thing(obj))
 		{
 			const Thing *T = Things[obj];
 
-			pos_x->value(Int_TmpStr(T->x));
-			pos_y->value(Int_TmpStr(T->y));
-			pos_z->value(Int_TmpStr(T->z));
-
+			// @@ FIXME show decimals in UDMF
+			pos_x->value(Int_TmpStr(T->x()));
+			pos_y->value(Int_TmpStr(T->y()));
+			pos_z->value(Int_TmpStr(T->h()));
 		}
 		else
 		{
@@ -895,7 +867,7 @@ void UI_ThingBox::UpdateField(int field)
 			OptionsFromInt(0);
 	}
 
-	if (Level_format != MAPF_Hexen)
+	if (Level_format == MAPF_Doom)
 		return;
 
 	if (field < 0 || field == Thing::F_SPECIAL)
@@ -968,7 +940,7 @@ void UI_ThingBox::UpdateTotal()
 
 void UI_ThingBox::UpdateGameInfo()
 {
-	if (game_info.coop_dm_flags || Level_format == MAPF_Hexen)
+	if (Features.coop_dm_flags || Level_format != MAPF_Doom)
 	{
 		o_sp  ->show();
 		o_coop->show();
@@ -987,13 +959,13 @@ void UI_ThingBox::UpdateGameInfo()
 		o_dm  ->hide();
 	}
 
-	if (game_info.friend_flag && !game_info.strife_flags)
+	if (Features.friend_flag && !Features.strife_flags)
 		o_friend->show();
 	else
 		o_friend->hide();
 
 
-	if (game_info.strife_flags)
+	if (Features.strife_flags)
 	{
 		o_ambush->hide();
 
@@ -1019,7 +991,7 @@ void UI_ThingBox::UpdateGameInfo()
 
 	thing_opt_CB_data_c *ocb;
 
-	if (Level_format == MAPF_Hexen)
+	if (Level_format != MAPF_Doom)
 	{
 		pos_z->show();
 
@@ -1072,5 +1044,12 @@ void UI_ThingBox::UpdateGameInfo()
 }
 
 
+void UI_ThingBox::UnselectPics()
+{
+	sprite->Unhighlight();
+	sprite->Selected(false);
+}
+
+
 //--- editor settings ---
 // vi:ts=4:sw=4:noexpandtab
