@@ -30,35 +30,29 @@
 #include <unistd.h>
 #endif
 
-#if (FL_MAJOR_VERSION != 1 ||  \
-     FL_MINOR_VERSION != 3 ||  \
-     FL_PATCH_VERSION < 0)
+#if (FL_MAJOR_VERSION != 1 || FL_MINOR_VERSION < 3)
 #error "Require FLTK version 1.3.0 or later"
 #endif
 
 
 
-#define MAIN_WINDOW_W  (800 - 32)
-#define MAIN_WINDOW_H  (600 - 40)
+#define WINDOW_MIN_W  928
+#define WINDOW_MIN_H  640
 
 UI_MainWindow *main_win;
 
-// Kromulent Factor crud
-int KF;
-int KF_fonth;
-
 
 //
 // MainWin Constructor
 //
 UI_MainWindow::UI_MainWindow() :
-	Fl_Double_Window(MAIN_WINDOW_W + 64, MAIN_WINDOW_H, EUREKA_TITLE),
+	Fl_Double_Window(WINDOW_MIN_W, WINDOW_MIN_H, EUREKA_TITLE),
 	cursor_shape(FL_CURSOR_DEFAULT),
 	last_x(0), last_y(0), last_w(0), last_h(0)
 {
 	end(); // cancel begin() in Fl_Group constructor
 
-	size_range(MAIN_WINDOW_W, MAIN_WINDOW_H);
+	size_range(WINDOW_MIN_W, WINDOW_MIN_H);
 
 	callback((Fl_Callback *) quit_callback);
 
@@ -67,11 +61,11 @@ UI_MainWindow::UI_MainWindow() :
 	int cy = 0;
 	int ey = h();
 
-	panel_W = 276 + KF * 32;
+	panel_W = 308;
 
 	/* ---- Menu bar ---- */
 	{
-		menu_bar = Menu_Create(0, 0, w()-3 - panel_W, 28+KF*3);
+		menu_bar = Menu_Create(0, 0, w()-3 - panel_W, 31);
 		add(menu_bar);
 
 #ifndef __APPLE__
@@ -80,7 +74,7 @@ UI_MainWindow::UI_MainWindow() :
 	}
 
 
-	info_bar = new UI_InfoBar(0, ey - (28+KF*3), w(), 28+KF*3);
+	info_bar = new UI_InfoBar(0, ey - 31, w(), 31);
 	add(info_bar);
 
 	ey = ey - info_bar->h();
@@ -93,8 +87,9 @@ UI_MainWindow::UI_MainWindow() :
 
 	scroll = new UI_CanvasScroll(0, cy, cw, ch);
 
+	// UI_CanvasScroll creates these, we mirror them for easier access
 	canvas = scroll->canvas;
-	render = scroll->render;
+	status_bar = scroll->status;
 
 	browser = new UI_Browser(w() - panel_W - browser_W, cy, browser_W, ey - cy);
 
@@ -129,27 +124,6 @@ UI_MainWindow::UI_MainWindow() :
 	find_box = new UI_FindAndReplace(w() - panel_W, BY, panel_W, BH);
 	find_box->hide();
 	add(find_box);
-
-
-	// create bare-bone operation menu widgets
-	// [ they are fleshed out by M_LoadOperationMenus ]
-	op_thing  = new Fl_Menu_Button(0, 0, 99, 99, "Thing Operations");
-	op_line   = new Fl_Menu_Button(0, 0, 99, 99, "Line Operations");
-	op_sector = new Fl_Menu_Button(0, 0, 99, 99, "Sector Operations");
-	op_vertex = new Fl_Menu_Button(0, 0, 99, 99, "Vertex Operations");
-	op_render = new Fl_Menu_Button(0, 0, 99, 99, "3D Operations");
-
-	op_thing ->hide();
-	op_line  ->hide();
-	op_sector->hide();
-	op_vertex->hide();
-	op_render->hide();
-
-	add(op_thing);
-	add(op_line);
-	add(op_sector);
-	add(op_vertex);
-	add(op_render);
 }
 
 
@@ -221,9 +195,9 @@ void UI_MainWindow::BrowserMode(char kind)
 		else
 			tile->HideRight();
 
-		// hiding the browser also clears any pic selection
-		if (! want_visible)
-			UnselectPics();
+//??	// hiding the browser also clears any pic selection
+//??	if (! want_visible)
+//??		UnselectPics();
 	}
 
 	if (kind != 0 && kind != '/')
@@ -356,6 +330,7 @@ void UI_MainWindow::UnselectPics()
 {
 	 line_box->UnselectPics();
 	  sec_box->UnselectPics();
+	thing_box->UnselectPics();
 	props_box->UnselectPics();
 }
 
@@ -368,19 +343,14 @@ void UI_MainWindow::SetTitle(const char *wad_name, const char *map_name,
 	if (wad_name)
 	{
 		wad_name = fl_filename_name(wad_name);
-		sprintf(title_buf, "%s (%s)%s", wad_name, Level_name,
-				read_only ? " [Read-Only]" : "");
+		snprintf(title_buf, sizeof(title_buf), "%s (%s)%s",
+				wad_name, map_name, read_only ? " [Read-Only]" : "");
 	}
 	else
 	{
-		sprintf(title_buf, "Unsaved %s", Level_name);
+		snprintf(title_buf, sizeof(title_buf), "Unsaved %s", map_name);
 	}
 
-	// this just seems like visual noise, disabled for now
-#if 0
-	strcat(title_buf, " - Eureka");
-#endif
-
 	copy_label(title_buf);
 }
 
@@ -438,8 +408,6 @@ void UI_MainWindow::ToggleFullscreen()
 
 bool UI_MainWindow::ClipboardOp(char op)
 {
-	// Note : this is for the panels, we don't handle the 3D view here
-
 	if (props_box->visible())
 	{
 		return props_box->ClipboardOp(op);
@@ -470,12 +438,6 @@ void UI_MainWindow::BrowsedItem(char kind, int number, const char *name, int e_s
 {
 //	fprintf(stderr, "BrowsedItem: kind '%c' --> %d / \"%s\"\n", kind, number, name);
 
-	if (edit.render3d)
-	{
-		if (Render3D_BrowsedItem(kind, number, name, e_state))
-			return;	 // ate it
-	}
-
 	if (props_box->visible())
 	{
 		props_box->BrowsedItem(kind, number, name, e_state);
@@ -498,7 +460,7 @@ void UI_MainWindow::BrowsedItem(char kind, int number, const char *name, int e_s
 	}
 	else
 	{
-		fl_beep();
+		Beep("no target for browsed item");
 	}
 }
 
@@ -564,6 +526,28 @@ void UI_MainWindow::UpdateGameInfo()
 }
 
 
+// draw() and handle() are overridden here merely to prevent fatal
+// errors becoming infinite loops.  That can happen because we use
+// FLTK to show a dialog with the error message, and the same code
+// which triggered the fatal error can be re-entered.
+
+int UI_MainWindow::handle(int event)
+{
+	if (in_fatal_error)
+		return 0;
+
+	return Fl_Double_Window::handle(event);
+}
+
+void UI_MainWindow::draw()
+{
+	if (in_fatal_error)
+		return;
+
+	return Fl_Double_Window::draw();
+}
+
+
 //------------------------------------------------------------------------
 
 
@@ -835,7 +819,7 @@ void UI_LogViewer::save_callback(Fl_Widget *w, void *data)
 
 	if (! fp)
 	{
-		sprintf(filename, "%s", strerror(errno));
+		snprintf(filename, sizeof(filename), "%s", strerror(errno));
 
 		DLG_Notify("Unable to save the log file:\n\n%s", filename);
 		return;
