@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2016 Andrew Apted
+//  Copyright (C) 2001-2020 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -29,6 +29,8 @@
 #include "im_color.h"  /* trans_replace */
 #include "im_img.h"
 
+#include "lib_tga.h"
+
 #include "w_loadpic.h"
 #include "w_rawdef.h"
 #include "w_wad.h"
@@ -95,6 +97,113 @@ static void DrawColumn(Img_c& img, const post_t *column, int x, int y)
 }
 
 
+Img_c * LoadImage_PNG(Lump_c *lump, const char *name)
+{
+	// load the raw data
+	byte *tex_data;
+	int tex_length = W_LoadLumpData(lump, &tex_data);
+
+	// pass it to FLTK for decoding
+	Fl_PNG_Image fltk_img(NULL, tex_data, tex_length);
+
+	W_FreeLumpData(&tex_data);
+
+	if (fltk_img.w() <= 0)
+	{
+		// failed to decode
+		LogPrintf("Failed to decode PNG image in '%s' lump.\n", name);
+		return NULL;
+	}
+
+	// convert it
+	Img_c *img = IM_ConvertRGBImage(&fltk_img);
+
+	return img;
+}
+
+
+Img_c * LoadImage_JPEG(Lump_c *lump, const char *name)
+{
+	// load the raw data
+	byte *tex_data;
+	int tex_length = W_LoadLumpData(lump, &tex_data);
+
+	(void) tex_length;
+
+	// pass it to FLTK for decoding
+	Fl_JPEG_Image fltk_img(NULL, tex_data);
+
+	W_FreeLumpData(&tex_data);
+
+	if (fltk_img.w() <= 0)
+	{
+		// failed to decode
+		LogPrintf("Failed to decode JPEG image in '%s' lump.\n", name);
+		return NULL;
+	}
+
+	// convert it
+	Img_c *img = IM_ConvertRGBImage(&fltk_img);
+
+	return img;
+}
+
+
+Img_c * LoadImage_TGA(Lump_c *lump, const char *name)
+{
+	// load the raw data
+	byte *tex_data;
+	int tex_length = W_LoadLumpData(lump, &tex_data);
+
+	// decode it
+	int width;
+	int height;
+
+	rgba_color_t * rgba = TGA_DecodeImage(tex_data, (size_t)tex_length, width, height);
+
+	W_FreeLumpData(&tex_data);
+
+	if (! rgba)
+	{
+		// failed to decode
+		LogPrintf("Failed to decode TGA image in '%s' lump.\n", name);
+		return NULL;
+	}
+
+	// convert it
+	Img_c *img = IM_ConvertTGAImage(rgba, width, height);
+
+	TGA_FreeImage(rgba);
+
+	return img;
+}
+
+
+static bool ComposePicture(Img_c& dest, Img_c *sub,
+	int pic_x_offset, int pic_y_offset,
+	int *pic_width, int *pic_height)
+{
+	if (sub == NULL)
+		return false;
+
+	int width  = sub->width();
+	int height = sub->height();
+
+	if (pic_width)  *pic_width  = width;
+	if (pic_height) *pic_height = height;
+
+	if (dest.is_null())
+	{
+		// our new image will be completely transparent
+		dest.resize (width, height);
+	}
+
+	dest.compose(sub, pic_x_offset, pic_y_offset);
+
+	return true;
+}
+
+
 //
 //  LoadPicture - read a picture from a wad file into an Img_c object
 //
@@ -110,10 +219,7 @@ static void DrawColumn(Img_c& img, const post_t *column, int x, int y)
 //
 //  Return true on success, false on failure.
 //
-//  If pic_x_offset == INT_MIN, the picture is centred horizontally.
-//  If pic_y_offset == INT_MIN, the picture is centred vertically.
-//
-bool LoadPicture(Img_c& img,      // image to load picture into
+bool LoadPicture(Img_c& dest,      // image to load picture into
 	Lump_c *lump,
 	const char *pic_name,   // picture name (for messages)
 	int pic_x_offset,    // coordinates of top left corner of picture
@@ -121,6 +227,38 @@ bool LoadPicture(Img_c& img,      // image to load picture into
 	int *pic_width,    // To return the size of the picture
 	int *pic_height)   // (can be NULL)
 {
+	char img_fmt = W_DetectImageFormat(lump);
+	Img_c *sub;
+
+	switch (img_fmt)
+	{
+	case 'd':
+		// use the code below to load/compose the DOOM format
+		break;
+
+	case 'p':
+		sub = LoadImage_PNG(lump, pic_name);
+		return ComposePicture(dest, sub, pic_x_offset, pic_y_offset, pic_width, pic_height);
+
+	case 'j':
+		sub = LoadImage_JPEG(lump, pic_name);
+		return ComposePicture(dest, sub, pic_x_offset, pic_y_offset, pic_width, pic_height);
+
+	case 't':
+		sub = LoadImage_TGA(lump, pic_name);
+		return ComposePicture(dest, sub, pic_x_offset, pic_y_offset, pic_width, pic_height);
+
+	case 0:
+		LogPrintf("Unknown image format in '%s' lump\n", pic_name);
+		return false;
+
+	default:
+		LogPrintf("Unsupported image format in '%s' lump\n", pic_name);
+		return false;
+	}
+
+	/* DOOM format */
+
 	byte *raw_data;
 	W_LoadLumpData(lump, &raw_data);
 
@@ -136,18 +274,12 @@ bool LoadPicture(Img_c& img,      // image to load picture into
 	if (pic_width)  *pic_width  = width;
 	if (pic_height) *pic_height = height;
 
-	if (img.is_null())
+	if (dest.is_null())
 	{
 		// our new image will be completely transparent
-		img.resize (width, height);
+		dest.resize (width, height);
 	}
 
-	// Centre the picture?
-	if (pic_x_offset == INT_MIN)
-		pic_x_offset = (img.width() - width) / 2;
-	if (pic_y_offset == INT_MIN)
-		pic_y_offset = (img.height() - height) / 2;
-
 	for (int x = 0 ; x < width ; x++)
 	{
 		int offset = LE_S32(pat->columnofs[x]);
@@ -161,7 +293,7 @@ bool LoadPicture(Img_c& img,      // image to load picture into
 
 		const post_t *column = (const post_t *) ((const byte *)pat + offset);
 
-		DrawColumn(img, column, pic_x_offset + x, pic_y_offset);
+		DrawColumn(dest, column, pic_x_offset + x, pic_y_offset);
 	}
 
 	W_FreeLumpData(&raw_data);
@@ -186,40 +318,35 @@ char W_DetectImageFormat(Lump_c *lump)
 
 	// PNG is clearly marked in the header, so check it first.
 
-	if (header[0] == 0x89 &&
-		header[1] == 'P'  &&
-		header[2] == 'N'  &&
-		header[3] == 'G')
+	if (header[0] == 0x89 && header[1] == 'P' &&
+		header[2] == 'N'  && header[3] == 'G' &&
+		header[4] == 0x0D && header[5] == 0x0A)
 	{
 		return 'p'; /* PNG */
 	}
 
 	// check some other common image formats....
 
-	if (header[0] == 0xFF &&
-		header[1] == 0xD8)
+	if (header[0] == 0xFF && header[1] == 0xD8 &&
+		header[2] == 0xFF && header[3] >= 0xE0 &&
+		((header[6] == 'J' && header[7] == 'F') ||
+		 (header[6] == 'E' && header[7] == 'x')))
 	{
 		return 'j'; /* JPEG */
 	}
 
-	if (header[0] == 'G' &&
-		header[1] == 'I' &&
-		header[2] == 'F' &&
-		header[3] == '8')
+	if (header[0] == 'G' && header[1] == 'I' &&
+		header[2] == 'F' && header[3] == '8' &&
+		header[4] >= '7' && header[4] <= '9' &&
+		header[5] == 'a')
 	{
 		return 'g'; /* GIF */
 	}
 
-	if (header[0] == 'B' &&
-		header[1] == 'M')
-	{
-		return 'b'; /* BMP */
-	}
-
-	if (header[0] == 'D' &&
-		header[1] == 'D' &&
-		header[2] == 'S' &&
-		header[3] == 0x20)
+	if (header[0] == 'D' && header[1] == 'D'  &&
+		header[2] == 'S' && header[3] == 0x20 &&
+		header[4] == 124 && header[5] == 0    &&
+		header[6] == 0)
 	{
 		return 's'; /* DDS (DirectDraw Surface) */
 	}
@@ -248,8 +375,8 @@ char W_DetectImageFormat(Lump_c *lump)
 	 width = (int)header[0] + (int)(header[1] << 8);
 	height = (int)header[2] + (int)(header[3] << 8);
 
-	int ofs_x = (int)header[4] + (int)(header[5] << 8);
-	int ofs_y = (int)header[6] + (int)(header[7] << 8);
+	int ofs_x = (int)header[4] + (int)((signed char)header[5] * 256);
+	int ofs_y = (int)header[6] + (int)((signed char)header[7] * 256);
 
 	if (width  > 0 && width  <= 2048 && abs(ofs_x) <= 2048 &&
 		height > 0 && height <=  512 && abs(ofs_y) <=  512 &&
