@@ -31,7 +31,7 @@
 #include "w_wad.h"
 
 
-bool LoadPicture(Img_c& img,
+bool LoadPicture(Img_c& dest,
 	Lump_c *lump,           /* Lump containing picture */
 	const char *pic_name,   /* Picture name, for messages */
 	int pic_x_offset,    /* Coordinates of top left corner of picture */
@@ -40,6 +40,11 @@ bool LoadPicture(Img_c& img,
 	int *pic_height = NULL);  /* (can be NULL) */
 
 
+Img_c * LoadImage_PNG(Lump_c *lump, const char *name);
+Img_c * LoadImage_JPEG(Lump_c *lump, const char *name);
+Img_c * LoadImage_TGA(Lump_c *lump, const char *name);
+
+
 // Determine the image format of the given wad lump.
 //
 // Return values are:
@@ -49,7 +54,6 @@ bool LoadPicture(Img_c& img,
 //
 //    'j'  : JPEG
 //    'g'  : GIF
-//    'b'  : BMP
 //    's'  : DDS
 //
 //    NUL  : unrecognized
