@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2018 Andrew Apted
+//  Copyright (C) 2001-2020 Andrew Apted
 //  Copyright (C) 1997-2003 Andr Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -30,8 +30,6 @@
 #include <algorithm>
 #include <string>
 
-#include "lib_tga.h"
-
 #include "m_game.h"      /* yg_picture_format */
 #include "w_loadpic.h"
 #include "w_rawdef.h"
@@ -44,6 +42,7 @@
 
 
 std::map<std::string, Img_c *> textures;
+std::map<std::string, Img_c *> flats;
 
 // textures which can cause the Medusa Effect in vanilla/chocolate DOOM
 static std::map<std::string, int> medusa_textures;
@@ -167,7 +166,7 @@ static void LoadTextureEntry_Strife(byte *tex_data, int tex_length, int offset,
 		Lump_c *lump = W_FindPatchLump(picname);
 
 		if (! lump ||
-			! LoadPicture(*img, lump, picname, xofs, yofs, 0, 0))
+			! LoadPicture(*img, lump, picname, xofs, yofs))
 		{
 			LogPrintf("texture '%.8s': patch '%.8s' not found.\n", raw->name, picname);
 		}
@@ -243,7 +242,7 @@ static void LoadTextureEntry_DOOM(byte *tex_data, int tex_length, int offset,
 		Lump_c *lump = W_FindPatchLump(picname);
 
 		if (! lump ||
-			! LoadPicture(*img, lump, picname, xofs, yofs, 0, 0))
+			! LoadPicture(*img, lump, picname, xofs, yofs))
 		{
 			LogPrintf("texture '%.8s': patch '%.8s' not found.\n", raw->name, picname);
 		}
@@ -310,103 +309,6 @@ static void LoadTexturesLump(Lump_c *lump, byte *pnames, int pname_size,
 }
 
 
-static void LoadTexture_SinglePatch(const char *name, Lump_c *lump)
-{
-	Img_c *img = new Img_c();
-
-	if (! LoadPicture(*img, lump, name, 0, 0))
-	{
-		delete img;
-
-		return;
-	}
-
-	W_AddTexture(name, img, false /* is_medusa */);
-}
-
-
-static void LoadTexture_PNG(const char *name, Lump_c *lump, char img_fmt)
-{
-	// load the raw data
-	byte *tex_data;
-	int tex_length = W_LoadLumpData(lump, &tex_data);
-
-	// pass it to FLTK for decoding
-	Fl_PNG_Image fltk_img(NULL, tex_data, tex_length);
-
-	W_FreeLumpData(&tex_data);
-
-	if (fltk_img.w() <= 0)
-	{
-		// failed to decode
-		LogPrintf("Failed to decode PNG image in '%s' lump.\n", name);
-		return;
-	}
-
-	// convert it
-	Img_c *img = IM_ConvertRGBImage(&fltk_img);
-
-	W_AddTexture(name, img, false /* is_medusa */);
-}
-
-
-static void LoadTexture_JPEG(const char *name, Lump_c *lump, char img_fmt)
-{
-	// load the raw data
-	byte *tex_data;
-	int tex_length = W_LoadLumpData(lump, &tex_data);
-
-	(void) tex_length;
-
-	// pass it to FLTK for decoding
-	Fl_JPEG_Image fltk_img(NULL, tex_data);
-
-	W_FreeLumpData(&tex_data);
-
-	if (fltk_img.w() <= 0)
-	{
-		// failed to decode
-		LogPrintf("Failed to decode JPEG image in '%s' lump.\n", name);
-		return;
-	}
-
-	// convert it
-	Img_c *img = IM_ConvertRGBImage(&fltk_img);
-
-	W_AddTexture(name, img, false /* is_medusa */);
-}
-
-
-static void LoadTexture_TGA(const char *name, Lump_c *lump, char img_fmt)
-{
-	// load the raw data
-	byte *tex_data;
-	int tex_length = W_LoadLumpData(lump, &tex_data);
-
-	// decode it
-	int width;
-	int height;
-
-	rgba_color_t * rgba = TGA_DecodeImage(tex_data, (size_t)tex_length, width, height);
-
-	W_FreeLumpData(&tex_data);
-
-	if (! rgba)
-	{
-		// failed to decode
-		LogPrintf("Failed to decode TGA image in '%s' lump.\n", name);
-		return;
-	}
-
-	// convert it
-	Img_c *img = IM_ConvertTGAImage(rgba, width, height);
-
-	W_AddTexture(name, img, false /* is_medusa */);
-
-	TGA_FreeImage(rgba);
-}
-
-
 void W_LoadTextures_TX_START(Wad_file *wf)
 {
 	for (int k = 0 ; k < (int)wf->tx_tex.size() ; k++)
@@ -414,35 +316,48 @@ void W_LoadTextures_TX_START(Wad_file *wf)
 		Lump_c *lump = wf->GetLump(wf->tx_tex[k]);
 
 		char img_fmt = W_DetectImageFormat(lump);
+		const char *name = lump->Name();
+		Img_c *img = NULL;
 
-		DebugPrintf("TX_TEX %d : '%s' type=%c\n", k, lump->Name(), img_fmt ? img_fmt : '?');
+		DebugPrintf("TX_TEX %d : '%s' type=%c\n", k, name, img_fmt ? img_fmt : '?');
 
 		switch (img_fmt)
 		{
 			case 'd': /* Doom patch */
-				LoadTexture_SinglePatch(lump->Name(), lump);
+				img = new Img_c();
+				if (! LoadPicture(*img, lump, name, 0, 0))
+				{
+					delete img;
+					img = NULL;
+				}
 				break;
 
 			case 'p': /* PNG */
-				LoadTexture_PNG(lump->Name(), lump, img_fmt);
+				img = LoadImage_PNG(lump, name);
 				break;
 
 			case 't': /* TGA */
-				LoadTexture_TGA(lump->Name(), lump, img_fmt);
+				img = LoadImage_TGA(lump, name);
 				break;
 
 			case 'j': /* JPEG */
-				LoadTexture_JPEG(lump->Name(), lump, img_fmt);
+				img = LoadImage_JPEG(lump, name);
 				break;
 
 			case 0:
-				LogPrintf("Unknown texture format in '%s' lump\n", lump->Name());
+				LogPrintf("Unknown texture format in '%s' lump\n", name);
 				break;
 
 			default:
 				LogPrintf("Unsupported texture format in '%s' lump\n", lump->Name());
 				break;
 		}
+
+		// if we successfully loaded the texture, add it
+		if (img)
+		{
+			W_AddTexture(name, img, false /* is_medusa */);
+		}
 	}
 }
 
@@ -480,7 +395,7 @@ void W_LoadTextures()
 			W_FreeLumpData(&pname_data);
 		}
 
-		if (game_info.tx_start)
+		if (Features.tx_start)
 		{
 			W_LoadTextures_TX_START(master_dir[i]);
 		}
@@ -497,7 +412,6 @@ Img_c * W_GetTexture(const char *name, bool try_uppercase)
 		return NULL;
 
 	std::string t_str = name;
-
 	std::map<std::string, Img_c *>::iterator P = textures.find(t_str);
 
 	if (P != textures.end())
@@ -508,6 +422,14 @@ Img_c * W_GetTexture(const char *name, bool try_uppercase)
 		return W_GetTexture(NormalizeTex(name), false);
 	}
 
+	if (Features.mix_textures_flats)
+	{
+		std::map<std::string, Img_c *>::iterator P = flats.find(t_str);
+
+		if (P != flats.end())
+			return P->second;
+	}
+
 	return NULL;
 }
 
@@ -532,10 +454,20 @@ bool W_TextureIsKnown(const char *name)
 		return false;
 
 	std::string t_str = name;
-
 	std::map<std::string, Img_c *>::iterator P = textures.find(t_str);
 
-	return (P != textures.end());
+	if (P != textures.end())
+		return true;
+
+	if (Features.mix_textures_flats)
+	{
+		std::map<std::string, Img_c *>::iterator P = flats.find(t_str);
+
+		if (P != flats.end())
+			return true;
+	}
+
+	return false;
 }
 
 
@@ -558,10 +490,16 @@ const char *NormalizeTex(const char *name)
 
 	memset(buffer, 0, sizeof(buffer));
 
-	strncpy(buffer, name, WAD_TEX_NAME);
+	for (size_t i = 0 ; i < WAD_TEX_NAME && name[i]; i++)
+	{
+		buffer[i] = name[i];
 
-	for (int i = 0 ; i < WAD_TEX_NAME ; i++)
-		buffer[i] = toupper(buffer[i]);
+		// remove double quotes
+		if (buffer[i] == '"')
+			buffer[i] = '_';
+		else
+			buffer[i] = toupper(buffer[i]);
+	}
 
 	return buffer;
 }
@@ -571,10 +509,6 @@ const char *NormalizeTex(const char *name)
 //    FLAT HANDLING
 //----------------------------------------------------------------------
 
-
-std::map<std::string, Img_c *> flats;
-
-
 static void DeleteFlat(const std::map<std::string, Img_c *>::value_type& P)
 {
 	delete P.second;
@@ -667,12 +601,19 @@ void W_LoadFlats()
 Img_c * W_GetFlat(const char *name, bool try_uppercase)
 {
 	std::string f_str = name;
-
 	std::map<std::string, Img_c *>::iterator P = flats.find(f_str);
 
 	if (P != flats.end())
 		return P->second;
 
+	if (Features.mix_textures_flats)
+	{
+		std::map<std::string, Img_c *>::iterator P = textures.find(f_str);
+
+		if (P != textures.end())
+			return P->second;
+	}
+
 	if (try_uppercase)
 	{
 		return W_GetFlat(NormalizeTex(name), false);
@@ -692,10 +633,20 @@ bool W_FlatIsKnown(const char *name)
 		return false;
 
 	std::string f_str = name;
-
 	std::map<std::string, Img_c *>::iterator P = flats.find(f_str);
 
-	return (P != flats.end());
+	if (P != flats.end())
+		return true;
+
+	if (Features.mix_textures_flats)
+	{
+		std::map<std::string, Img_c *>::iterator P = textures.find(f_str);
+
+		if (P != textures.end())
+			return true;
+	}
+
+	return false;
 }
 
 
@@ -712,6 +663,7 @@ static sprite_map_t sprites;
 
 static void DeleteSprite(const sprite_map_t::value_type& P)
 {
+	// Note that P.second can be NULL here
 	delete P.second;
 }
 
@@ -758,7 +710,7 @@ Lump_c * Sprite_loc_by_root (const char *name)
 
 	// check outside of the sprite namespace...
 
-	if (! game_info.lax_sprites)
+	if (! Features.lax_sprites)
 		return NULL;
 
 	strcpy(buffer, name);
@@ -800,23 +752,35 @@ Img_c * W_GetSprite(int type)
 
 	Img_c *result = NULL;
 
-	if (y_stricmp(info->sprite, "_LYT") == 0)
+	if (strncmp(info->desc, "UNKNOWN", 7) == 0)
+	{
+		// leave as NULL
+	}
+	else if (y_stricmp(info->sprite, "_LYT") == 0)
 	{
 		result = IM_CreateLightSprite();
 	}
-	else if (y_stricmp(info->sprite, "NULL") != 0)
+	else if (y_stricmp(info->sprite, "_MSP") == 0)
+	{
+		result = IM_CreateMapSpotSprite(0, 255, 0);
+	}
+	else if (y_stricmp(info->sprite, "NULL") == 0)
+	{
+		result = IM_CreateMapSpotSprite(70, 70, 255);
+	}
+	else
 	{
 		Lump_c *lump = Sprite_loc_by_root(info->sprite);
 		if (! lump)
 		{
-			LogPrintf("Sprite not found: '%s'\n", info->sprite);
-
 			// for the MBF dog, create our own sprite for it, since
 			// it is defined in the Boom definition file and the
 			// missing sprite looks ugly in the thing browser.
 
 			if (y_stricmp(info->sprite, "DOGS") == 0)
 				result = IM_CreateDogSprite();
+			else
+				LogPrintf("Sprite not found: '%s'\n", info->sprite);
 		}
 		else
 		{
@@ -878,5 +842,34 @@ Img_c * W_GetSprite(int type)
 }
 
 
+//----------------------------------------------------------------------
+
+static void UnloadTex(const std::map<std::string, Img_c *>::value_type& P)
+{
+	if (P.second != NULL)
+		P.second->unload_gl(false);
+}
+
+static void UnloadFlat(const std::map<std::string, Img_c *>::value_type& P)
+{
+	if (P.second != NULL)
+		P.second->unload_gl(false);
+}
+
+static void UnloadSprite(const sprite_map_t::value_type& P)
+{
+	if (P.second != NULL)
+		P.second->unload_gl(false);
+}
+
+void W_UnloadAllTextures()
+{
+	std::for_each(textures.begin(), textures.end(), UnloadTex);
+	std::for_each(   flats.begin(),    flats.end(), UnloadFlat);
+	std::for_each( sprites.begin(),  sprites.end(), UnloadSprite);
+
+	IM_UnloadDummyTextures();
+}
+
 //--- editor settings ---
 // vi:ts=4:sw=4:noexpandtab
