@@ -4,7 +4,7 @@
 //
 //  Eureka DOOM Editor
 //
-//  Copyright (C) 2001-2018 Andrew Apted
+//  Copyright (C) 2001-2019 Andrew Apted
 //  Copyright (C) 1997-2003 André Majorel et al
 //
 //  This program is free software; you can redistribute it and/or
@@ -38,6 +38,9 @@ Wad_file * edit_wad;
 
 std::vector<Wad_file *> master_dir;
 
+// UDMF support is unfinished and hence disabled by default.
+bool udmf_testing = false;
+
 
 #define MAX_LUMPS_IN_A_LEVEL	21
 
@@ -86,7 +89,7 @@ Lump_c::~Lump_c()
 
 void Lump_c::MakeEntry(struct raw_wad_entry_s *entry)
 {
-	strncpy(entry->name, name, 8);
+	W_StoreString(entry->name, name, sizeof(entry->name));
 
 	entry->pos  = LE_U32(l_start);
 	entry->size = LE_U32(l_length);
@@ -267,7 +270,13 @@ retry:
 	if (w->total_size < 0)
 		FatalError("Error determining WAD size.\n");
 
-	w->ReadDirectory();
+	if (! w->ReadDirectory())
+	{
+		delete w;
+		LogPrintf("Open wad failed (reading directory)\n");
+		return NULL;
+	}
+
 	w->DetectLevels();
 	w->ProcessNamespaces();
 
@@ -394,19 +403,12 @@ short Wad_file::LevelLookupLump(short lev_num, const char *name)
 	short start = LevelHeader(lev_num);
 
 	// determine how far past the level marker (MAP01 etc) to search
-	short last = start + MAX_LUMPS_IN_A_LEVEL;
+	short finish = LevelLastLump(lev_num);
 
-	if (last >= NumLumps())
-		last = NumLumps() - 1;
-
-	for (short k = start+1 ; k <= last ; k++)
+	for (short k = start+1 ; k <= finish ; k++)
 	{
 		SYS_ASSERT(0 <= k && k < NumLumps());
 
-		if (! IsLevelLump(directory[k]->name) &&
-			! IsGLNodeLump(directory[k]->name))
-			break;
-
 		if (y_stricmp(directory[k]->name, name) == 0)
 			return k;
 	}
@@ -438,6 +440,24 @@ short Wad_file::LevelLastLump(short lev_num)
 
 	int count = 1;
 
+	// UDMF level?
+	if (y_stricmp(directory[start+1]->name, "TEXTMAP") == 0)
+	{
+		while (count < MAX_LUMPS_IN_A_LEVEL && start+count < NumLumps())
+		{
+			if (y_stricmp(directory[start+count]->name, "ENDMAP") == 0)
+			{
+				count++;
+				break;
+			}
+
+			count++;
+		}
+
+		return start + count - 1;
+	}
+
+	// standard DOOM or HEXEN format
 	while (count < MAX_LUMPS_IN_A_LEVEL &&
 		   start+count < NumLumps() &&
 		   (IsLevelLump(directory[start+count]->name) ||
@@ -456,18 +476,18 @@ short Wad_file::LevelFindByNumber(int number)
 	if (number <= 0 || number > 99)
 		return -1;
 
-	char buffer[10];
+	char buffer[16];
 	short index;
 
 	 // try MAP## first
-	sprintf(buffer, "MAP%02d", number);
+	snprintf(buffer, sizeof(buffer), "MAP%02d", number);
 
 	index = LevelFind(buffer);
 	if (index >= 0)
 		return index;
 
 	// otherwise try E#M#
-	sprintf(buffer, "E%dM%d", MAX(1, number / 10), number % 10);
+	snprintf(buffer, sizeof(buffer), "E%dM%d", MAX(1, number / 10), number % 10);
 
 	index = LevelFind(buffer);
 	if (index >= 0)
@@ -498,6 +518,9 @@ map_format_e Wad_file::LevelFormat(short lev_num)
 {
 	int start = LevelHeader(lev_num);
 
+	if (y_stricmp(directory[start+1]->name, "TEXTMAP") == 0)
+		return MAPF_UDMF;
+
 	if (start + LL_BEHAVIOR < (int)NumLumps())
 	{
 		const char *name = GetLump(start + LL_BEHAVIOR)->Name();
@@ -542,18 +565,17 @@ Lump_c * Wad_file::FindLumpInNamespace(const char *name, char group)
 }
 
 
-void Wad_file::ReadDirectory()
+bool Wad_file::ReadDirectory()
 {
-	// TODO: no fatal errors
-
 	rewind(fp);
 
 	raw_wad_header_t header;
 
 	if (fread(&header, sizeof(header), 1, fp) != 1)
-		FatalError("Error reading WAD header.\n");
-
-	// TODO: check ident for PWAD or IWAD
+	{
+		LogPrintf("Error reading WAD header.\n");
+		return false;
+	}
 
 	kind = header.ident[0];
 
@@ -561,19 +583,28 @@ void Wad_file::ReadDirectory()
 	dir_count = LE_S32(header.num_entries);
 
 	if (dir_count < 0 || dir_count > 32000)
-		FatalError("Bad WAD header, too many entries (%d)\n", dir_count);
+	{
+		LogPrintf("Bad WAD header, too many entries (%d)\n", dir_count);
+		return false;
+	}
 
 	crc32_c checksum;
 
 	if (fseek(fp, dir_start, SEEK_SET) != 0)
-		FatalError("Error seeking to WAD directory.\n");
+	{
+		LogPrintf("Error seeking to WAD directory.\n");
+		return false;
+	}
 
 	for (short i = 0 ; i < dir_count ; i++)
 	{
 		raw_wad_entry_t entry;
 
 		if (fread(&entry, sizeof(entry), 1, fp) != 1)
-			FatalError("Error reading WAD directory.\n");
+		{
+			LogPrintf("Error reading entry in WAD directory.\n");
+			return false;
+		}
 
 		// update the checksum with each _RAW_ entry
 		checksum.AddBlock((u8_t *) &entry, sizeof(entry));
@@ -605,6 +636,7 @@ void Wad_file::ReadDirectory()
 	dir_crc = checksum.raw;
 
 	DebugPrintf("Loaded directory. crc = %08x\n", dir_crc);
+	return true;
 }
 
 
@@ -615,14 +647,25 @@ void Wad_file::DetectLevels()
 	// vector.  The test here is rather lax, as I'm told certain
 	// wads exist with a non-standard ordering of level lumps.
 
-	for (short k = 0 ; k+5 < NumLumps() ; k++)
+	for (short k = 0 ; k+1 < NumLumps() ; k++)
 	{
 		int part_mask  = 0;
 		int part_count = 0;
 
+		// check for UDMF levels
+		if (udmf_testing && y_stricmp(directory[k+1]->name, "TEXTMAP") == 0)
+		{
+			levels.push_back(k);
+			DebugPrintf("Detected level : %s (UDMF)\n", directory[k]->name);
+			continue;
+		}
+
 		// check whether the next four lumps are level lumps
 		for (short i = 1 ; i <= 4 ; i++)
 		{
+			if (k+i >= NumLumps())
+				break;
+
 			int part = WhatLevelPart(directory[k+i]->name);
 
 			if (part == 0)
@@ -907,22 +950,12 @@ void Wad_file::RemoveLevel(short lev_num)
 	SYS_ASSERT(begun_write);
 	SYS_ASSERT(0 <= lev_num && lev_num < LevelCount());
 
-	short start = LevelHeader(lev_num);
-	short count = 1;
-
-	// collect associated lumps (THINGS, VERTEXES etc)
-	// this will stop when it hits a non-level lump
-	while (count < MAX_LUMPS_IN_A_LEVEL &&
-		   start + count < NumLumps() &&
-		   (IsLevelLump(directory[start+count]->name) ||
-		    IsGLNodeLump(directory[start+count]->name)) )
-	{
-		count++;
-	}
+	short start  = LevelHeader(lev_num);
+	short finish = LevelLastLump(lev_num);
 
 	// NOTE: FixGroup() will remove the entry in levels[]
 
-	RemoveLumps(start, count);
+	RemoveLumps(start, finish - start + 1);
 }
 
 
@@ -932,10 +965,7 @@ void Wad_file::RemoveGLNodes(short lev_num)
 	SYS_ASSERT(0 <= lev_num && lev_num < LevelCount());
 
 	short start  = LevelHeader(lev_num);
-	short finish = start + MAX_LUMPS_IN_A_LEVEL - 1;
-
-	if (finish >= NumLumps())
-		finish = NumLumps() - 1;
+	short finish = LevelLastLump(lev_num);
 
 	start++;
 
@@ -958,6 +988,25 @@ void Wad_file::RemoveGLNodes(short lev_num)
 }
 
 
+void Wad_file::RemoveZNodes(short lev_num)
+{
+	SYS_ASSERT(begun_write);
+	SYS_ASSERT(0 <= lev_num && lev_num < LevelCount());
+
+	short start  = LevelHeader(lev_num);
+	short finish = LevelLastLump(lev_num);
+
+	for ( ; start <= finish ; start++)
+	{
+		if (y_stricmp(directory[start]->name, "ZNODES") == 0)
+		{
+			RemoveLumps(start, 1);
+			break;
+		}
+	}
+}
+
+
 void Wad_file::FixGroup(std::vector<short>& group, short index,
                         short num_added, short num_removed)
 {
@@ -1271,7 +1320,7 @@ void Wad_file::WriteDirectory()
 
 	raw_wad_header_t header;
 
-	strncpy(header.ident, (kind == 'I') ? "IWAD" : "PWAD", 4);
+	memcpy(header.ident, (kind == 'I') ? "IWAD" : "PWAD", 4);
 
 	header.dir_start   = LE_U32(dir_start);
 	header.num_entries = LE_U32(dir_count);
@@ -1417,6 +1466,15 @@ int W_FilenameAbsCompare(const char *A, const char *B)
 }
 
 
+void W_StoreString(char *buf, const char *str, size_t buflen)
+{
+	memset(buf, 0, buflen);
+
+	for (size_t i = 0 ; i < buflen && str[i] ; i++)
+		buf[i] = str[i];
+}
+
+
 bool MasterDir_HaveFilename(const char *chk_path)
 {
 	for (unsigned int k = 0 ; k < master_dir.size() ; k++)
